﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using DcnSwSmd.TraceListeners.Data;
using System.Globalization;

namespace DcnSwSmd
{
    /// <summary>
    /// Form to display the details of an Agenda.
    /// </summary>
    public partial class AgendaForm : Form
    {
        #region Functions

        /// <summary>
        /// Contructor of AgendaForm.
        /// </summary>
        public AgendaForm()
        {
            InitializeComponent();
            miFullScreen.Checked = true;
            miFullScreen.Text = "Full Screen";
            miFullScreen.Click += new EventHandler(OnFullScreenClick);
            cmIndvResults.MenuItems.Add(miFullScreen);
            this.ContextMenu = cmIndvResults;
            this.headerControl.lblTimeLimit.Visible = false;
        }

        private void OnFullScreenClick(object sender, EventArgs e)
        {
            if (this.miFullScreen.Checked)
            {
                this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.FixedSingle;
                this.miFullScreen.Checked = false;
            }
            else if (!this.miFullScreen.Checked)
            {
                this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.None;
                this.WindowState = FormWindowState.Maximized;
                this.miFullScreen.Checked = true;
            }
        }
        /// <summary>
        /// Function to load the required data to display.
        /// </summary>
        /// <param name="session"></param>

        private delegate void UpdateSessionData(SMDSessionData session);

        public void LoadAgendaData(SMDSessionData session)
        {
            if (this.InvokeRequired)
            {
                UpdateSessionData obj = new UpdateSessionData(LoadAgendaData);
                this.BeginInvoke(obj, session);
            }
            else
            {
                
                this.headerControl.lblSubject.Text = Properties.Resources.AgendaTxtSubject;
                this.headerControl.lblDate.Text = DateTime.Now.ToShortDateString().ToString(CultureInfo.InvariantCulture);
                this.headerControl.lblTime.Text = DateTime.Now.ToShortTimeString().ToString(CultureInfo.InvariantCulture);
                if (session.SessionData != null)
                {
                    this.subjectLabel.Text = session.SessionData.Subject;
                    this.descriptionLabel.Text = session.SessionData.Description;
                }
            }
        }

        private delegate void LoadDefaultAgendaDataDelegate();
        public void LoadDefaultAgendaData()
        {
            if (this.InvokeRequired)
            {
                LoadDefaultAgendaDataDelegate obj = new LoadDefaultAgendaDataDelegate(LoadDefaultAgendaData);
                this.BeginInvoke(obj);
            }
            else
            {
                this.subjectLabel.Text = Properties.Resources.EmptyAgendaTxt;
                this.descriptionLabel.Text = string.Empty;
            }
        }

        public delegate void UpdateHeaderDateAndTimeDelegate();
        public void UpdateHeaderDateAndTime()
        {
            if (this.InvokeRequired)
            {
                UpdateHeaderDateAndTimeDelegate obj = new UpdateHeaderDateAndTimeDelegate(UpdateHeaderDateAndTime);
                this.BeginInvoke(obj);
            }
            else
            {
                this.headerControl.lblDate.Text = DateTime.Now.ToShortDateString().ToString(CultureInfo.InvariantCulture);
                this.headerControl.lblTime.Text = DateTime.Now.ToShortTimeString().ToString(CultureInfo.InvariantCulture);
            }
        }
        #endregion

        #region Fields

        ContextMenu cmIndvResults = new System.Windows.Forms.ContextMenu();
        MenuItem miFullScreen = new MenuItem();

        #endregion
    }
}
