﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Globalization;

namespace DcnSwSmd
{
    /// <summary>
    /// Form to display the Attendance Registration details.
    /// </summary>
    public partial class AttendanceRegistrationForm : Form
    {
        #region Methods

        /// <summary>
        /// Contructor.
        /// </summary>
        public AttendanceRegistrationForm()
        {
            InitializeComponent();
            miFullScreen.Checked = true;
            miFullScreen.Text = "Full Screen";
            miFullScreen.Click += new EventHandler(OnFullScreenClick);
            cmIndvResults.MenuItems.Add(miFullScreen);
            this.ContextMenu = cmIndvResults;
            this.headerControl.lblTimeLimit.Visible = false;
        }


        /// <summary>
        /// Function to load the required data to display.
        /// </summary>
        /// <param name="participantinfo"></param>
        public delegate void UpdateAttendanceRegistrationData(SMDParticipantData participantInfo);

        public void LoadAttendanceRegistratioData(SMDParticipantData participantinfo)
        {
            if (this.InvokeRequired)
            {
                UpdateAttendanceRegistrationData obj = new UpdateAttendanceRegistrationData(LoadAttendanceRegistratioData);
                this.BeginInvoke(obj, participantinfo);
            }
            else
            {
                if (participantinfo != null)
                {

                    DataTable participantInformation = new DataTable();

                    this.headerControl.lblDate.Text = DateTime.Now.ToShortDateString().ToString(CultureInfo.InvariantCulture);
                    this.headerControl.lblTime.Text = DateTime.Now.ToShortTimeString().ToString(CultureInfo.InvariantCulture);
                    participantInformation = participantinfo.ParticipantApptendanceDetails.Copy();
                    this.headerControl.lblSubject.Text = Properties.Resources.AttendanceRegistrationTxtSubject;
                    this.headerControl.lblTimeLimit.Visible = false;

                    this.lblPresent.Text = Properties.Resources.AttendanceRegistrationTxtPresent + " " + participantinfo.ParticipantApptendanceDetails.Select("Attendance_Status = 'true'").Length.ToString();
                    this.lblAbsent.Text = Properties.Resources.AttendanceRegistrationTxtAbsent + " " + participantinfo.ParticipantApptendanceDetails.Select("Attendance_Status = 'false'").Length.ToString();


                    this.dgvAttendanceRegistration.DataSource = participantInformation;
                    
                    foreach (DataGridViewRow dgvRow in dgvAttendanceRegistration.Rows)
                    {
                        if (dgvRow.Cells["Attendance_Status"].Value.ToString() == "true")
                        {
                            dgvRow.DefaultCellStyle.ForeColor = Color.White;
                        }
                        else if (dgvRow.Cells["Attendance_Status"].Value.ToString() == "false")
                        {
                            dgvRow.DefaultCellStyle.ForeColor = Color.Gray;
                        }
                    }
                    if (this.dgvAttendanceRegistration.ColumnCount > 0)
                    {
                        if (this.dgvAttendanceRegistration.Columns.Contains("Participant_Id"))
                        {
                            this.dgvAttendanceRegistration.Columns["Participant_Id"].Visible = false;
                        }
                        if (this.dgvAttendanceRegistration.Columns.Contains("Attendance_Status"))
                        {
                            this.dgvAttendanceRegistration.Columns["Attendance_Status"].Visible = false;
                        }
                    }
                    this.dgvAttendanceRegistration.Refresh();
                }
            }
        }

        public delegate void UpdateHeaderDateAndTimeDelegate();
        public void UpdateHeaderDateAndTime()
        {
            if (this.InvokeRequired)
            {
                UpdateHeaderDateAndTimeDelegate obj = new UpdateHeaderDateAndTimeDelegate(UpdateHeaderDateAndTime);
                this.BeginInvoke(obj);
            }
            else
            {
                this.headerControl.lblDate.Text = DateTime.Now.ToShortDateString().ToString(CultureInfo.InvariantCulture);
                this.headerControl.lblTime.Text = DateTime.Now.ToShortTimeString().ToString(CultureInfo.InvariantCulture);
            }
        }
        

        private void OnFullScreenClick(object sender, EventArgs e)
        {
            if (this.miFullScreen.Checked)
            {
                this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.FixedSingle;
                this.miFullScreen.Checked = false;
            }
            else if (!this.miFullScreen.Checked)
            {
                this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.None;
                this.WindowState = FormWindowState.Maximized;
                this.miFullScreen.Checked = true;
            }
        }

        private void dgvAttendanceRegistration_SelectionChanged(object sender, EventArgs e)
        {
            if (dgvAttendanceRegistration.SelectedRows.Count > 0)
            {
                dgvAttendanceRegistration.ClearSelection();
            }
        }
        #endregion
        
        #region Fields

        ContextMenu cmIndvResults = new System.Windows.Forms.ContextMenu();
        MenuItem miFullScreen = new MenuItem();

        #endregion

        
    }
}
