using System;
using System.Collections.Generic;
using System.Text;

namespace DcnSwSmd
{
    /// <summary>
    /// This defines the possible types of activities which are send out by the 
    /// server if configured. 
    /// </summary>
    /// <remarks>
    /// This source code is provided for educational purposes only, without 
    /// warranties of any kind.
    /// </remarks>
    public enum EnumActivityType
    {
        // System topic

        /// <summary>
        /// Activity for System started.
        /// </summary>
        SystemStarted,

        /// <summary>
        /// Activity for System stopped.
        /// </summary>
        SystemStopped,

        
        // Meeting topic
        /// <summary>
        /// Activity for Meeting Data Updation
        /// </summary>
        MeetingDataUpdated,

        /// <summary>
        /// Activity for Meeting Start.
        /// </summary>
        MeetingStarted,

        /// <summary>
        /// Activity for Meeting Stop.
        /// </summary>
        MeetingStopped,

        /// <summary>
        /// Activity for Attendance Registration Start.
        /// </summary>
        AttendanceRegistrationStarted,

        /// <summary>
        /// Activity for Attendance Registration Stop.
        /// </summary>
        AttendanceRegistrationStopped,

        /// <summary>
        /// Activity for Participant Addition.
        /// </summary>
        ParticipantAdded,

        /// <summary>
        /// Activity for Participant Removal.
        /// </summary>
        ParticipantRemoved,

        // Session topic
        /// <summary>
        /// Activity for Session Data Updation.
        /// </summary>
        SessionDataUpdated,

        /// <summary>
        /// Activity for Session Start.
        /// </summary>
        SessionStarted,

        /// <summary>
        /// Activity for Session Stop.
        /// </summary>
        SessionStopped,

        /// <summary>
        /// Activity for Session Suspension.
        /// </summary>
        SessionSuspended,

        /// <summary>
        /// Activity for Session Resumption.
        /// </summary>
        SessionResumed,

        // Discussion topic

        /// <summary>
        /// Activity for Discussion Data Updation.
        /// </summary>
        DiscussionDataUpdated,

        /// <summary>
        /// Activity for Request List updation.
        /// </summary>
        RequestListUpdated,

        /// <summary>
        /// Activity for Response List Updation.
        /// </summary>
        ResponseListUpdated,

        /// <summary>
        /// Activity for Active List Updation.
        /// </summary>
        ActiveListUpdated,

        /// <summary>
        /// Activity for Active Response List Updation.
        /// </summary>
        ActiveResponseListUpdated,

        /// <summary>
        /// Activity for Special Equipment List Updation.
        /// </summary>
        SpecialEquipmentListUpdated,

        // Seat topic

        /// <summary>
        /// Activity for Seat Addition.
        /// </summary>
        SeatAdded,

        /// <summary>
        /// Activity for Seat updation.
        /// </summary>
        SeatUpdated,

        /// <summary>
        /// Activity for Seat Removal.
        /// </summary>
        SeatRemoved,

        /// <summary>
        /// Activity for Seat Prioprity Button Activation.
        /// </summary>
        SeatPriorityButtonActivated,

        /// <summary>
        /// Activity for Seat Prioprity De-activation.
        /// </summary>
        SeatPriorityButtonDeactivated,

        // Participant topic
        /// <summary>
        /// Activity for Particiapnt Updation.
        /// </summary>
        ParticipantUpdated,

        // Voting topic

        /// <summary>
        /// Activity for Voting Data updation.
        /// </summary>
        VotingDataUpdated,

        /// <summary>
        /// Activity for Voting Start.
        /// </summary>
        VotingStarted,

        /// <summary>
        /// Activity for Voting Stop.
        /// </summary>
        VotingStopped,

        /// <summary>
        /// Activity for Voting On hold.
        /// </summary>
        VotingOnHold,

        /// <summary>
        /// Activity for Voting Resumption.
        /// </summary>
        VotingResumed,

        /// <summary>
        /// Activity for Voting Selection.
        /// </summary>
        VotingSelected,

        /// <summary>
        /// Activity for Voting Interim Result.
        /// </summary>
        VotingInterimResult,

        //Intrerpretation topic

        /// <summary>
        /// Activity for Interpretation Translation Start.
        /// </summary>
        InterpretationTranslationStarted,

        /// <summary>
        /// Activity for Interpretation Translation Stop.
        /// </summary>
        InterpretationTranslationStopped,

        // service call topic

        /// <summary>
        /// Activity for Service Call Start.
        /// </summary>
        ServiceCallStarted,

        /// <summary>
        /// Activity for Service Call Is Being Served.
        /// </summary>
        ServiceCallIsBeingServiced,

        /// <summary>
        /// Activity for Service Call Handled.
        /// </summary>
        ServiceCallHandled,

        /// <summary>
        /// Activity for Service Call Cancelled.
        /// </summary>
        ServiceCallCanceled,

        // Booth topic

        /// <summary>
        /// Activity for Booth In Use.
        /// </summary>
        BoothInUse,

        /// <summary>
        /// Activity for Booth NOt in Use.
        /// </summary>
        BoothNotInUse,

        // Desk topic

        /// <summary>
        /// Activity for Desk Addition.
        /// </summary>
        DeskAdded,

        /// <summary>
        /// Activity for Desk Updation.
        /// </summary>
        DeskUpdated,

        /// <summary>
        /// Activity for Desk Removal.
        /// </summary>
        DeskRemoved,

        //System Test topic

        /// <summary>
        /// Activity for Channel Test Start.
        /// </summary>
        ChannelTestStarted,

        /// <summary>
        /// Activity for Channel Test stop.
        /// </summary>
        ChannelTestStopped,

        /// <summary>
        /// Activity for Microphone Test Start.
        /// </summary>
        MicrophoneTestStarted,

        /// <summary>
        /// Activity for Microphone Test End.
        /// </summary>
        MicrophoneTestEnded,

        /// <summary>
        /// Activity for Microphone Test Cancellation.
        /// </summary>
        MicrophoneTestCanceled,

        /// <summary>
        /// Activity for Microphone Test Failure.
        /// </summary>
        MicrophoneTestFailed,
    }
}
