﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using DcnSwSmd.TraceListeners.Data;
using System.Globalization;
namespace DcnSwSmd
{
    /// <summary>
    /// Form to display the Group Voting Results.
    /// </summary>
    public partial class GroupResultsForm : Form
    {
        class GroupComparer : IEqualityComparer<GroupResultsData>
        {

            public bool Equals(GroupResultsData grData1, GroupResultsData grData2)
            {
                if(Object.ReferenceEquals(grData1,grData1))
                {
                    return true;
                }
                if(Object.ReferenceEquals(grData1, null) || Object.ReferenceEquals(grData2, null))
                {
                    return false;
                }
                
                return grData1.GroupName == grData2.GroupName;       
            }

            public int GetHashCode(GroupResultsData grpData)
            {
                if(Object.ReferenceEquals(grpData, null))
                {
                    return 0;
                }

                int grpNameHash = grpData.GroupName == null ? 0 : grpData.GroupName.GetHashCode();

                return grpNameHash;
            }
        }
        private class GroupResultsData
        {
            
            private string groupName;
            
            public string GroupName
            {
                set
                {
                    groupName = value;
                }
                get
                {
                    return groupName;
                }
            }

            private int answerId;

            public int AnswerID
            {
                get
                {
                    return answerId;
                }
                set
                {
                    answerId = value;
                }
            }
        }

        /// <summary>
        /// Constructor.
        /// </summary>
        /// <param name="votingIndvResults"></param>
        public GroupResultsForm(SMDVotingData votingIndvResults)
        {
            InitializeComponent();
            miFullScreen.Checked = true;
            miFullScreen.Text = "Full Screen";
            miFullScreen.Click += new EventHandler(OnFullScreenClick);
            cmIndvResults.MenuItems.Add(miFullScreen);
            this.ContextMenu = cmIndvResults;
            ResetUserInterface();
            CreateDtIntermediateTable();
        }
        private void ResetUserInterface()
        {
            this.lblAns1Value.Text = "0";
            this.lblAns2Value.Text = "0";
            this.lblAns3Value.Text = "0";
            this.lblAns4Value.Text = "0";
            this.lblAns5Value.Text = "0";
            this.lblNotVotedValue.Text = "0";
            this.lblTotalValue.Text = "0"; ;
        }

       
        /// <summary>
        /// To recreate the Datatable for Intermediate Voting results.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        public void CreateDtIntermediateTable()
        {
            dtInterimIndvResults = new DataTable();
            dtInterimIndvResults.Columns.Add("Participant_Id", typeof(string));
            dtInterimIndvResults.Columns.Add("Answer_Id", typeof(string));
            dtInterimIndvResults.Columns.Add("ParticipantGroup", typeof(string));
            DataColumn[] columnPrimaryKey = new DataColumn[1];
            columnPrimaryKey[0] = dtInterimIndvResults.Columns["Participant_Id"];
            dtInterimIndvResults.PrimaryKey = columnPrimaryKey;
        }
        /// <summary>
        /// To Update the Header info.
        /// </summary>
        /// <param name="votingGroupResults"></param>
        public void UpdateHeaderData(SMDVotingData votingGroupResults)
        {
            if (this.InvokeRequired)
            {
                UpdateDataSource obj = new UpdateDataSource(UpdateHeaderData);
                this.BeginInvoke(obj, votingGroupResults);
            }
            else
            {
                if (votingGroupResults != null)
                {
                    this.headerControl.lblDate.Text = DateTime.Now.ToShortDateString().ToString(CultureInfo.InvariantCulture);
                    this.headerControl.lblTime.Text = DateTime.Now.ToShortTimeString().ToString(CultureInfo.InvariantCulture);

                    if (votingGroupResults.VotingData != null)
                    {
                        this.headerControl.lblSubject.Text = votingGroupResults.VotingData.Subject;
                        if (votingGroupResults.VotingData.RemainingVotingTime <= -1)
                        {
                            this.headerControl.lblTimeLimit.Text = Properties.Resources.VotingResultsTxtTimeLimit + ": NA";
                        }
                        else
                        {
                            TimeSpan votingTimeLimit;
                            votingTimeLimit = new TimeSpan(votingGroupResults.VotingData.RemainingVotingTime * TimeSpan.TicksPerSecond);
                            this.headerControl.lblTimeLimit.Text = Properties.Resources.VotingResultsTxtTimeLimit + ": " + string.Format(System.Globalization.CultureInfo.InvariantCulture, RemainingvotingTimeFormatString, Math.Abs(votingTimeLimit.Minutes), Math.Abs(votingTimeLimit.Seconds));
                        }
                    }
                    HideLabels();
                    if (votingGroupResults.Answers != null)
                    {
                        foreach (AnswerContainer answer in votingGroupResults.Answers)
                        {
                            #region Parlimentary
                            if (answer.AnswerText == "Yes")
                            {
                                lblAns1.Text = "Yes: ";
                                lblAns1.ForeColor = Color.Green;
                                lblAns1.Visible = true;

                                lblAns1Value.Text = "0";
                                lblAns1Value.ForeColor = Color.Green;
                                lblAns1Value.Visible = true;
                            }
                            else if (answer.AnswerText == "No")
                            {
                                lblAns2.Text = "No: ";
                                lblAns2.ForeColor = Color.Red;
                                lblAns2.Visible = true;

                                lblAns2Value.ForeColor = Color.Red;
                                lblAns2Value.Text = "0";
                                lblAns2Value.Visible = true;
                            }
                            else if (answer.AnswerText == "Abstain")
                            {
                                lblAns3.Text = "Abstain: ";
                                lblAns3.ForeColor = Color.Yellow;
                                lblAns3.Visible = true;

                                lblAns3Value.ForeColor = Color.Yellow;
                                lblAns3Value.Text = "0";
                                lblAns3Value.Visible = true;
                            }
                            else if (answer.AnswerText == "DNPV")
                            {

                                lblAns4.Text = "DNPV: ";
                                lblAns4.ForeColor = Color.Orange;
                                lblAns4.Visible = true;

                                lblAns4Value.Text = "0";
                                lblAns4Value.ForeColor = Color.Orange;
                                lblAns4Value.Visible = true;
                            }
                            #endregion

                            #region AudienceResponse

                            else if (answer.AnswerText == "--")
                            {
                                lblAns1.Text = "--: ";
                                lblAns1.ForeColor = Color.Red;
                                lblAns1.Visible = true;

                                lblAns1Value.Text = "0";
                                lblAns1Value.ForeColor = Color.Red;
                                lblAns1Value.Visible = true;
                            }
                            else if (answer.AnswerText == "-")
                            {
                                lblAns2.Text = "-: ";
                                lblAns2.ForeColor = Color.Orange;
                                lblAns2.Visible = true;

                                lblAns2Value.ForeColor = Color.Orange;
                                lblAns2Value.Text = "0";
                                lblAns2Value.Visible = true;
                            }
                            else if (answer.AnswerText == "0")
                            {
                                lblAns3.Text = "0: ";
                                lblAns3.ForeColor = Color.Yellow;
                                lblAns3.Visible = true;

                                lblAns3Value.ForeColor = Color.Yellow;
                                lblAns3Value.Text = "0";
                                lblAns3Value.Visible = true;
                            }
                            else if (answer.AnswerText == "+")
                            {

                                lblAns4.Text = "+: ";
                                lblAns4.ForeColor = Color.Green;
                                lblAns4.Visible = true;

                                lblAns4Value.Text = "0";
                                lblAns4Value.ForeColor = Color.Green;
                                lblAns4Value.Visible = true;
                            }
                            else if (answer.AnswerText == "++")
                            {
                                lblAns5.Text = "++: ";
                                lblAns5.ForeColor = Color.DarkGreen;
                                lblAns5.Visible = true;

                                lblAns5Value.Text = "0";
                                lblAns5Value.ForeColor = Color.DarkGreen;
                                lblAns5Value.Visible = true;
                            }

                            #endregion

                            #region OpinionPoll

                            else if (answer.AnswerText == "1")
                            {

                                lblAns1.Text = "1: ";
                                lblAns1.ForeColor = Color.Red;
                                lblAns1.Visible = true;

                                lblAns1Value.Text = "0";
                                lblAns1Value.ForeColor = Color.Red;
                                lblAns1Value.Visible = true;
                            }
                            else if (answer.AnswerText == "2")
                            {
                                lblAns2.Text = "2: ";
                                lblAns2.ForeColor = Color.Orange;
                                lblAns2.Visible = true;

                                lblAns2Value.ForeColor = Color.Orange;
                                lblAns2Value.Text = "0";
                                lblAns2Value.Visible = true;
                            }
                            else if (answer.AnswerText == "3")
                            {
                                lblAns3.Text = "3: ";
                                lblAns3.ForeColor = Color.Yellow;
                                lblAns3.Visible = true;

                                lblAns3Value.ForeColor = Color.Yellow;
                                lblAns3Value.Text = "0";
                                lblAns3Value.Visible = true;
                            }
                            else if (answer.AnswerText == "4")
                            {
                                lblAns4.Text = "4: ";
                                lblAns4.ForeColor = Color.Green;
                                lblAns4.Visible = true;

                                lblAns4Value.Text = "0";
                                lblAns4Value.ForeColor = Color.Green;
                                lblAns4Value.Visible = true;
                            }
                            else if (answer.AnswerText == "5")
                            {
                                lblAns5.Text = "5: ";
                                lblAns5.ForeColor = Color.DarkGreen;
                                lblAns5.Visible = true;


                                lblAns5Value.Text = "0";
                                lblAns5Value.ForeColor = Color.DarkGreen;
                                lblAns5Value.Visible = true;
                            }
                            #endregion
                            this.lblTotal.Visible = true;
                            this.lblTotalValue.Visible = true;
                            this.lblNotVoted.Visible = true;
                            this.lblNotVotedValue.Visible = true;
                        }
                    }
                    ResetUserInterface();
                }
            }
        }
        private void OnFullScreenClick(object sender, EventArgs e)
        {
            if (this.miFullScreen.Checked)
            {
                this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.FixedSingle;
                this.miFullScreen.Checked = false;
            }
            else if (!this.miFullScreen.Checked)
            {
                this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.None;
                this.WindowState = FormWindowState.Maximized;
                this.miFullScreen.Checked = true;
            }

        }

        public delegate void ClearPreviousResultsDelegate();
        public void ClearPreviousResults()
        {
            if (this.InvokeRequired)
            {
                ClearPreviousResultsDelegate obj = new ClearPreviousResultsDelegate(ClearPreviousResults);
                this.BeginInvoke(obj);
            }
            else
            {
                if (dtGroupData != null)
                {
                    dtGroupData.Clear();
                }
                if (dtGroupResults != null)
                {
                    dtGroupResults.Clear();
                    dtGroupResults.Columns.Clear();
                }
                if (dtInterimIndvResults != null)
                {
                    dtInterimIndvResults.Clear();
                }
                if (answerList != null)
                {
                    answerList.Clear();
                }
            }
        }

        public delegate void LoadVotingdataDelegate(SMDVotingData votingIndvResults, bool votingStopped);
        /// <summary>
        /// Function to load the required data to display.
        /// </summary>
        /// <param name="votingGroupResults"></param>
        
        public void LoadVotingData(SMDVotingData votingGroupResults, bool votingStopped)
        {
            if (this.InvokeRequired)
            {
                LoadVotingdataDelegate obj = new LoadVotingdataDelegate(LoadVotingData);
                this.BeginInvoke(obj, votingGroupResults, votingStopped);
            }
            else
            {
                try
                {
                    TimeSpan votingTimeLimit;
                    if (votingGroupResults != null)
                    {

                        this.headerControl.lblDate.Text = DateTime.Now.ToShortDateString().ToString(CultureInfo.InvariantCulture);
                        this.headerControl.lblTime.Text = DateTime.Now.ToShortTimeString().ToString(CultureInfo.InvariantCulture);
                        if (votingGroupResults.VotingData != null)
                        {
                            this.headerControl.lblSubject.Text = votingGroupResults.VotingData.Subject;
                                
                            if (votingGroupResults.VotingData.RemainingVotingTime <= -1)
                            {
                                this.headerControl.lblTimeLimit.Text = Properties.Resources.VotingResultsTxtTimeLimit + ": NA";
                            }
                            else
                            {
                                votingTimeLimit = new TimeSpan(votingGroupResults.VotingData.RemainingVotingTime * TimeSpan.TicksPerSecond);
                                this.headerControl.lblTimeLimit.Text = Properties.Resources.VotingResultsTxtTimeLimit + ": " + string.Format(System.Globalization.CultureInfo.InvariantCulture, RemainingvotingTimeFormatString, Math.Abs(votingTimeLimit.Minutes), Math.Abs(votingTimeLimit.Seconds));
                            }
                        }
                    }

                    // Add columns to this DataTable only if voting is stopped, as it is used after voting is stopped.
                    if (votingStopped)
                    {
                        if (!dtGroupData.Columns.Contains("GroupName"))
                        {
                            dtGroupData.Columns.Add("GroupName");
                        }
                        if (!dtGroupData.Columns.Contains("AnswerId"))
                        {
                            dtGroupData.Columns.Add("AnswerId");
                        }
                    }

                    #region AnswerTypes
                    if (!dtGroupResults.Columns.Contains("Group"))
                    {
                        dtGroupResults.Columns.Add("Group");
                    }


                    int ansIndex = 0;
                    if (votingGroupResults.Answers != null)
                    {
                        foreach (AnswerContainer answer in votingGroupResults.Answers)
                        {
                            ansIndex = votingGroupResults.Answers.FindIndex(delegate(AnswerContainer ansCont) { return ansCont.Id == answer.Id; });
                            string ansText = votingGroupResults.Answers[ansIndex].AnswerText;
                            if (!dtGroupResults.Columns.Contains(ansText))
                            {
                                dtGroupResults.Columns.Add(ansText);
                            }
                            if (!answerList.ContainsKey(ansText))
                            {
                                answerList.Add(ansText, 0);
                            }
                        }
                    }


                    #endregion

                    #region requiredData
                    if (votingStopped)
                    {
                        if (votingGroupResults.Answers != null)
                        {
                            foreach (VotingIndividualResultContainer indvReults in votingGroupResults.IndividualResults)
                            {
                                ansIndex = votingGroupResults.Answers.FindIndex(delegate(AnswerContainer ansCont) { return ansCont.Id == indvReults.AnswerId; });
                                if (ansIndex > -1)
                                {
                                    dtGroupData.Rows.Add(indvReults.Participant.Group.Name, indvReults.AnswerId);
                                }
                            }
                        }
                    }
                    #endregion
                    HideLabels();
                    UpdateHeaderData(votingGroupResults);

                    if (votingStopped)
                    {
                        UpdateDataSourceAfterVotingStops(votingGroupResults);
                        UpdateTotalResults(votingGroupResults, true);
                    }
                    else
                    {
                        UpdateDataSourceBeforeVotingStops(votingGroupResults);
                        UpdateTotalResults(votingGroupResults, false);

                    }
                    if (!votingStopped && DtIntermediateIndvResults.Rows.Count <= 0)
                    {
                        ResetUserInterface();
                    }
                    UpdateDataGridViewColumnBackground();

                }
                catch (Exception)
                {

                }
            }
        }
        private delegate void UpdateDataSource(SMDVotingData votingGroupResults);
        private void UpdateDataSourceBeforeVotingStops(SMDVotingData votingGroupResults)
        {

            if (this.InvokeRequired)
            {
                UpdateDataSource obj = new UpdateDataSource(UpdateDataSourceBeforeVotingStops);
                this.BeginInvoke(obj, votingGroupResults);
            }
            else
            {
                try
                {
                    lock (this)
                    {
                        DataTable dtGroups;
                        DataRow newRow;
                        DataRow[] result;
                        int ansIndex = 0;
                        List<string> answers;
                        dtGroupResults.Rows.Clear();
                        dtGroups = dtInterimIndvResults.DefaultView.ToTable(true, "ParticipantGroup");
                        string answerText = String.Empty;
                        int count = 0;
                        if (!dtGroupResults.Columns.Contains("Group"))
                        {
                            dtGroupResults.Columns.Add("Group");
                        }
                        foreach (DataRow dr in dtGroups.Rows)
                        {
                            newRow = dtGroupResults.NewRow();
                            result = dtInterimIndvResults.Select("ParticipantGroup = " + "'" + dr["ParticipantGroup"] + "'");
                            for (int i = 0; i < result.Length; i++)
                            {
                                int ansId = Int32.Parse(result[i].ItemArray[1].ToString());
                                newRow["Group"] = dr["ParticipantGroup"];

                                // To findout in which column we have to add the count.
                                ansIndex = votingGroupResults.Answers.FindIndex(delegate(AnswerContainer ansCont) { return ansCont.Id == ansId; });
                                if (ansIndex > -1)
                                {
                                    answerText = votingGroupResults.Answers[ansIndex].AnswerText;

                                    count = Int32.Parse(dtInterimIndvResults.Compute("Count(Answer_Id)", "ParticipantGroup = " + "'" + dr["ParticipantGroup"] + "' AND Answer_Id = " + ansId).ToString());
                                    if (dtGroupResults.Columns.Contains(answerText) && answerList[answerText] != count)
                                    {
                                        newRow[answerText] = count;
                                        answerList[answerText] = count;
                                    }
                                }
                            }
                            foreach (KeyValuePair<string, int> ans in answerList)
                            {
                                if (ans.Value == 0)
                                {
                                    newRow[ans.Key] = 0;
                                }
                            }

                            dtGroupResults.Rows.Add(newRow);
                            dtGroupResults.AcceptChanges();
                            // resetting the answer count to zero in the answerlist 
                            answers = new List<string>(answerList.Keys);
                            foreach (string ans in answers)
                            {
                                answerList[ans] = 0;
                            }
                        }
                        dgvGroupVotingResults.DataSource = dtGroupResults;
                    }
                }
                catch (Exception)
                {

                }
            
            }
            
        }
        private void UpdateDataSourceAfterVotingStops(SMDVotingData votingGroupResults)
        {
            if (this.InvokeRequired)
            {
                UpdateDataSource obj = new UpdateDataSource(UpdateDataSourceAfterVotingStops);
                this.BeginInvoke(obj, votingGroupResults);
            }
            else
            {
                try
                {
                    lock (this)
                    {
                        DataTable dtGroups;
                        DataRow newRow;
                        DataRow[] result;
                        List<string> answers;
                        int ansIndex = 0;
                        dtGroups = dtGroupData.DefaultView.ToTable(true, "GroupName");
                        string answerText = String.Empty;
                        int count = 0;
                        if (dtGroupResults != null)
                        {
                            dtGroupResults.Clear();
                        }
                        foreach (DataRow dr in dtGroups.Rows)
                        {
                            newRow = dtGroupResults.NewRow();
                            result = dtGroupData.Select("GroupName = " + "'" + dr["GroupName"] + "'");
                            for (int i = 0; i < result.Length; i++)
                            {
                                int ansId = Int32.Parse(result[i].ItemArray[1].ToString());
                                newRow["Group"] = dr["GroupName"];

                                // To findout in which column we have to add the count.
                                ansIndex = votingGroupResults.Answers.FindIndex(delegate(AnswerContainer ansCont) { return ansCont.Id == ansId; });
                                if (ansIndex > -1)
                                {
                                    answerText = votingGroupResults.Answers[ansIndex].AnswerText;

                                    count = Int32.Parse(dtGroupData.Compute("Count(AnswerId)", "GroupName = " + "'" + dr["GroupName"] + "' AND AnswerId = " + ansId).ToString());
                                    if (dtGroupResults.Columns.Contains(answerText) && answerList[answerText] != count)
                                    {
                                        newRow[answerText] = count;
                                        answerList[answerText] = count;
                                    }
                                }
                            }
                            foreach (KeyValuePair<string, int> ans in answerList)
                            {
                                if (ans.Value == 0)
                                {
                                    newRow[ans.Key] = 0;
                                }
                            }

                            dtGroupResults.Rows.Add(newRow);

                            // resetting the answer count to zero in the answerlist 
                            answers = new List<string>(answerList.Keys);
                            foreach (string ans in answers)
                            {
                                answerList[ans] = 0;
                            }
                        }
                        dgvGroupVotingResults.DataSource = dtGroupResults;
                    }

                }
                catch (Exception)
                {

                }
            }
        }


        private delegate void UpdateVotingResults(SMDVotingData votingGroupResults, bool votingStopped);
        /*private void UpdateTotalResults(SMDVotingData votingGroupResults, bool votingStopped)
        {
            if (this.InvokeRequired)
            {
                UpdateVotingResults obj = new UpdateVotingResults(UpdateTotalResults);
                this.BeginInvoke(obj, votingGroupResults, votingStopped);
            }
            else
            {
                lock (this)
                {
                    if (votingGroupResults.TotalResults != null)
                    {
                        int index = 0;
                        string ans = String.Empty;
                        int total = 0;
                        for (int iCount = 0; iCount < votingGroupResults.TotalResults.VotingAnswerResults.Count; iCount++)
                        {
                            index = votingGroupResults.Answers.FindIndex(delegate(AnswerContainer ansCont) { return ansCont.Id == votingGroupResults.TotalResults.VotingAnswerResults[iCount].AnswerId; });
                            if (votingGroupResults.Answers[index].AnswerText == "Yes")
                            {
                                this.lblAns1.Visible = true;
                                this.lblAns1Value.Visible = true;
                                this.lblAns1Value.Text = votingGroupResults.TotalResults.VotingAnswerResults[index].NumberOfCasts.ToString(CultureInfo.InvariantCulture);
                            }
                            else if (votingGroupResults.Answers[index].AnswerText == "No")
                            {
                                this.lblAns2.Visible = true;
                                this.lblAns2Value.Visible = true;
                                this.lblAns2Value.Text = votingGroupResults.TotalResults.VotingAnswerResults[index].NumberOfCasts.ToString(CultureInfo.InvariantCulture);
                            }
                            else if (votingGroupResults.Answers[index].AnswerText == "Abstain")
                            {
                                this.lblAns3.Visible = true;
                                this.lblAns3Value.Visible = true;
                                this.lblAns3Value.Text = votingGroupResults.TotalResults.VotingAnswerResults[index].NumberOfCasts.ToString(CultureInfo.InvariantCulture);
                            }
                            else if (votingGroupResults.Answers[index].AnswerText == "DNPV")
                            {
                                this.lblAns4.Visible = true;
                                this.lblAns4Value.Visible = true;
                                this.lblAns4Value.Text = votingGroupResults.TotalResults.VotingAnswerResults[index].NumberOfCasts.ToString(CultureInfo.InvariantCulture);
                            }
                            total += votingGroupResults.TotalResults.VotingAnswerResults[index].NumberOfCasts;
                        }
                        this.lblTotal.Visible = true;
                        this.lblTotalValue.Visible = true;
                        this.lblNotVoted.Visible = true;
                        this.lblNotVotedValue.Visible = true;

                        this.lblTotalValue.Text = total.ToString(CultureInfo.InvariantCulture);
                        this.lblNotVotedValue.Text = (votingGroupResults.TotalResults.MaximumMajority - total).ToString(CultureInfo.InvariantCulture);
                    }
                }
            }
        }*/

        public void UpdateTotalResults(SMDVotingData votingGroupResults, bool votingStopped)
        {
            if (this.InvokeRequired)
            {
                UpdateVotingResults obj = new UpdateVotingResults(UpdateTotalResults);
                this.BeginInvoke(obj, votingGroupResults, votingStopped);
            }
            else
            {
                try
                {
                    lock (this)
                    {
                        if (votingGroupResults.TotalResults != null)
                        {
                            int index = 0;
                            string ans = String.Empty;
                            int total = 0;
                            for (int iCount = 0; iCount < votingGroupResults.TotalResults.VotingAnswerResults.Count; iCount++)
                            {


                                index = votingGroupResults.Answers.FindIndex(delegate(AnswerContainer ansCont) { return ansCont.Id == votingGroupResults.TotalResults.VotingAnswerResults[iCount].AnswerId; });
                                #region Parlimentary
                                if (votingGroupResults.Answers[index].AnswerText == "Yes")
                                {
                                    this.lblAns1.Visible = true;
                                    this.lblAns1.Text = votingGroupResults.Answers[index].AnswerText + ": ";
                                    this.lblAns1Value.Image = Properties.Resources.Green;
                                    this.lblAns1Value.ImageAlign = ContentAlignment.MiddleCenter;
                                    this.lblAns1Value.Visible = true;
                                    this.lblAns1Value.Text = votingGroupResults.TotalResults.VotingAnswerResults[index].NumberOfCasts.ToString(CultureInfo.InvariantCulture);
                                }
                                else if (votingGroupResults.Answers[index].AnswerText == "No")
                                {
                                    this.lblAns2.Text = votingGroupResults.Answers[index].AnswerText + ": ";
                                    this.lblAns2.Visible = true;
                                    this.lblAns2Value.Image = Properties.Resources.Red;
                                    this.lblAns2Value.ImageAlign = ContentAlignment.MiddleCenter;
                                    this.lblAns2Value.Visible = true;
                                    this.lblAns2Value.Text = votingGroupResults.TotalResults.VotingAnswerResults[index].NumberOfCasts.ToString(CultureInfo.InvariantCulture);
                                }
                                else if (votingGroupResults.Answers[index].AnswerText == "Abstain")
                                {
                                    this.lblAns3.Text = votingGroupResults.Answers[index].AnswerText + ": ";
                                    this.lblAns3.Visible = true;
                                    this.lblAns3Value.Image = Properties.Resources.Yellow;
                                    this.lblAns3Value.ImageAlign = ContentAlignment.MiddleCenter;
                                    this.lblAns3Value.Visible = true;
                                    this.lblAns3Value.Text = votingGroupResults.TotalResults.VotingAnswerResults[index].NumberOfCasts.ToString(CultureInfo.InvariantCulture);
                                }
                                else if (votingGroupResults.Answers[index].AnswerText == "DNPV")
                                {
                                    this.lblAns4.Text = votingGroupResults.Answers[index].AnswerText + ": ";
                                    this.lblAns4.Visible = true;
                                    this.lblAns4Value.Image = Properties.Resources.Orange;
                                    this.lblAns4Value.ImageAlign = ContentAlignment.MiddleCenter;
                                    this.lblAns4Value.Visible = true;
                                    this.lblAns4Value.Text = votingGroupResults.TotalResults.VotingAnswerResults[index].NumberOfCasts.ToString(CultureInfo.InvariantCulture);
                                }
                                #endregion
                                #region AudienceResponse
                                else if (votingGroupResults.Answers[index].AnswerText == "--")
                                {
                                    this.lblAns1.Text = votingGroupResults.Answers[index].AnswerText + ": ";
                                    this.lblAns1.Visible = true;
                                    this.lblAns1Value.Image = Properties.Resources.Red;
                                    this.lblAns1Value.ImageAlign = ContentAlignment.MiddleCenter;
                                    this.lblAns1Value.Visible = true;
                                    this.lblAns1Value.Text = votingGroupResults.TotalResults.VotingAnswerResults[index].NumberOfCasts.ToString(CultureInfo.InvariantCulture);
                                }
                                else if (votingGroupResults.Answers[index].AnswerText == "-")
                                {
                                    this.lblAns2.Text = votingGroupResults.Answers[index].AnswerText + ": ";
                                    this.lblAns2.Visible = true;
                                    this.lblAns2Value.Image = Properties.Resources.Orange;
                                    this.lblAns2Value.ImageAlign = ContentAlignment.MiddleCenter;
                                    this.lblAns2Value.Visible = true;
                                    this.lblAns2Value.Text = votingGroupResults.TotalResults.VotingAnswerResults[index].NumberOfCasts.ToString(CultureInfo.InvariantCulture);
                                }
                                else if (votingGroupResults.Answers[index].AnswerText == "0")
                                {
                                    this.lblAns3.Text = votingGroupResults.Answers[index].AnswerText + ": ";
                                    this.lblAns3.Visible = true;
                                    this.lblAns3Value.Image = Properties.Resources.Yellow;
                                    this.lblAns3Value.ImageAlign = ContentAlignment.MiddleCenter;
                                    this.lblAns3Value.Visible = true;
                                    this.lblAns3Value.Text = votingGroupResults.TotalResults.VotingAnswerResults[index].NumberOfCasts.ToString(CultureInfo.InvariantCulture);
                                }
                                else if (votingGroupResults.Answers[index].AnswerText == "+")
                                {
                                    this.lblAns4.Text = votingGroupResults.Answers[index].AnswerText + ": ";
                                    this.lblAns4.Visible = true;
                                    this.lblAns4Value.Image = Properties.Resources.Green;
                                    this.lblAns4Value.ImageAlign = ContentAlignment.MiddleCenter;
                                    this.lblAns4Value.Visible = true;
                                    this.lblAns4Value.Text = votingGroupResults.TotalResults.VotingAnswerResults[index].NumberOfCasts.ToString(CultureInfo.InvariantCulture);
                                }
                                else if (votingGroupResults.Answers[index].AnswerText == "++")
                                {
                                    this.lblAns5.Text = votingGroupResults.Answers[index].AnswerText + ": ";
                                    this.lblAns5.Visible = true;
                                    this.lblAns5Value.Image = Properties.Resources.DarkGreen;
                                    this.lblAns5Value.ImageAlign = ContentAlignment.MiddleCenter;
                                    this.lblAns5Value.Visible = true;
                                    this.lblAns5Value.Text = votingGroupResults.TotalResults.VotingAnswerResults[index].NumberOfCasts.ToString(CultureInfo.InvariantCulture);
                                }
                                #endregion
                                #region OpinionPoll
                                else if (votingGroupResults.Answers[index].AnswerText == "1")
                                {
                                    this.lblAns1.Text = votingGroupResults.Answers[index].AnswerText + ": ";
                                    this.lblAns1.Visible = true;
                                    this.lblAns1Value.Image = Properties.Resources.Red;
                                    this.lblAns1Value.ImageAlign = ContentAlignment.MiddleCenter;
                                    this.lblAns1Value.Visible = true;
                                    this.lblAns1Value.Text = votingGroupResults.TotalResults.VotingAnswerResults[index].NumberOfCasts.ToString(CultureInfo.InvariantCulture);
                                }
                                else if (votingGroupResults.Answers[index].AnswerText == "2")
                                {
                                    this.lblAns2.Text = votingGroupResults.Answers[index].AnswerText + ": ";
                                    this.lblAns2.Visible = true;
                                    this.lblAns2Value.Image = Properties.Resources.Orange;
                                    this.lblAns2Value.ImageAlign = ContentAlignment.MiddleCenter;
                                    this.lblAns2Value.Visible = true;
                                    this.lblAns2Value.Text = votingGroupResults.TotalResults.VotingAnswerResults[index].NumberOfCasts.ToString(CultureInfo.InvariantCulture);
                                }
                                else if (votingGroupResults.Answers[index].AnswerText == "3")
                                {
                                    this.lblAns3.Text = votingGroupResults.Answers[index].AnswerText + ": ";
                                    this.lblAns3.Visible = true;
                                    this.lblAns3Value.Image = Properties.Resources.Yellow;
                                    this.lblAns3Value.ImageAlign = ContentAlignment.MiddleCenter;
                                    this.lblAns3Value.Visible = true;
                                    this.lblAns3Value.Text = votingGroupResults.TotalResults.VotingAnswerResults[index].NumberOfCasts.ToString(CultureInfo.InvariantCulture);
                                }
                                else if (votingGroupResults.Answers[index].AnswerText == "4")
                                {
                                    this.lblAns4.Text = votingGroupResults.Answers[index].AnswerText + ": ";
                                    this.lblAns4.Visible = true;
                                    this.lblAns4Value.Image = Properties.Resources.Green;
                                    this.lblAns4Value.ImageAlign = ContentAlignment.MiddleCenter;
                                    this.lblAns4Value.Visible = true;
                                    this.lblAns4Value.Text = votingGroupResults.TotalResults.VotingAnswerResults[index].NumberOfCasts.ToString(CultureInfo.InvariantCulture);
                                }
                                else if (votingGroupResults.Answers[index].AnswerText == "5")
                                {
                                    this.lblAns5.Text = votingGroupResults.Answers[index].AnswerText + ": ";
                                    this.lblAns5.Visible = true;
                                    this.lblAns5Value.Image = Properties.Resources.DarkGreen;
                                    this.lblAns5Value.ImageAlign = ContentAlignment.MiddleCenter;
                                    this.lblAns5Value.Visible = true;
                                    this.lblAns5Value.Text = votingGroupResults.TotalResults.VotingAnswerResults[index].NumberOfCasts.ToString(CultureInfo.InvariantCulture);
                                }

                                #endregion

                                total += votingGroupResults.TotalResults.VotingAnswerResults[index].NumberOfCasts;
                            }
                            this.lblTotal.Visible = true;
                            this.lblTotalValue.Visible = true;
                            this.lblNotVoted.Visible = true;
                            this.lblNotVotedValue.Visible = true;

                            this.lblTotalValue.Text = total.ToString(CultureInfo.InvariantCulture);
                            this.lblNotVotedValue.Text = (votingGroupResults.TotalResults.MaximumMajority - total).ToString(CultureInfo.InvariantCulture);
                        }
                    }
                }
                catch (Exception)
                {
                }
            }
        }
        private delegate void UpdateDataGridViewColumnBackgroundDelegate();
        private void UpdateDataGridViewColumnBackground()
        {
            try
            {
                if (this.InvokeRequired)
                {
                    UpdateDataGridViewColumnBackgroundDelegate obj = new UpdateDataGridViewColumnBackgroundDelegate(UpdateDataGridViewColumnBackground);
                    this.BeginInvoke(obj);
                }
                else
                {
                    foreach (DataGridViewRow row in dgvGroupVotingResults.Rows)
                    {
                        #region Parlimentary
                        if (dgvGroupVotingResults.Columns.Contains("Yes"))
                        {
                            row.Cells["Yes"].Style.BackColor = Color.Green;
                            row.Cells["Yes"].Style.ForeColor = Color.Black;
                        }
                        if (dgvGroupVotingResults.Columns.Contains("No"))
                        {
                            row.Cells["No"].Style.BackColor = Color.Red;
                            row.Cells["No"].Style.ForeColor = Color.Black;
                        }
                        if (dgvGroupVotingResults.Columns.Contains("Abstain"))
                        {
                            row.Cells["Abstain"].Style.BackColor = Color.Yellow;
                            row.Cells["Abstain"].Style.ForeColor = Color.Black;
                        }
                        if (dgvGroupVotingResults.Columns.Contains("DNPV"))
                        {
                            row.Cells["DNPV"].Style.BackColor = Color.Orange;
                            row.Cells["DNPV"].Style.ForeColor = Color.Black;
                        }
                        #endregion
                        #region AudienceResponse
                        if (dgvGroupVotingResults.Columns.Contains("--"))
                        {
                            row.Cells["--"].Style.BackColor = Color.Red;
                            row.Cells["--"].Style.ForeColor = Color.Black;
                        }
                        if (dgvGroupVotingResults.Columns.Contains("-"))
                        {
                            row.Cells["-"].Style.BackColor = Color.Orange;
                            row.Cells["-"].Style.ForeColor = Color.Black;
                        }
                        if (dgvGroupVotingResults.Columns.Contains("0"))
                        {
                            row.Cells["0"].Style.BackColor = Color.Yellow;
                            row.Cells["0"].Style.ForeColor = Color.Black;
                        }
                        if (dgvGroupVotingResults.Columns.Contains("+"))
                        {
                            row.Cells["+"].Style.BackColor = Color.Green;
                            row.Cells["+"].Style.ForeColor = Color.Black;
                        }
                        if (dgvGroupVotingResults.Columns.Contains("++"))
                        {
                            row.Cells["++"].Style.BackColor = Color.DarkGreen;
                            row.Cells["++"].Style.ForeColor = Color.Black;
                        }
                        #endregion
                        #region Opinion Poll
                        if (dgvGroupVotingResults.Columns.Contains("1"))
                        {
                            row.Cells["1"].Style.BackColor = Color.Red;
                            row.Cells["1"].Style.ForeColor = Color.Black;
                        }
                        if (dgvGroupVotingResults.Columns.Contains("2"))
                        {
                            row.Cells["2"].Style.BackColor = Color.Orange;
                            row.Cells["2"].Style.ForeColor = Color.Black;
                        }
                        if (dgvGroupVotingResults.Columns.Contains("3"))
                        {
                            row.Cells["3"].Style.BackColor = Color.Yellow;
                            row.Cells["3"].Style.ForeColor = Color.Black;
                        }
                        if (dgvGroupVotingResults.Columns.Contains("4"))
                        {
                            row.Cells["4"].Style.BackColor = Color.Green;
                            row.Cells["4"].Style.ForeColor = Color.Black;
                        }
                        if (dgvGroupVotingResults.Columns.Contains("5"))
                        {
                            row.Cells["5"].Style.BackColor = Color.DarkGreen;
                            row.Cells["5"].Style.ForeColor = Color.Black;
                        }
                        #endregion

                    }
                }
            }
            catch (Exception)
            {

            }
        }
        private void HideLabels()
        {
            this.lblAns1.Visible = false;
            this.lblAns1Value.Visible = false;
            this.lblAns2.Visible = false;
            this.lblAns2Value.Visible = false;
            this.lblAns3.Visible = false;
            this.lblAns3Value.Visible = false;
            this.lblAns4.Visible = false;
            this.lblAns4Value.Visible = false;
            this.lblAns5.Visible = false;
            this.lblAns5Value.Visible = false;
            
           
            this.lblTotal.Visible = false;
            this.lblTotalValue.Visible = false;
            this.lblNotVoted.Visible = false;
            this.lblNotVotedValue.Visible = false;
            
        }

        public delegate void AddorUpdateInterimResultsDelegate(SMDVotingData votingGroupResults);

        public void AddOrUpdateInterimResults(SMDVotingData votingGroupResults)
        {
            if (this.InvokeRequired)
            {
                AddorUpdateInterimResultsDelegate obj = new AddorUpdateInterimResultsDelegate(AddOrUpdateInterimResults);
                this.BeginInvoke(obj, votingGroupResults);
            }
            else
            {
                try
                {
                    if (votingGroupResults != null)
                    {
                        string participantId = String.Empty;
                        string participantName = String.Empty;

                        if (votingGroupResults.IndividualResults != null && votingGroupResults.IndividualResults.Count > 0)
                        {

                            #region requiredData
                            int ansIndex = 0;
                            foreach (VotingIndividualResultContainer indvReults in votingGroupResults.IndividualResults)
                            {
                                participantId = indvReults.Participant.Id.ToString(CultureInfo.InvariantCulture);
                                participantName = indvReults.Participant.ParticipantData.FirstName + indvReults.Participant.ParticipantData.MiddleName + indvReults.Participant.ParticipantData.LastName;
                                ansIndex = votingGroupResults.Answers.FindIndex(delegate(AnswerContainer ansCont) { return ansCont.Id == indvReults.AnswerId; });
                                if (ansIndex > -1)
                                {
                                    if ((dtInterimIndvResults.Rows.Count > 0))
                                    {
                                        string filterExpression = "Participant_Id = '" + participantId.ToString() + "'";
                                        if (dtInterimIndvResults.Select(filterExpression).Length == 0)
                                        {
                                            DtIntermediateIndvResults.Rows.Add(indvReults.Participant.Id, indvReults.AnswerId, indvReults.Participant.Group.Name);
                                        }
                                        else
                                        {
                                            DataRow dr = DtIntermediateIndvResults.Rows.Find(participantId);
                                            DtIntermediateIndvResults.Rows.Remove(dr);
                                            DtIntermediateIndvResults.Rows.Add(indvReults.Participant.Id, indvReults.AnswerId, indvReults.Participant.Group.Name);
                                        }
                                    }
                                    else
                                    {
                                        DtIntermediateIndvResults.Rows.Add(indvReults.Participant.Id, indvReults.AnswerId, indvReults.Participant.Group.Name);
                                    }
                                }
                                else if (ansIndex == -1)
                                {
                                    //notVotedCount++;
                                }
                            }
                            #endregion
                        }
                        UpdateDataSourceBeforeVotingStops(votingGroupResults);
                        UpdateTotalResults(votingGroupResults, false);
                        UpdateDataGridViewColumnBackground();


                    }
                }
                catch (Exception)
                {

                }
            }
        }

        private delegate void UpdateCaptionOfControls(Control c, string str);

        private void UpdateCaptionOfControl(Control cntr, string value)
        {
            if (!IsDisposed)
            {
                if (cntr.InvokeRequired)
                {
                    BeginInvoke(new UpdateCaptionOfControls(UpdateCaptionOfControl), cntr, value);
                }
                else
                {
                    cntr.Text = value;
                }
            }
        }
        
        public delegate void UpdateVotingRemainingTimeDelegate(SMDVotingData votingResults);
        public void UpdateRemainingVotingTime(SMDVotingData votingResults)
        {
            if (this.InvokeRequired)
            {
                UpdateVotingRemainingTimeDelegate obj = new UpdateVotingRemainingTimeDelegate(UpdateRemainingVotingTime);
                this.BeginInvoke(obj, votingResults);
            }
            else
            {
                if (votingResults != null)
                {
                    if (votingResults.VotingData != null)
                    {
                        TimeSpan votingTimeLimit;


                        if (votingResults.VotingData.RemainingVotingTime <= -1)
                        {
                            this.headerControl.lblTimeLimit.Text = Properties.Resources.VotingResultsTxtTimeLimit + ": NA";
                        }
                        else
                        {
                            votingTimeLimit = new TimeSpan(votingResults.VotingData.RemainingVotingTime * TimeSpan.TicksPerSecond);
                            this.headerControl.lblTimeLimit.Text = Properties.Resources.VotingResultsTxtTimeLimit + ": " + string.Format(System.Globalization.CultureInfo.InvariantCulture, RemainingvotingTimeFormatString, Math.Abs(votingTimeLimit.Minutes), Math.Abs(votingTimeLimit.Seconds));
                        }
                    }
                }
            }
        }

        public delegate void UpdateHeaderDateAndTimeDelegate();
        public void UpdateHeaderDateAndTime()
        {
            if (this.InvokeRequired)
            {
                UpdateHeaderDateAndTimeDelegate obj = new UpdateHeaderDateAndTimeDelegate(UpdateHeaderDateAndTime);
                this.BeginInvoke(obj);
            }
            else
            {
                this.headerControl.lblDate.Text = DateTime.Now.ToShortDateString().ToString(CultureInfo.InvariantCulture);
                this.headerControl.lblTime.Text = DateTime.Now.ToShortTimeString().ToString(CultureInfo.InvariantCulture);
            }
        }
        private void dgvGroupVotingResults_SelectionChanged(object sender, EventArgs e)
        {
            if (dgvGroupVotingResults.SelectedRows.Count > 0)
            {
                dgvGroupVotingResults.ClearSelection();
            }
        }
        #region Properties

        public DataTable DtIntermediateIndvResults
        {
            get
            {
                return dtInterimIndvResults;
            }
            set
            {
                dtInterimIndvResults = value;
            }
        }
        #endregion

        #region Fields

        const string RemainingvotingTimeFormatString = "{0:00}:{1:00}";

        Dictionary<string, int> answerList = new Dictionary<string, int>();
        /// <summary>
        /// A datatable to hold the interim individual Voting results.
        /// </summary>
        DataTable dtInterimIndvResults;

        ContextMenu cmIndvResults = new System.Windows.Forms.ContextMenu();
        MenuItem miFullScreen = new MenuItem();
        List<GroupResultsData> groupDataList = new List<GroupResultsData>();
        /// <summary>
        /// Datasource of the DatagridView.
        /// </summary>
        DataTable dtGroupResults = new DataTable();
        
        /// <summary>
        /// DataTable to hold the voting results after voting is stopped.
        /// </summary>
        DataTable dtGroupData = new DataTable();
        #endregion

       
    }
}
