﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using DcnSwSmd.TraceListeners.Data;
using System.Globalization;

namespace DcnSwSmd
{
    /// <summary>
    /// Form to display the Group speech Time.
    /// </summary>
    public partial class GroupSpeechTimerForm : Form
    {
        /// <summary>
        /// Constructor.
        /// </summary>
        public GroupSpeechTimerForm()
        {
            InitializeComponent();
            miFullScreen.Checked = true;
            miFullScreen.Text = "Full Screen";
            miFullScreen.Click += new EventHandler(OnFullScreenClick);
            cmIndvResults.MenuItems.Add(miFullScreen);
            this.ContextMenu = cmIndvResults;
            this.headerControl.lblTimeLimit.Visible = false;
        }

        #region Methods

        private void dgvGroupSpeechTimers_SelectionChanged(object sender, EventArgs e)
        {
            if (dgvGroupSpeechTimers.SelectedRows.Count > 0)
            {
                dgvGroupSpeechTimers.ClearSelection();
            }
        }

        public delegate void UpdateHeaderDateAndTimeDelegate();
        public void UpdateHeaderDateAndTime()
        {
            if (this.InvokeRequired)
            {
                UpdateHeaderDateAndTimeDelegate obj = new UpdateHeaderDateAndTimeDelegate(UpdateHeaderDateAndTime);
                this.BeginInvoke(obj);
            }
            else
            {
                this.headerControl.lblDate.Text = DateTime.Now.ToShortDateString().ToString(CultureInfo.InvariantCulture);
                this.headerControl.lblTime.Text = DateTime.Now.ToShortTimeString().ToString(CultureInfo.InvariantCulture);
            }
        }

        private delegate void UpdateGroupSpeechDataDelegate();

        public void LoadGroupSpeechData()
        {
            if (this.InvokeRequired)
            {
                UpdateGroupSpeechDataDelegate obj = new UpdateGroupSpeechDataDelegate(LoadGroupSpeechData);
                this.BeginInvoke(obj);
            }
            else
            {
                
                this.headerControl.lblSubject.Text = Properties.Resources.GroupSpeechTimersTxtSubject;
                this.headerControl.lblDate.Text = DateTime.Now.ToShortDateString().ToString(CultureInfo.InvariantCulture);
                this.headerControl.lblTime.Text = DateTime.Now.ToShortTimeString().ToString(CultureInfo.InvariantCulture);

                if(dgvGroupSpeechTimers.SelectedRows.Count > 0)
                {
                    dgvGroupSpeechTimers.ClearSelection();
                }
            }
        }

        private void dgvGroupSpeechTimers_CellFormatting(object sender, DataGridViewCellFormattingEventArgs e)
        {
            string displatTime = string.Empty;

            int columnIndex = dgvGroupSpeechTimers.Columns["GroupSpeechTimeLimit"].Index;
            if (columnIndex == e.ColumnIndex)
            {
                if (e.Value is long && ((long)e.Value != -1000))
                {
                    try
                    {
                        long timeInMillis = (long)e.Value;
                        TimeSpan timeSpan = new TimeSpan(timeInMillis * TimeSpan.TicksPerMillisecond); // Convert to ticks

                        string timeString = string.Empty;
                        timeString = string.Format(CultureInfo.InvariantCulture
                            , SpeechTimeFormatString
                            , Math.Abs(timeSpan.Hours)
                            , Math.Abs(timeSpan.Minutes)
                            , Math.Abs(timeSpan.Seconds));

                        if (timeSpan.TotalMilliseconds < 0)
                        {
                            timeString = "-" + timeString;
                        }

                        e.Value = timeString;
                    }
                    catch (FormatException)
                    {
                        e.Value = "##:##:##";
                    }
                    catch (ArgumentNullException)
                    {
                        e.Value = "##:##:##";
                    }
                }
                else
                {
                    e.Value = string.Empty;
                }
            }
        }

        private void OnFullScreenClick(object sender, EventArgs e)
        {
            if (this.miFullScreen.Checked)
            {
                this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.FixedSingle;
                this.miFullScreen.Checked = false;
            }
            else if (!this.miFullScreen.Checked)
            {
                this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.None;
                this.WindowState = FormWindowState.Maximized;
                this.miFullScreen.Checked = true;
            }
        }
        #endregion

        #region Fields

        ContextMenu cmIndvResults = new System.Windows.Forms.ContextMenu();
        MenuItem miFullScreen = new MenuItem();
        string remainSpeechTime = string.Empty;
        public const string SpeechTimeFormatString = "{0:00}:{1:00}:{2:00}";
        #endregion

       
    }
    
}
