﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Reflection;
using DcnSwSmd.TraceListeners.Data;
using System.Globalization;

namespace DcnSwSmd
{

    /// <summary>
    /// Form to display the Individual Voting results.
    /// </summary>
    public partial class IndividualResults : Form
    {
        #region Functions
        /// <summary>
        /// Constructor.
        /// </summary>
        /// <param name="votingIndvResults"></param>
        public IndividualResults(SMDVotingData votingIndvResults)
        {

            InitializeComponent();
            
            miFullScreen.Checked = true;
            miFullScreen.Text = "Full Screen";
            miFullScreen.Click += new EventHandler(OnFullScreenClick);
            cmIndvResults.MenuItems.Add(miFullScreen);
                        
            this.ContextMenu = cmIndvResults;
           
            ResetUserInterface();
            VotingIndvResults = votingIndvResults;

            //CreateDtIntermediateTable();
            //this.indvResultsDataGridView.Columns["IndividualResult"].Visible = false;
        }

        /// <summary>
        /// To recreate the Datatable for Intermediate Voting results.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        public void CreateDtIntermediateTable()
        {
        //    dtInterimIndvResults = new DataTable();
        //    dtInterimIndvResults.Columns.Add("Participant_Id", typeof(string));
        //    dtInterimIndvResults.Columns.Add("Answer", typeof(string));
        //    dtInterimIndvResults.Columns.Add("ParticipantName", typeof(string));
        //    dtInterimIndvResults.Columns.Add("ParticipantGroup", typeof(string));
        //    DataColumn[] columnPrimaryKey = new DataColumn[1];
        //    columnPrimaryKey[0] = dtInterimIndvResults.Columns["Participant_Id"];
        //    dtInterimIndvResults.PrimaryKey = columnPrimaryKey;
           
        }
        public delegate void ClearPreviousResultsDelegate();
        public void ClearPreviousResults()
        {
            if (this.InvokeRequired)
            {
                ClearPreviousResultsDelegate obj = new ClearPreviousResultsDelegate(ClearPreviousResults);
                this.BeginInvoke(obj);
            }
            else
            {
                if (indvResultsDataSource != null)
                {
                    indvResultsDataSource.Clear();
                }
            }
        }
            
        private void OnFullScreenClick(object sender, EventArgs e)
        {
            if (this.miFullScreen.Checked)
            {
                this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.FixedSingle;
                this.miFullScreen.Checked = false;
            }
            else if (!this.miFullScreen.Checked)
            {
                this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.None;
                this.WindowState = FormWindowState.Maximized;
                this.miFullScreen.Checked = true;
            }
        }
        private void ResetUserInterface()
        {
            this.lblTotalValue.Text = "0";
            this.lblNotVotedValue.Text = "0";
            this.lblAns1Value.Text = "0";
            this.lblAns2Value.Text = "0";
            this.lblAns3Value.Text = "0";
            this.lblAns4Value.Text = "0";
            this.lblAns5Value.Text = "0";
        }

        /// <summary>
        /// Function to load the required data to display.
        /// </summary>
        /// <param name="votingIndvResults"></param>
        public delegate void LoadVotingdataDelegate(SMDVotingData votingIndvResults, bool votingStopped);
        public void LoadVotingData(SMDVotingData votingIndvResults, bool votingStopped)
        {
            if (this.InvokeRequired)
            {
                LoadVotingdataDelegate obj = new LoadVotingdataDelegate(LoadVotingData);
                this.BeginInvoke(obj, votingIndvResults, votingStopped);
            }
            else
            {
                if (votingIndvResults != null)
                {
                    //Update voting Metadata.
                    
                    if (votingIndvResults.VotingData != null)
                    {
                        TimeSpan votingTimeLimit;
                        this.headerControl.lblDate.Text = DateTime.Now.ToShortDateString().ToString(CultureInfo.InvariantCulture);
                        this.headerControl.lblTime.Text = DateTime.Now.ToShortTimeString().ToString(CultureInfo.InvariantCulture);
                        this.headerControl.lblSubject.Text = votingIndvResults.VotingData.Subject;

                        if (votingIndvResults.VotingData.RemainingVotingTime <= -1)
                        {
                            this.headerControl.lblTimeLimit.Text = Properties.Resources.VotingResultsTxtTimeLimit + ": NA";
                        }
                        else
                        {
                            votingTimeLimit = new TimeSpan(votingIndvResults.VotingData.RemainingVotingTime * TimeSpan.TicksPerSecond);
                            this.headerControl.lblTimeLimit.Text = Properties.Resources.VotingResultsTxtTimeLimit + ": " + string.Format(System.Globalization.CultureInfo.InvariantCulture, RemainingvotingTimeFormatString, Math.Abs(votingTimeLimit.Minutes), Math.Abs(votingTimeLimit.Seconds));
                        }
                    }
                    else
                    {
                        MessageBox.Show("Voting Metadata not available");

                    }

                    // Updating the Total results.
                    if (votingIndvResults.Answers != null)
                    {
                        UpdateTotalResults(votingIndvResults);
                        
                     
                        
                        // Updating the DataGridView With Individual results.
                    //    if (votingIndvResults.IndividualResults != null && votingIndvResults.IndividualResults.Count > 0)
                    //    {
                    //        dtIndividualResults = new DataTable();
                    //        dtIndividualResults.Columns.Add("Answer", typeof(string));
                    //        dtIndividualResults.Columns.Add("Participant Name", typeof(string));
                    //        dtIndividualResults.Columns.Add("Participant Group", typeof(string));

                    //        int ansId = 0;
                    //        string participantName = String.Empty;
                    //        string participantGroup = String.Empty;
                    //        string answer = String.Empty;

                    //        for (int iCount = 0; iCount < votingIndvResults.IndividualResults.Count && iCount < 10; iCount++)
                    //        {
                    //            ansId = votingIndvResults.IndividualResults[iCount].AnswerId;
                    //            int ansIndex = votingIndvResults.Answers.FindIndex(delegate(AnswerContainer ansCont) { return ansCont.Id == ansId; });
                    //            if (ansIndex > -1)
                    //            {
                    //                answer = votingIndvResults.Answers[ansIndex].AnswerText;
                    //            }
                    //            else
                    //            {
                    //                answer = "Not Voted";
                    //            }
                    //            participantName = votingIndvResults.IndividualResults[iCount].Participant.ParticipantData.FirstName + " " + votingIndvResults.IndividualResults[iCount].Participant.ParticipantData.LastName;
                    //            participantGroup = votingIndvResults.IndividualResults[iCount].Participant.Group.Name;
                    //            dtIndividualResults.Rows.Add(answer, participantName, participantGroup);
                    //            dtIndividualResults.TableName = "Individual Resluts";
                    //        }

                    //        if (votingStopped)
                    //        {
                    //            this.indvResultsDataGridView.DataSource = dtIndividualResults;
                    //            if (indvResultsDataGridView.Columns.Contains("IndividualResult"))
                    //            {
                    //                this.indvResultsDataGridView.Columns["IndividualResult"].Visible = true;
                    //            }
                    //            //this.indvResultsDataGridView.Columns["IndividualResult"].Visible = true;
                    //        }
                    //        else
                    //        {
                    //            this.indvResultsDataGridView.DataSource = dtInterimIndvResults;
                    //            if (DtIntermediateIndvResults.Rows.Count <= 0)
                    //            {
                    //                ResetUserInterface();
                    //            }
                    //            if (indvResultsDataGridView.Columns.Contains("Participant_Id"))
                    //            {
                    //                this.indvResultsDataGridView.Columns["Participant_Id"].Visible = false;
                    //            }
                    //            if (indvResultsDataGridView.Columns.Contains("IndividualResult"))
                    //            {
                    //                this.indvResultsDataGridView.Columns["IndividualResult"].Visible = true;
                    //            }
                    //        }

                    //        foreach (DataGridViewColumn column in this.indvResultsDataGridView.Columns)
                    //        {
                    //            column.SortMode = DataGridViewColumnSortMode.NotSortable;
                    //        }
                    //        UpdateGridViewImage();
                    //    }
                    //}

                    //if (indvResultsDataGridView.Columns.Contains("Answer"))
                    //{
                    //    indvResultsDataGridView.Columns["Answer"].Visible = false;
                    }
                    //indvResultsBindingSource.DataSource = indvResultsDataSource;
                }

                else
                {
                    MessageBox.Show("Voting metadata is not available");
                }
               
                
            }
        }

        private delegate void UpdateVotingResults(SMDVotingData votingIndvResults);

        public void UpdateTotalResults(SMDVotingData votingIndvResults)
        {
            if (this.InvokeRequired)
            {
                UpdateVotingResults obj = new UpdateVotingResults(UpdateTotalResults);
                this.BeginInvoke(obj, votingIndvResults);
            }
            else
            {
                try
                {
                    lock (this)
                    {
                        if (votingIndvResults.TotalResults != null)
                        {
                            int index = 0;
                            string ans = String.Empty;
                            int total = 0;
                            for (int iCount = 0; iCount < votingIndvResults.TotalResults.VotingAnswerResults.Count; iCount++)
                            {


                                index = votingIndvResults.Answers.FindIndex(delegate(AnswerContainer ansCont) { return ansCont.Id == votingIndvResults.TotalResults.VotingAnswerResults[iCount].AnswerId; });
                                #region Parlimentary
                                if (votingIndvResults.Answers[index].AnswerText == "Yes" && lblAns1.Visible && lblAns1Value.Visible)
                                {
                                    this.lblAns1.Visible = true;
                                    this.lblAns1.Text = votingIndvResults.Answers[index].AnswerText + ": ";
                                    this.lblAns1Value.Image = Properties.Resources.Green;
                                    this.lblAns1Value.ImageAlign = ContentAlignment.MiddleCenter;
                                    this.lblAns1Value.Visible = true;
                                    this.lblAns1Value.Text = votingIndvResults.TotalResults.VotingAnswerResults[index].NumberOfCasts.ToString(CultureInfo.InvariantCulture);
                                }
                                else if (votingIndvResults.Answers[index].AnswerText == "No" && lblAns2.Visible && lblAns2Value.Visible)
                                {
                                    this.lblAns2.Text = votingIndvResults.Answers[index].AnswerText + ": "; 
                                    this.lblAns2.Visible = true;
                                    this.lblAns2Value.Image = Properties.Resources.Red;
                                    this.lblAns2Value.ImageAlign = ContentAlignment.MiddleCenter;
                                    this.lblAns2Value.Visible = true;
                                    this.lblAns2Value.Text = votingIndvResults.TotalResults.VotingAnswerResults[index].NumberOfCasts.ToString(CultureInfo.InvariantCulture);
                                }
                                else if (votingIndvResults.Answers[index].AnswerText == "Abstain" && lblAns3.Visible && lblAns3Value.Visible)
                                {
                                    this.lblAns3.Text = votingIndvResults.Answers[index].AnswerText + ": "; 
                                    this.lblAns3.Visible = true;
                                    this.lblAns3Value.Image = Properties.Resources.Yellow;
                                    this.lblAns3Value.ImageAlign = ContentAlignment.MiddleCenter;
                                    this.lblAns3Value.Visible = true;
                                    this.lblAns3Value.Text = votingIndvResults.TotalResults.VotingAnswerResults[index].NumberOfCasts.ToString(CultureInfo.InvariantCulture);
                                }
                                else if (votingIndvResults.Answers[index].AnswerText == "DNPV" && lblAns4.Visible && lblAns4Value.Visible)
                                {
                                    this.lblAns4.Text = votingIndvResults.Answers[index].AnswerText + ": ";
                                    this.lblAns4.Visible = true;
                                    this.lblAns4Value.Image = Properties.Resources.Orange;
                                    this.lblAns4Value.ImageAlign = ContentAlignment.MiddleCenter;
                                    this.lblAns4Value.Visible = true;
                                    this.lblAns4Value.Text = votingIndvResults.TotalResults.VotingAnswerResults[index].NumberOfCasts.ToString(CultureInfo.InvariantCulture);
                                }
                                #endregion
                                #region AudienceResponse
                                else if (votingIndvResults.Answers[index].AnswerText == "--" && lblAns1.Visible && lblAns1Value.Visible)
                                {
                                    this.lblAns1.Text = votingIndvResults.Answers[index].AnswerText + ": ";
                                    this.lblAns1.Visible = true;
                                    this.lblAns1Value.Image = Properties.Resources.Red;
                                    this.lblAns1Value.ImageAlign = ContentAlignment.MiddleCenter;
                                    this.lblAns1Value.Visible = true;
                                    this.lblAns1Value.Text = votingIndvResults.TotalResults.VotingAnswerResults[index].NumberOfCasts.ToString(CultureInfo.InvariantCulture);
                                }
                                else if (votingIndvResults.Answers[index].AnswerText == "-" && lblAns2.Visible && lblAns2Value.Visible)
                                {
                                    this.lblAns2.Text = votingIndvResults.Answers[index].AnswerText + ": ";
                                    this.lblAns2.Visible = true;
                                    this.lblAns2Value.Image = Properties.Resources.Orange;
                                    this.lblAns2Value.ImageAlign = ContentAlignment.MiddleCenter;
                                    this.lblAns2Value.Visible = true;
                                    this.lblAns2Value.Text = votingIndvResults.TotalResults.VotingAnswerResults[index].NumberOfCasts.ToString(CultureInfo.InvariantCulture);
                                }
                                else if (votingIndvResults.Answers[index].AnswerText == "0" && lblAns3.Visible && lblAns3Value.Visible)
                                {
                                    this.lblAns3.Text = votingIndvResults.Answers[index].AnswerText + ": ";
                                    this.lblAns3.Visible = true;
                                    this.lblAns3Value.Image = Properties.Resources.Yellow;
                                    this.lblAns3Value.ImageAlign = ContentAlignment.MiddleCenter;
                                    this.lblAns3Value.Visible = true;
                                    this.lblAns3Value.Text = votingIndvResults.TotalResults.VotingAnswerResults[index].NumberOfCasts.ToString(CultureInfo.InvariantCulture);
                                }
                                else if (votingIndvResults.Answers[index].AnswerText == "+" && lblAns4.Visible && lblAns4Value.Visible)
                                {
                                    this.lblAns4.Text = votingIndvResults.Answers[index].AnswerText + ": ";
                                    this.lblAns4.Visible = true;
                                    this.lblAns4Value.Image = Properties.Resources.Green;
                                    this.lblAns4Value.ImageAlign = ContentAlignment.MiddleCenter;
                                    this.lblAns4Value.Visible = true;
                                    this.lblAns4Value.Text = votingIndvResults.TotalResults.VotingAnswerResults[index].NumberOfCasts.ToString(CultureInfo.InvariantCulture);
                                }
                                else if (votingIndvResults.Answers[index].AnswerText == "++" && lblAns5.Visible && lblAns5Value.Visible)
                                {
                                    this.lblAns5.Text = votingIndvResults.Answers[index].AnswerText + ": ";
                                    this.lblAns5.Visible = true;
                                    this.lblAns5Value.Image = Properties.Resources.DarkGreen;
                                    this.lblAns5Value.ImageAlign = ContentAlignment.MiddleCenter;
                                    this.lblAns5Value.Visible = true;
                                    this.lblAns5Value.Text = votingIndvResults.TotalResults.VotingAnswerResults[index].NumberOfCasts.ToString(CultureInfo.InvariantCulture);
                                }
                                #endregion
                                #region OpinionPoll
                                else if (votingIndvResults.Answers[index].AnswerText == "1" && lblAns1.Visible && lblAns1Value.Visible)
                                {
                                    this.lblAns1.Text = votingIndvResults.Answers[index].AnswerText + ": ";
                                    this.lblAns1.Visible = true;
                                    this.lblAns1Value.Image = Properties.Resources.Red;
                                    this.lblAns1Value.ImageAlign = ContentAlignment.MiddleCenter;
                                    this.lblAns1Value.Visible = true;
                                    this.lblAns1Value.Text = votingIndvResults.TotalResults.VotingAnswerResults[index].NumberOfCasts.ToString(CultureInfo.InvariantCulture);
                                }
                                else if (votingIndvResults.Answers[index].AnswerText == "2" && lblAns2.Visible && lblAns2Value.Visible)
                                {
                                    this.lblAns2.Text = votingIndvResults.Answers[index].AnswerText + ": ";
                                    this.lblAns2.Visible = true;
                                    this.lblAns2Value.Image = Properties.Resources.Orange;
                                    this.lblAns2Value.ImageAlign = ContentAlignment.MiddleCenter;
                                    this.lblAns2Value.Visible = true;
                                    this.lblAns2Value.Text = votingIndvResults.TotalResults.VotingAnswerResults[index].NumberOfCasts.ToString(CultureInfo.InvariantCulture);
                                }
                                else if (votingIndvResults.Answers[index].AnswerText == "3" && lblAns3.Visible && lblAns3Value.Visible)
                                {
                                    this.lblAns3.Text = votingIndvResults.Answers[index].AnswerText + ": ";
                                    this.lblAns3.Visible = true;
                                    this.lblAns3Value.Image = Properties.Resources.Yellow;
                                    this.lblAns3Value.ImageAlign = ContentAlignment.MiddleCenter;
                                    this.lblAns3Value.Visible = true;
                                    this.lblAns3Value.Text = votingIndvResults.TotalResults.VotingAnswerResults[index].NumberOfCasts.ToString(CultureInfo.InvariantCulture);
                                }
                                else if (votingIndvResults.Answers[index].AnswerText == "4" && lblAns4.Visible && lblAns4Value.Visible)
                                {
                                    this.lblAns4.Text = votingIndvResults.Answers[index].AnswerText + ": ";
                                    this.lblAns4.Visible = true;
                                    this.lblAns4Value.Image = Properties.Resources.Green;
                                    this.lblAns4Value.ImageAlign = ContentAlignment.MiddleCenter;
                                    this.lblAns4Value.Visible = true;
                                    this.lblAns4Value.Text = votingIndvResults.TotalResults.VotingAnswerResults[index].NumberOfCasts.ToString(CultureInfo.InvariantCulture);
                                }
                                else if (votingIndvResults.Answers[index].AnswerText == "5" && lblAns5.Visible && lblAns5Value.Visible)
                                {
                                    this.lblAns5.Text = votingIndvResults.Answers[index].AnswerText + ": ";
                                    this.lblAns5.Visible = true;
                                    this.lblAns5Value.Image = Properties.Resources.DarkGreen;
                                    this.lblAns5Value.ImageAlign = ContentAlignment.MiddleCenter;
                                    this.lblAns5Value.Visible = true;
                                    this.lblAns5Value.Text = votingIndvResults.TotalResults.VotingAnswerResults[index].NumberOfCasts.ToString(CultureInfo.InvariantCulture);
                                }

                                #endregion

                                total += votingIndvResults.TotalResults.VotingAnswerResults[index].NumberOfCasts;
                            }
                            this.lblTotal.Visible = true;
                            this.lblTotalValue.Visible = true;
                            this.lblNotVoted.Visible = true;
                            this.lblNotVotedValue.Visible = true;

                            this.lblTotalValue.Text = total.ToString(CultureInfo.InvariantCulture);
                            this.lblNotVotedValue.Text = (votingIndvResults.TotalResults.MaximumMajority - total).ToString(CultureInfo.InvariantCulture);
                        }
                    }
                }
                catch (Exception)
                {
                }
            }
        }

        private delegate void UpdateGridImage();
        public void UpdateGridViewImage()
        {
            if (this.InvokeRequired)
            {
                UpdateGridImage obj = new UpdateGridImage(UpdateGridViewImage);
                this.BeginInvoke(obj);
            }
            else
            {
                try
                {
                    
                        if (this.indvResultsDataGridView != null && this.indvResultsDataGridView.RowCount > 0)
                        {
                            foreach (DataGridViewRow row in this.indvResultsDataGridView.Rows)
                            {
                                if (row.Cells["Answer"].Value != null && row.Cells["Answer"].Value.ToString() == "Yes")
                                {
                                    row.Cells["IndividualResult"].Value = null;
                                    row.Cells["IndividualResult"].Value = Properties.Resources.Green;
                                    row.Cells["IndividualResult"].Style.Alignment = DataGridViewContentAlignment.MiddleCenter;
                                }
                                else if (row.Cells["Answer"].Value != null && row.Cells["Answer"].Value.ToString() == "No")
                                {
                                    row.Cells["IndividualResult"].Value = null;
                                    row.Cells["IndividualResult"].Value = Properties.Resources.Red;
                                    row.Cells["IndividualResult"].Style.Alignment = DataGridViewContentAlignment.MiddleCenter;
                                }
                                else if (row.Cells["Answer"].Value != null && row.Cells["Answer"].Value.ToString() == "Abstain")
                                {
                                    row.Cells["IndividualResult"].Value = null;
                                    row.Cells["IndividualResult"].Value = Properties.Resources.Yellow;
                                    row.Cells["IndividualResult"].Style.Alignment = DataGridViewContentAlignment.MiddleCenter;
                                }
                                else if (row.Cells["Answer"].Value != null && row.Cells["Answer"].Value.ToString() == "DNPV")
                                {
                                    row.Cells["IndividualResult"].Value = null;
                                    row.Cells["IndividualResult"].Value = Properties.Resources.Orange;
                                    row.Cells["IndividualResult"].Style.Alignment = DataGridViewContentAlignment.MiddleCenter;
                                }
                                else if (row.Cells["Answer"].Value != null && row.Cells["Answer"].Value.ToString() == "Not Voted")
                                {
                                    row.Cells["IndividualResult"].Value = null;
                                    row.Cells["IndividualResult"].Value = Properties.Resources.Blue;
                                    row.Cells["IndividualResult"].Style.Alignment = DataGridViewContentAlignment.MiddleCenter;
                                }

                                if (row.Cells["Answer"].Value != null && row.Cells["Answer"].Value.ToString() == "--")
                                {
                                    row.Cells["IndividualResult"].Value = null;
                                    row.Cells["IndividualResult"].Value = Properties.Resources.Red;
                                    row.Cells["IndividualResult"].Style.Alignment = DataGridViewContentAlignment.MiddleCenter;
                                }
                                else if (row.Cells["Answer"].Value != null && row.Cells["Answer"].Value.ToString() == "-")
                                {
                                    row.Cells["IndividualResult"].Value = null;
                                    row.Cells["IndividualResult"].Value = Properties.Resources.Orange;
                                    row.Cells["IndividualResult"].Style.Alignment = DataGridViewContentAlignment.MiddleCenter;
                                }
                                else if (row.Cells["Answer"].Value != null && row.Cells["Answer"].Value.ToString() == "0")
                                {
                                    row.Cells["IndividualResult"].Value = null;
                                    row.Cells["IndividualResult"].Value = Properties.Resources.Yellow;
                                    row.Cells["IndividualResult"].Style.Alignment = DataGridViewContentAlignment.MiddleCenter;
                                }
                                else if (row.Cells["Answer"].Value != null && row.Cells["Answer"].Value.ToString() == "+")
                                {
                                    row.Cells["IndividualResult"].Value = null;
                                    row.Cells["IndividualResult"].Value = Properties.Resources.Green;
                                    row.Cells["IndividualResult"].Style.Alignment = DataGridViewContentAlignment.MiddleCenter;
                                }
                                else if (row.Cells["Answer"].Value != null && row.Cells["Answer"].Value.ToString() == "++")
                                {
                                    row.Cells["IndividualResult"].Value = null;
                                    row.Cells["IndividualResult"].Value = Properties.Resources.DarkGreen;
                                    row.Cells["IndividualResult"].Style.Alignment = DataGridViewContentAlignment.MiddleCenter;
                                }

                                if (row.Cells["Answer"].Value != null && row.Cells["Answer"].Value.ToString() == "1")
                                {
                                    row.Cells["IndividualResult"].Value = null;
                                    row.Cells["IndividualResult"].Value = Properties.Resources.Red;
                                    row.Cells["IndividualResult"].Style.Alignment = DataGridViewContentAlignment.MiddleCenter;
                                }
                                else if (row.Cells["Answer"].Value != null && row.Cells["Answer"].Value.ToString() == "2")
                                {
                                    row.Cells["IndividualResult"].Value = null;
                                    row.Cells["IndividualResult"].Value = Properties.Resources.Orange;
                                    row.Cells["IndividualResult"].Style.Alignment = DataGridViewContentAlignment.MiddleCenter;
                                }
                                else if (row.Cells["Answer"].Value != null && row.Cells["Answer"].Value.ToString() == "3")
                                {
                                    row.Cells["IndividualResult"].Value = null;
                                    row.Cells["IndividualResult"].Value = Properties.Resources.Yellow;
                                    row.Cells["IndividualResult"].Style.Alignment = DataGridViewContentAlignment.MiddleCenter;
                                }
                                else if (row.Cells["Answer"].Value != null && row.Cells["Answer"].Value.ToString() == "4")
                                {
                                    row.Cells["IndividualResult"].Value = null;
                                    row.Cells["IndividualResult"].Value = Properties.Resources.Green;
                                    row.Cells["IndividualResult"].Style.Alignment = DataGridViewContentAlignment.MiddleCenter;
                                }
                                else if (row.Cells["Answer"].Value != null && row.Cells["Answer"].Value.ToString() == "5")
                                {
                                    row.Cells["IndividualResult"].Value = null;
                                    row.Cells["IndividualResult"].Value = Properties.Resources.DarkGreen;
                                    row.Cells["IndividualResult"].Style.Alignment = DataGridViewContentAlignment.MiddleCenter;
                                }
                            }
                        }
                    
                   
                }
                catch (Exception)
                {

                }
            }
        }

        private void HideLabels()
        {
            this.lblAns1.Visible = false;
            this.lblAns1Value.Visible = false;
            this.lblAns2.Visible = false;
            this.lblAns2Value.Visible = false;
            this.lblAns3.Visible = false;
            this.lblAns3Value.Visible = false;
            this.lblAns4.Visible = false;
            this.lblAns4Value.Visible = false;
            this.lblAns5.Visible = false;
            this.lblAns5Value.Visible = false;

            this.lblTotal.Visible = false;
            this.lblTotalValue.Visible = false;
            this.lblNotVoted.Visible = false;
            this.lblNotVotedValue.Visible = false;
        }


        private delegate void UpdateStaticDataDelegate(SMDVotingData votingIndvResults);
        
        /// <summary>
        /// To update the header info.
        /// </summary>
        /// <param name="votingIndvResults"></param>
        public void UpdateStaticData(SMDVotingData votingIndvResults)
        {
            if (this.InvokeRequired)
            {
                UpdateStaticDataDelegate obj = new UpdateStaticDataDelegate(UpdateStaticData);
                this.BeginInvoke(obj, votingIndvResults);
            }
            else
            {
                if (votingIndvResults != null)
                {
                    
                    ResetUserInterface();
                    
                    this.headerControl.lblDate.Text = DateTime.Now.ToShortDateString().ToString(CultureInfo.InvariantCulture);
                    this.headerControl.lblTime.Text = DateTime.Now.ToShortTimeString().ToString(CultureInfo.InvariantCulture);
                    if (votingIndvResults.VotingData != null)
                    {
                        this.headerControl.lblSubject.Text = votingIndvResults.VotingData.Subject;
                        if (votingIndvResults.VotingData.RemainingVotingTime <= -1)
                        {
                            this.headerControl.lblTimeLimit.Text = Properties.Resources.VotingResultsTxtTimeLimit + ": NA";
                        }
                        else
                        {
                            TimeSpan votingTimeLimit;
                            votingTimeLimit = new TimeSpan(votingIndvResults.VotingData.RemainingVotingTime * TimeSpan.TicksPerSecond);
                            this.headerControl.lblTimeLimit.Text = Properties.Resources.VotingResultsTxtTimeLimit + ": " + string.Format(System.Globalization.CultureInfo.InvariantCulture, RemainingvotingTimeFormatString, Math.Abs(votingTimeLimit.Minutes), Math.Abs(votingTimeLimit.Seconds));
                        }
                    }
                    HideLabels();
                    if (votingIndvResults.Answers != null)
                    {
                        foreach (AnswerContainer answer in votingIndvResults.Answers)
                        {
                            #region Parlimentary
                            if (answer.AnswerText == "Yes")
                            {
                                lblAns1.Text = "Yes: ";
                                lblAns1.ForeColor = Color.Green;
                                lblAns1.Visible = true;

                                lblAns1Value.Text = "0";
                                lblAns1Value.ForeColor = Color.Green;
                                lblAns1Value.Image = Properties.Resources.Green;
                                lblAns1Value.ImageAlign = ContentAlignment.MiddleCenter;
                                lblAns1Value.Visible = true;
                            }
                            else if (answer.AnswerText == "No")
                            {
                                lblAns2.Text = "No: ";
                                lblAns2.ForeColor = Color.Red;
                                lblAns2.Visible = true;

                                lblAns2Value.ForeColor = Color.Red;
                                lblAns2Value.Text = "0";
                                lblAns2Value.Image = Properties.Resources.Red;
                                lblAns2Value.ImageAlign = ContentAlignment.MiddleCenter;
                                lblAns2Value.Visible = true;
                            }
                            else if (answer.AnswerText == "Abstain")
                            {
                                lblAns3.Text = "Abstain: ";
                                lblAns3.ForeColor = Color.Yellow;
                                lblAns3.Visible = true;

                                lblAns3Value.ForeColor = Color.Yellow;
                                lblAns3Value.Text = "0";
                                lblAns3Value.Image = Properties.Resources.Yellow;
                                lblAns3Value.ImageAlign = ContentAlignment.MiddleCenter;
                                lblAns3Value.Visible = true;
                            }
                            else if (answer.AnswerText == "DNPV")
                            {

                                lblAns4.Text = "DNPV: ";
                                lblAns4.ForeColor = Color.Orange;
                                lblAns4.Visible = true;

                                lblAns4Value.Text = "0";
                                lblAns4Value.ForeColor = Color.Orange;
                                lblAns4Value.Image = Properties.Resources.Orange;
                                lblAns4Value.ImageAlign = ContentAlignment.MiddleCenter;
                                lblAns4Value.Visible = true;
                            }
                            #endregion

                            #region AudienceResponse

                            else if (answer.AnswerText == "--")
                            {
                                lblAns1.Text = "--: ";
                                lblAns1.ForeColor = Color.Red;
                                lblAns1.Visible = true;

                                lblAns1Value.Text = "0";
                                lblAns1Value.ForeColor = Color.Red;
                                lblAns1Value.Image = Properties.Resources.Red;
                                lblAns1Value.ImageAlign = ContentAlignment.MiddleCenter;
                                lblAns1Value.Visible = true;
                            }
                            else if (answer.AnswerText == "-")
                            {
                                lblAns2.Text = "-: ";
                                lblAns2.ForeColor = Color.Orange;
                                lblAns2.Visible = true;

                                lblAns2Value.ForeColor = Color.Orange;
                                lblAns2Value.Text = "0";
                                lblAns2Value.Image = Properties.Resources.Orange;
                                lblAns2Value.ImageAlign = ContentAlignment.MiddleCenter;
                                lblAns2Value.Visible = true;
                            }
                            else if (answer.AnswerText == "0")
                            {
                                lblAns3.Text = "0: ";
                                lblAns3.ForeColor = Color.Yellow;
                                lblAns3.Visible = true;

                                lblAns3Value.ForeColor = Color.Yellow;
                                lblAns3Value.Text = "0";
                                lblAns3Value.Image = Properties.Resources.Yellow;
                                lblAns3Value.ImageAlign = ContentAlignment.MiddleCenter;
                                lblAns3Value.Visible = true;
                            }
                            else if (answer.AnswerText == "+")
                            {

                                lblAns4.Text = "+: ";
                                lblAns4.ForeColor = Color.Green;
                                lblAns4.Visible = true;

                                lblAns4Value.Text = "0";
                                lblAns4Value.ForeColor = Color.Green;
                                lblAns4Value.Image = Properties.Resources.Green;
                                lblAns4Value.ImageAlign = ContentAlignment.MiddleCenter;
                                lblAns4Value.Visible = true;
                            }
                            else if (answer.AnswerText == "++")
                            {
                                lblAns5.Text = "++: ";
                                lblAns5.ForeColor = Color.DarkGreen;
                                lblAns5.Visible = true;

                                lblAns5Value.Text = "0";
                                lblAns5Value.ForeColor = Color.DarkGreen;
                                lblAns5Value.Image = Properties.Resources.DarkGreen;
                                lblAns5Value.ImageAlign = ContentAlignment.MiddleCenter;
                                lblAns5Value.Visible = true;
                            }

                            #endregion

                            #region OpinionPoll

                            else if (answer.AnswerText == "1")
                            {

                                lblAns1.Text = "1: ";
                                lblAns1.ForeColor = Color.Red;
                                lblAns1.Visible = true;

                                lblAns1Value.Text = "0";
                                lblAns1Value.ForeColor = Color.Red;
                                lblAns1Value.Image = Properties.Resources.Red;
                                lblAns1Value.ImageAlign = ContentAlignment.MiddleCenter;
                                lblAns1Value.Visible = true;
                            }
                            else if (answer.AnswerText == "2")
                            {
                                lblAns2.Text = "2: ";
                                lblAns2.ForeColor = Color.Orange;
                                lblAns2.Visible = true;

                                lblAns2Value.ForeColor = Color.Orange;
                                lblAns2Value.Text = "0";
                                lblAns2Value.Image = Properties.Resources.Orange;
                                lblAns2Value.ImageAlign = ContentAlignment.MiddleCenter;
                                lblAns2Value.Visible = true;
                            }
                            else if (answer.AnswerText == "3")
                            {
                                lblAns3.Text = "3: ";
                                lblAns3.ForeColor = Color.Yellow;
                                lblAns3.Visible = true;

                                lblAns3Value.ForeColor = Color.Yellow;
                                lblAns3Value.Text = "0";
                                lblAns3Value.Image = Properties.Resources.Yellow;
                                lblAns3Value.ImageAlign = ContentAlignment.MiddleCenter;
                                lblAns3Value.Visible = true;
                            }
                            else if (answer.AnswerText == "4")
                            {
                                lblAns4.Text = "4: ";
                                lblAns4.ForeColor = Color.Green;
                                lblAns4.Visible = true;

                                lblAns4Value.Text = "0";
                                lblAns4Value.ForeColor = Color.Green;
                                lblAns4Value.Image = Properties.Resources.Green;
                                lblAns4Value.ImageAlign = ContentAlignment.MiddleCenter;
                                lblAns4Value.Visible = true;
                            }
                            else if (answer.AnswerText == "5")
                            {
                                lblAns5.Text = "5: ";
                                lblAns5.ForeColor = Color.DarkGreen;
                                lblAns5.Visible = true;


                                lblAns5Value.Text = "0";
                                lblAns5Value.ForeColor = Color.DarkGreen;
                                lblAns5Value.Image = Properties.Resources.DarkGreen;
                                lblAns5Value.ImageAlign = ContentAlignment.MiddleCenter;
                                lblAns5Value.Visible = true;
                            }
                            #endregion

                            this.lblTotal.Visible = true;
                            this.lblTotalValue.Visible = true;
                            this.lblNotVoted.Visible = true;
                            this.lblNotVotedValue.Visible = true;
                            ResetUserInterface();
                        }
                    }
                }
            }
        }
        #region Interim Results
        // For Imterim Results
        public delegate void AddorUpdateInterimResultsDelegate(SMDVotingData votingIndvResults);
        public void AddOrUpdateInterimResults(SMDVotingData votingIndvResults)
        {
            if (this.InvokeRequired)
            {
                AddorUpdateInterimResultsDelegate obj = new AddorUpdateInterimResultsDelegate(AddOrUpdateInterimResults);
                this.BeginInvoke(obj, votingIndvResults);
            }
            else
            {
                try
                {
                    if (votingIndvResults != null)
                    {
                        if (votingIndvResults.IndividualResults != null && votingIndvResults.IndividualResults.Count > 0)
                        {

                            int ansId = 0;

                            IndvResults indvResult = new IndvResults();
                            string participantId = String.Empty;
                            string participantName = String.Empty;
                            string participantGroup = String.Empty;
                            string answer = String.Empty;

                            for (int iCount = 0; iCount < votingIndvResults.IndividualResults.Count && iCount < 10; iCount++)
                            {
                                ansId = votingIndvResults.IndividualResults[iCount].AnswerId;
                                int ansIndex = votingIndvResults.Answers.FindIndex(delegate(AnswerContainer ansCont) { return ansCont.Id == ansId; });
                                if (ansIndex > -1)
                                {
                                    indvResult.Answer = votingIndvResults.Answers[ansIndex].AnswerText;
                                    //answer = votingIndvResults.Answers[ansIndex].AnswerText;
                                }
                                else
                                {
                                    indvResult.Answer = "Not Voted";
                                    //answer = "Not Voted";
                                }

                                indvResult.ParticipantId = votingIndvResults.IndividualResults[iCount].Participant.Id.ToString(CultureInfo.InvariantCulture);
                                indvResult.ParticipantName = votingIndvResults.IndividualResults[iCount].Participant.ParticipantData.FirstName + " " + votingIndvResults.IndividualResults[iCount].Participant.ParticipantData.LastName;
                                indvResult.ParticipantGroup = votingIndvResults.IndividualResults[iCount].Participant.Group.Name;

                                //participantId = votingIndvResults.IndividualResults[iCount].Participant.Id.ToString(CultureInfo.InvariantCulture);
                                //participantName = votingIndvResults.IndividualResults[iCount].Participant.ParticipantData.FirstName + " " + votingIndvResults.IndividualResults[iCount].Participant.ParticipantData.LastName;
                                //participantGroup = votingIndvResults.IndividualResults[iCount].Participant.Group.Name;
                                if (indvResultsDataSource.Count > 0)
                                {
                                    var modifyIndvResultList = indvResultsDataSource.Where(x => x.ParticipantId == indvResult.ParticipantId).ToList();
                                    if (modifyIndvResultList.Count > 0)
                                    {
                                        foreach (var iResult in modifyIndvResultList)
                                        {
                                            indvResultsDataSource.Remove(iResult);
                                        }
                                        indvResultsDataSource.Add(indvResult);
                                    }
                                    else
                                    {
                                        indvResultsDataSource.Add(indvResult);
                                    }
                                }
                                else if (indvResultsDataSource.Count == 0)
                                {
                                    indvResultsDataSource.Add(indvResult);
                                }
                            }
                        }
                    }
                    indvResultsBindingSource.DataSource = indvResultsDataSource;
                    UpdateGridViewImage();
                    UpdateTotalResults(votingIndvResults);

                }
                catch (Exception)
                {

                }
            }
        }

        private void indvResultsDataGridView_DataBindingComplete(object sender, DataGridViewBindingCompleteEventArgs e)
        {
            UpdateGridViewImage();
        }
        #endregion

        //private delegate void UpdateCaptionOfControls(Control c, string str);

        //private void UpdateCaptionOfControl(Control cntr, string value)
        //{
        //    if (!IsDisposed)
        //    {
        //        if (cntr.InvokeRequired)
        //        {
        //            BeginInvoke(new UpdateCaptionOfControls(UpdateCaptionOfControl), cntr, value);
        //        }
        //        else
        //        {
        //            cntr.Text = value;
        //        }
        //    }
        //}

        //private delegate void UpdateDataSource(DataGridView ctrl, DataTable source);

        //private void UpdateDataSourceOfGrid(DataGridView ctrl, DataTable source)
        //{
        //    if (!IsDisposed)
        //    {
        //        if (ctrl.InvokeRequired)
        //        {
        //            BeginInvoke(new UpdateDataSource(UpdateDataSourceOfGrid), indvResultsDataGridView, source);
        //        }
        //        else
        //        {
        //            ctrl.DataSource = source;
        //        }
        //    }
        //}

        private delegate void UpdateDatgridView(DataGridView ctrl);
     
        private void UpdateDataGird(DataGridView ctrl)
        {
            if(!IsDisposed)
            {
                if(ctrl.InvokeRequired)
                {
                    BeginInvoke(new UpdateDatgridView(UpdateDataGird), indvResultsDataGridView);
                }
                else
                {
                    ctrl = indvResultsDataGridView;
                }
            }
        }

        private void indvResultsDataGridView_SelectionChanged(object sender, EventArgs e)
        {
            if (indvResultsDataGridView.SelectedRows.Count > 0)
            {
                indvResultsDataGridView.ClearSelection();
            }
        }

        public delegate void UpdateVotingRemainingTimeDelegate(int remainingVotingTime);

        public void UpdateRemainingVotingTime(int remainingVotingTime)
        {
            if (this.InvokeRequired)
            {
                UpdateVotingRemainingTimeDelegate obj = new UpdateVotingRemainingTimeDelegate(UpdateRemainingVotingTime);
                this.BeginInvoke(obj,remainingVotingTime);
            }
            else
            {
                this.headerControl.lblDate.Text = DateTime.Now.ToShortDateString().ToString(CultureInfo.InvariantCulture);
                this.headerControl.lblTime.Text = DateTime.Now.ToShortTimeString().ToString(CultureInfo.InvariantCulture);
                if (votingResultsData != null)
                {
                    TimeSpan votingTimeLimit;
                    if (votingResultsData.VotingData != null)
                    {
                        if (votingResultsData.VotingData.RemainingVotingTime <= -1)
                        {
                            this.headerControl.lblTimeLimit.Text = Properties.Resources.VotingResultsTxtTimeLimit + ": NA";
                        }
                        else
                        {
                            votingResultsData.VotingData.RemainingVotingTime = remainingVotingTime;
                            votingTimeLimit = new TimeSpan(votingResultsData.VotingData.RemainingVotingTime * TimeSpan.TicksPerSecond);
                            this.headerControl.lblTimeLimit.Text = Properties.Resources.VotingResultsTxtTimeLimit + ": " + string.Format(System.Globalization.CultureInfo.InvariantCulture, RemainingvotingTimeFormatString, Math.Abs(votingTimeLimit.Minutes), Math.Abs(votingTimeLimit.Seconds));
                        }
                    }
                }
            }
        }

        public delegate void UpdateHeaderDateAndTimeDelegate();
        public void UpdateHeaderDateAndTime()
        {
            if (this.InvokeRequired)
            {
                UpdateHeaderDateAndTimeDelegate obj = new UpdateHeaderDateAndTimeDelegate(UpdateHeaderDateAndTime);
                this.BeginInvoke(obj);
            }
            else
            {
                this.headerControl.lblDate.Text = DateTime.Now.ToShortDateString().ToString(CultureInfo.InvariantCulture);
                this.headerControl.lblTime.Text = DateTime.Now.ToShortTimeString().ToString(CultureInfo.InvariantCulture);
            }
        }
             

        #endregion

        #region Properties
        /// <summary>
        /// Object to hold the voting data.
        /// </summary>
        public SMDVotingData VotingIndvResults
        {
            get
            {
                return votingResultsData;
            }
            set
            {
                votingResultsData = value;
            }
        }

        public DataTable DtIntermediateIndvResults
        {
            get
            {
                return dtInterimIndvResults;
            }
            set
            {
                dtInterimIndvResults = value;
            }
        }

        public BindingList<IndvResults> IndvResultsDataSource
        {
            get
            {
                return indvResultsDataSource;
            }
            set
            {
                IndvResultsDataSource = value;
            }
        }

        #endregion

        #region Fields
        const string RemainingvotingTimeFormatString = "{0:00}:{1:00}";
        private BindingList<IndvResults> indvResultsDataSource = new BindingList<IndvResults>();
        /// <summary>
        /// A datatable to hold the interim individual Voting results.
        /// </summary>
        DataTable dtInterimIndvResults;

        SMDVotingData votingResultsData;
        /// <summary>
        /// A datatable to hold the individual Voting results.
        /// </summary>
        
        ContextMenu cmIndvResults = new System.Windows.Forms.ContextMenu();
        MenuItem miFullScreen = new MenuItem();
        #endregion

        private void indvResultsDataGridView_DataError(object sender, DataGridViewDataErrorEventArgs e)
        {
            try
            {
            }
            catch (Exception)
            {

            }
        }

        

        

    }
    public class IndvResults : INotifyPropertyChanged
    {
        string participant_Id;
        string answer;
        string participant_Name;
        string participant_Group;

        #region Properties

        public event PropertyChangedEventHandler PropertyChanged;

        private void NotifyPropertyChanged(string name)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, new PropertyChangedEventArgs(name));
        }

        public string ParticipantId
        {
            get
            {
                return participant_Id;

            }
            set
            {
                participant_Id = value;
                this.NotifyPropertyChanged("ParticipantId");
            }
        }

        public string Answer
        {
            get
            {
                return answer;
            }
            set
            {
                answer = value;
                this.NotifyPropertyChanged("Answer");
            }
        }

        public string ParticipantName
        {
            get
            {
                return participant_Name;
            }
            set
            {
                participant_Name = value;
                this.NotifyPropertyChanged("ParticipantName");
            }
        }
        public string ParticipantGroup
        {
            get
            {
                return participant_Group;
            }
            set
            {
                participant_Group = value;
                this.NotifyPropertyChanged("ParticipantGroup");
            }
        }
        #endregion
    }
}
