﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Bosch.Dcn.Ecpc.Operational.Diagnostics;
using System.Reflection;
using Bosch.Dcn.Ecpc.Server.Services.TraceListeners.Data;
using System.Xml;
using System.Xml.Serialization;
using System.IO;
using System.Windows.Forms;
using System.Data;
using System.Collections;
using System.Globalization;

namespace DcnSwSmd
{

    [Obfuscation]
    [XmlRootAttribute(ElementName = "MeetingActivity", IsNullable = false)]
    public class MeetingActivity : Activity
    {
        #region Constructor

        public MeetingActivity()
        {
            Topic = EnumTopic.Meeting;
        }

        public MeetingActivity(EnumActivityType activityType)
            : this()
        {
            this.Type = activityType;
        }

        #endregion

        #region Properties

        /// <summary>
        /// This contains the data regarding the meeting that we want to 
        /// send to the external clients via the ActivityLog.
        /// </summary>
        public MeetingContainer Meeting
        {
            get
            {
                return _meeting;
            }
            set
            {
                _meeting = value;
            }
        }

        #endregion

        #region Methods

        /// <summary>
        /// Get the string that contains the serialized data for this activity 
        /// as xml.
        /// </summary>
        internal override string GetXmlString()
        {
            return this.SerializeObject<MeetingActivity>(this);
        }

        /// <summary>
        /// Create a string that contains some specific information regarding 
        /// the activity.
        /// </summary>
        /// <returns>The activity string</returns>
        protected override string CreateActivityString()
        {
            StringBuilder stringToReturn = new StringBuilder();
            if (this.Type == EnumActivityType.AttendanceRegistrationStarted || this.Type == EnumActivityType.AttendanceRegistrationStopped)
            {
                stringToReturn.AppendFormat(" MeetingId: '{0}'",
                    _meeting.Id);
            }
            else
            {
                if (_meeting.MeetingData != null)
                {
                    stringToReturn.AppendFormat(" Subject: '{0}'",
                        _meeting.MeetingData.Subject);
                }

                if (_meeting.Participants != null)
                {
                    stringToReturn.AppendFormat(" #Participants: {0}",
                        _meeting.Participants.Count);
                }
            }
            return stringToReturn.ToString();
        }

        #endregion

        #region Fields

        private MeetingContainer _meeting = new MeetingContainer();

        #endregion
    }
}
