﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using DcnSwSmd.TraceListeners.Data;
using System.Globalization;

namespace DcnSwSmd
{
    /// <summary>
    /// Form to display the Microphone status.
    /// </summary>
    public partial class MicrophoneStatusForm : Form
    {
        #region Methods
        /// <summary>
        /// Constructor.
        /// </summary>
        public MicrophoneStatusForm()
        {
            InitializeComponent();
            miFullScreen.Checked = true;
            miFullScreen.Text = "Full Screen";
            miFullScreen.Click += new EventHandler(OnFullScreenClick);
            cmIndvResults.MenuItems.Add(miFullScreen);
            this.ContextMenu = cmIndvResults;
            this.headerControl.lblTimeLimit.Visible = false;
        }

        /// <summary>
        /// Function to load teh required data to display.
        /// </summary>
        /// <param name="discussionData"></param>
        /// 
        private delegate void UpdateMicrophoneStatusData(SMDDiscussionData dicussionData);
        public void LoadMicrophoneStatusData(SMDDiscussionData discussionData)
        {
            try
            {
                if (this.InvokeRequired)
                {
                    UpdateMicrophoneStatusData obj = new UpdateMicrophoneStatusData(LoadMicrophoneStatusData);
                    if (this.IsHandleCreated)
                    {
                        this.BeginInvoke(obj, discussionData);
                    }
                }
                else
                {

                    this.headerControl.lblSubject.Text = Properties.Resources.MicrophoneStatusTxtSubject;
                    this.headerControl.lblDate.Text = DateTime.Now.ToShortDateString().ToString(CultureInfo.InvariantCulture);
                    this.headerControl.lblTime.Text = DateTime.Now.ToShortTimeString().ToString(CultureInfo.InvariantCulture);

                    if (this.dgvSpeakers.RowCount > 0)
                    {
                        foreach (DataGridViewRow row in this.dgvSpeakers.Rows)
                        {
                            if ((row.Cells["SeatType"].Value != null && row.Cells["SeatType"].Value.ToString() == "Chairman"))
                            {
                                row.Cells["RemainingSpeechTime"].Style.BackColor = Color.LimeGreen;
                            }
                        }
                    }

                    if (dgvSpeakers.SelectedRows.Count > 0)
                    {
                        dgvSpeakers.ClearSelection();
                    }
                    if (dgvRequests.SelectedRows.Count > 0)
                    {
                        dgvRequests.ClearSelection();
                    }
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.StackTrace.ToString());
            }
        }

        

        

        private void OnFullScreenClick(object sender, EventArgs e)
        {
            if (this.miFullScreen.Checked)
            {
                this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.FixedSingle;
                this.miFullScreen.Checked = false;
            }
            else if (!this.miFullScreen.Checked)
            {
                this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.None;
                this.WindowState = FormWindowState.Maximized;
                this.miFullScreen.Checked = true;
            }
        }
       
        private void dgvSpeakers_SelectionChanged(object sender, EventArgs e)
        {
            if (dgvSpeakers.SelectedRows.Count > 0)
            {
                dgvSpeakers.ClearSelection();
            }
        }

        private void dgvRequests_SelectionChanged(object sender, EventArgs e)
        {
            if (dgvRequests.SelectedRows.Count > 0)
            {
                dgvRequests.ClearSelection();
            }
        }

        public delegate void UpdateHeaderDateAndTimeDelegate();
        public void UpdateHeaderDateAndTime()
        {
            if (this.InvokeRequired)
            {
                UpdateHeaderDateAndTimeDelegate obj = new UpdateHeaderDateAndTimeDelegate(UpdateHeaderDateAndTime);
                this.BeginInvoke(obj);
            }
            else
            {
                this.headerControl.lblDate.Text = DateTime.Now.ToShortDateString().ToString(CultureInfo.InvariantCulture);
                this.headerControl.lblTime.Text = DateTime.Now.ToShortTimeString().ToString(CultureInfo.InvariantCulture);
            }
        }

        private void dgvSpeakers_DataError(object sender, DataGridViewDataErrorEventArgs e)
        {
            try
            {

            }

            catch (Exception)
            {
                this.dgvSpeakers.Invalidate();
            }
        }

        private void dgvRequests_DataError(object sender, DataGridViewDataErrorEventArgs e)
        {
            try
            {

            }

            catch (Exception)
            {
                this.dgvRequests.Invalidate();
            }
        }

        private void dgvSpeakers_CellFormatting(object sender, DataGridViewCellFormattingEventArgs e)
        {
            string displayTime = string.Empty;
            dgvSpeakers.Columns["Speakers"].Width = dgvRequests.Width;
            int columnIndex = dgvSpeakers.Columns["RemainingSpeechTime"].Index;
            if (columnIndex == e.ColumnIndex)
            {
                if (e.Value is long && ((long)e.Value != -1000))
                {
                    try
                    {
                        long timeInMillis = (long)e.Value;
                        TimeSpan timeSpan = new TimeSpan(timeInMillis * TimeSpan.TicksPerMillisecond); // Convert to ticks

                        string timeString = string.Empty;
                        timeString = string.Format(CultureInfo.InvariantCulture
                            , SpeechTimeFormatString
                            , Math.Abs(timeSpan.Hours)
                            , Math.Abs(timeSpan.Minutes)
                            , Math.Abs(timeSpan.Seconds));

                        if (timeSpan.TotalMilliseconds < 0)
                        {
                            timeString = "-" + timeString;
                        }

                        e.Value = timeString;
                     
                        DataGridViewRow row = dgvSpeakers.Rows[e.RowIndex];
                        if ((row.Cells["SeatType"].Value != null && row.Cells["SeatType"].Value.ToString() == "Chairman"))
                        {
                            row.Cells["RemainingSpeechTime"].Style.BackColor = Color.LimeGreen;
                        }
                        if ((row.Cells["SeatType"].Value != null && row.Cells["SeatType"].Value.ToString() != "Chairman") && (row.Cells["RemainingSpeechTime"].Value != null && Convert.ToDecimal(row.Cells["InitialRemSpeechTime"].Value) != -1000 && Convert.ToDecimal(row.Cells["RemainingSpeechTime"].Value) < 60000))
                        {
                            row.Cells["RemainingSpeechTime"].Style.BackColor = Color.Red;
                        }
                    }
                    catch (FormatException)
                    {
                        e.Value = "##:##:##";
                    }
                    catch (ArgumentNullException)
                    {
                        e.Value = "##:##:##";
                    }
                }
                else
                {
                    e.Value = string.Empty;
                }
            }
        }

        #endregion

        #region Properties

        #endregion

        public string SpeechTimeFormatter
        {
            get
            {
                return SpeechTimeFormatString;
            }
        }
            
        #region Fields
        public object microphoneStatusLock = new object();
        ContextMenu cmIndvResults = new System.Windows.Forms.ContextMenu();
        MenuItem miFullScreen = new MenuItem();
        string remainSpeechTime = string.Empty;
        public const string SpeechTimeFormatString = "{0:00}:{1:00}:{2:00}";
        #endregion

        private void dgvSpeakers_DataBindingComplete(object sender, DataGridViewBindingCompleteEventArgs e)
        {
            if (this.dgvSpeakers.RowCount > 0)
            {
                foreach (DataGridViewRow row in this.dgvSpeakers.Rows)
                {
                    if ((row.Cells["SeatType"].Value != null && row.Cells["SeatType"].Value.ToString() == "Chairman"))
                    {
                        row.Cells["RemainingSpeechTime"].Style.BackColor = Color.LimeGreen;
                    }
                    if ((row.Cells["SeatType"].Value != null && row.Cells["SeatType"].Value.ToString() != "Chairman") && (row.Cells["InitialRemSpeechTime"].Value != null && Convert.ToDecimal(row.Cells["InitialRemSpeechTime"].Value) != -1000) && (row.Cells["RemainingSpeechTime"].Value != null && Convert.ToDecimal(row.Cells["RemainingSpeechTime"].Value) < 60000))
                    {
                        row.Cells["RemainingSpeechTime"].Style.BackColor = Color.Red;
                    }
                }
            }
        }

        
    }
}
