﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Threading;


namespace DcnSwSmd
{
    public static class Oscillator
    {
        /// <summary>
        /// 1 Hz recurring event
        /// </summary>
        public static event EventHandler OneHzTick
        {
            add
            {
                lock (eventLock)
                {
                    if ((null == oneHzTick) && (null == fourHzTick))
                    {
                        if (null != timer)
                        {
                            timer.Dispose();
                            timer = null;
                        }
                        timer = new Timer(
                            new TimerCallback(OnTimerTick),
                            null,
                            0,
                            INTERVAL);
                    }
                    oneHzTick += value;
                }
            }
            remove
            {
                lock (eventLock)
                {
                    oneHzTick -= value;
                    if ((null == oneHzTick) && (null == fourHzTick) && (null != timer))
                    {
                        timer.Dispose();
                        timer = null;
                    }
                }
            }
        }

        /// <summary>
        /// 4 Hz recurring event
        /// </summary>
        public static event EventHandler FourHzTick
        {
            add
            {
                lock (eventLock)
                {
                    if ((null == oneHzTick) && (null == fourHzTick))
                    {
                        if (null != timer)
                        {
                            timer.Dispose();
                            timer = null;
                        }

                        timer = new Timer(
                            new TimerCallback(OnTimerTick),
                            null,
                            0,
                            INTERVAL);
                    }
                    fourHzTick += value;
                }
            }
            remove
            {
                lock (eventLock)
                {
                    fourHzTick -= value;
                    if ((null == oneHzTick) && (null == fourHzTick) && (null != timer))
                    {
                        timer.Dispose();
                        timer = null;
                    }
                }
            }
        }

        /// <summary>
        /// Handle oscillator timer tick by generating the tick events at their required interval
        /// </summary>
        private static void OnTimerTick(object arg)
        {
            fourHzOccuranceCounter++;
            if (fourHzOccuranceCounter >= 4)
            {
                EventsHelper.FireEvent(oneHzTick, timer, new EventArgs());
                fourHzOccuranceCounter = 0;
            }

            EventsHelper.FireEvent(fourHzTick, timer, new EventArgs());
        }

        /// <summary>Semaphore to protect access to eventhandler lists</summary>
        private static readonly object eventLock = new object();

        /// <summary>Oscillation timer used to trigger oscillator event firing</summary>
        private static Timer timer;

        /// <summary>The interval in milliseconds of the highest frequency</summary>
        private const int INTERVAL = 250;

        /// <summary>Occurs at a frequency of 1 Hz</summary>
        private static event EventHandler oneHzTick;

        /// <summary>Occurs at a frequency of 4 Hz</summary>
        private static event EventHandler fourHzTick;

        /// <summary>
        /// Highest frequency occurance counter, used to create a oneHzTick evenry four 
        /// fourHzTick occurances
        /// </summary>
        private static int fourHzOccuranceCounter;
    }
}
