﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Bosch.Dcn.Ecpc.Server.Services.TraceListeners.Data;

namespace DcnSwSmd
{
    /// <summary>
    /// Class to parse the particiapnt Activity.
    /// </summary>
     public class ParticipantActivity : Activity
    {
        #region Constructors

         /// <summary>
         /// Constructor.
         /// </summary>
        public ParticipantActivity()
        {
            Topic = EnumTopic.Participant;
        }
         /// <summary>
         /// Constructor.
         /// </summary>
         /// <param name="participant"></param>
        public ParticipantActivity(ParticipantContainer participant)
            : this()
        {
            _participant = participant;
        }

        #endregion

        #region Properties
         /// <summary>
         /// Object to hold the Participant data.
         /// </summary>
        public ParticipantContainer Participant
        {
            get
            {
                return _participant;
            }
            set
            {
                _participant = value;
            }
        }

        #endregion

        #region Methods

        /// <summary>
        /// Get the string that contains the serialized data for this activity 
        /// as xml.
        /// </summary>
        /// <returns></returns>
        internal override string GetXmlString()
        {
            return SerializeObject<ParticipantActivity>(this);
        }

        /// <summary>
        /// Create a string that contains some specific information regarding 
        /// the activity.
        /// </summary>
        /// <returns>The activity string</returns>
        protected override string CreateActivityString()
        {
            StringBuilder stringToReturn = new StringBuilder();

            if (_participant.ParticipantData != null)
            {
                stringToReturn.AppendFormat(
                    " Name: {0} {1} {2} Present: {3} RemainingSpeechTime: {4} SpeechTimerOnHold: {5}",
                    _participant.ParticipantData.FirstName,
                    _participant.ParticipantData.MiddleName,
                    _participant.ParticipantData.LastName,
                    _participant.ParticipantData.Present,
                    _participant.ParticipantData.RemainingSpeechTime,
                    _participant.ParticipantData.SpeechTimerOnHold);
            }

            if ((_participant.Seat != null) &&
                (_participant.Seat.SeatData != null))
            {
                stringToReturn.AppendFormat(" Seat: {0}",
                    _participant.Seat.SeatData.Name);
            }

            return stringToReturn.ToString();
        }

        #endregion

        #region Fields
         
        private ParticipantContainer _participant;

        #endregion
    }
}
