using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Threading;
using System.Threading.Tasks;
using System.Net.Sockets;
using System.Net;
using System.Diagnostics;
using System.Xml.Serialization;
using System.IO;
using System.Xml;
using System.Diagnostics.CodeAnalysis;
using DcnSwSmd.TraceListeners.Data;
using System.Globalization;
using DcnSwSmd.Collections;
using System.Linq;

[module: SuppressMessage("Microsoft.Naming", "CA1704:IdentifiersShouldBeSpelledCorrectly", MessageId = "Dcn")]
[module: SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "Sw")]
[module: SuppressMessage("Microsoft.Naming", "CA1704:IdentifiersShouldBeSpelledCorrectly", MessageId = "Smd")]
[assembly: CLSCompliant(true)]

namespace DcnSwSmd
{
    /// <summary>
    /// Main form of DCN-SWSMD.
    /// </summary>
    public partial class SmdClient : Form, IDisposable
    {
        #region Methods

        /// <summary>
        /// Constructors
        /// </summary>
        public SmdClient()
        {
            InitializeComponent();
            microphoneStatusTimer = new System.Timers.Timer();
            votingTimeLimitTimer = new System.Timers.Timer();
            groupSpeechLimitTimer = new System.Timers.Timer();
            smdTimer = new System.Timers.Timer();
            connectionStatus = false;
            ContextMenu cmActivityLog = new System.Windows.Forms.ContextMenu();
            miCopy = new MenuItem();
            miCopy.Text = "Copy";
            miCopy.Click += new EventHandler(OnCopyClick);
            cmActivityLog.MenuItems.Add(miCopy);
            activityLogListView.ContextMenu = cmActivityLog;
            this.ShowInTaskbar = true;
            activityLogUpdater = new ActivityLogUpdater();
            textBoxPortNumber.Text = Properties.Settings.Default.SMDPort.ToString();
            

        }

        /// <summary>
        /// This is called when the connect button is clicked in the UI. The 
        /// connection with the server should be setup.
        /// </summary>
        private void connectButton_Click(object sender, EventArgs e)
        {
            
            #region Commented
           
            if (!connectionStatus)
            {
                #region Connect
                terminate.Reset();
                votingResults = new SMDVotingData();
                session = new SMDSessionData();
                discussionData = new SMDDiscussionData();
                particiapantInfo = new SMDParticipantData();
                indvVotingResults = new IndividualResults(votingResults);
                groupResults = new GroupResultsForm(votingResults);
                totalResults = new TotalVotingResultsForm(votingResults);
                microPhoneStatus = new MicrophoneStatusForm();
                currentAgenda = new AgendaForm();
                attendanceRegistration = new AttendanceRegistrationForm();
                groupSpeechTimerForm = new GroupSpeechTimerForm();

                microphoneStatusTimer = new System.Timers.Timer();
                smdTimer = new System.Timers.Timer();
                votingTimeLimitTimer = new System.Timers.Timer();
                //communicationThread = new Thread(new ThreadStart(CommunicationThreadStart));
                //communicationThread.Start();

                var mainTask = Task.Factory.StartNew(() =>
                {
                    CommunicationThreadStart();
                });
                microPhoneStatusLock = new Object();
                mainLock = new Object();
                indvVotingResults.FormClosing += new FormClosingEventHandler(indvVotingResults_FormClosing);
                microPhoneStatus.speakerBindingSource.DataSource = speakerList;
                microPhoneStatus.requestsBindingSource.DataSource = requestlist;

                
                #endregion
            }
            else
            {
                #region Disconnect
                terminate.Set();
                connectionStatus = false;
                UpdateUserInterfaceState();
                
                smdTimer.Elapsed -= SmdTimer_Tick;
                smdTimer.Stop();
                smdTimer.Dispose();

                microphoneStatusTimer.Elapsed -= MicrophoneStatusTimer_Tick;
                microphoneStatusTimer.Stop();
                microphoneStatusTimer.Dispose();

                votingTimeLimitTimer.Elapsed -= VotingTimer_Tick;
                votingTimeLimitTimer.Stop();
                votingTimeLimitTimer.Dispose();

                radioBtnAgenda.Checked = false;
                radioBtnAttendanceRegn.Checked = false;
                radioBtnGroupResults.Checked = false;
                radioBtnSpeakers.Checked = false;
                radioBtnTotalResults.Checked = false;
                radioBtnIndvResults.Checked = false;
                #endregion
            }
           
            #endregion
        }

        private void OnCopyClick(object sender, EventArgs e)
        {
            StringBuilder str = new StringBuilder();
            if (activityLogListView.SelectedItems.Count > 0)
            {
                foreach (ListViewItem o in activityLogListView.SelectedItems)
                {
                    str.AppendLine(o.Text);
                }
                Clipboard.Clear();
                Clipboard.SetText(str.ToString());
            }
        }
        private void UpdateUserInterfaceState()
        {
            if (!IsDisposed)
            {
                if (InvokeRequired)
                {
                    BeginInvoke(new MethodInvoker(UpdateUserInterfaceState));
                }
                else
                {
                    if (connectionStatus)
                    {
                        // When we are connected we will clear the listbox with all 
                        // the received activities.
                        activityLogListView.Items.Clear();
                        connectButton.Text = "Disconnect";
                        foreach (ColumnHeader column in activityLogListView.Columns)
                        {
                            column.Width = -1;
                        }
                        activityLogListView.Columns[0].Width = activityLogListView.Width - 5;
                        activityLogListView.Items.Add(DateTime.Now.ToShortDateString() + " " + DateTime.Now.ToShortTimeString() + " : " + "Connection established");
                        textBoxIpAddress.Enabled = false;
                        textBoxPortNumber.Enabled = false;
                    }
                    else
                    {
                        connectButton.Text = "Connect";
                        activityLogListView.Items.Add(DateTime.Now.ToShortDateString() + " " + DateTime.Now.ToShortTimeString() + " : " + "Connection Closed");
                        activityLogListView.AutoResizeColumn(0, ColumnHeaderAutoResizeStyle.ColumnContent);
                        activityLogListView.Columns[0].Width = activityLogListView.Width - 5;
                        if (indvVotingResults != null)
                        {
                            indvVotingResults.Dispose();
                            indvVotingResults = null;
                        }
                        if (votingResults != null)
                        {
                            votingResults = null;
                        }
                        textBoxIpAddress.Enabled = true;
                        textBoxPortNumber.Enabled = true;
                        ClearRequestList();
                        ClearSpeakersList();
                        ClearIndvResultsDataSource();
                        CloseAllScreens();
                    }
                }
            }
        }


        /// <summary>
        /// This method is responsible for the connection with the server and 
        /// to receive and handle the activities which are send by the server. 
        /// This runs on a separate thread.
        /// </summary>
        private void CommunicationThreadStart()
        {
            Socket clientSocket = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);
            try
            {
                System.Net.IPAddress address = null;
                IPEndPoint ipEndPoint;
                bool validIpaddress = false;
                var VotingInterimResultActivity = new object();
                foreach (IPAddress IPA in Dns.GetHostAddresses(textBoxIpAddress.Text))
                {
                    if (IPA.AddressFamily == AddressFamily.InterNetwork)
                    {
                        address = IPA;
                        validIpaddress = true;
                        break;
                    }
                }

                if (validIpaddress)
                {

                    ushort port = ushort.Parse(textBoxPortNumber.Text);
                    ipEndPoint = new IPEndPoint(address, port);

                    // Used to indicate that the connection with the server was 
                    // succesfully made.
                    bool success = true;

                    try
                    {
                        clientSocket.Connect(ipEndPoint);
                    }
                    catch (SocketException e)
                    {
                        success = false;

                        DisplayString(e.Message);
                    }

                    // Set the connected state based on the result of the previous 
                    // action.
                    connectionStatus = success;

                    if (connectionStatus)
                    {
                        if (textBoxPortNumber.Text != Properties.Settings.Default.SMDPort.ToString())
                        {
                            Properties.Settings.Default.SMDPort = ushort.Parse(textBoxPortNumber.Text);
                            Properties.Settings.Default.Save();
                        }

                        smdTimer.Interval = 1000;
                        smdTimer.Start();
                        smdTimer.Elapsed += SmdTimer_Tick;
                        UpdateUserInterfaceState();
                        Int32 numberOfReceivedActivities = 0;

                        while (!terminate.WaitOne(20, false))
                        {
                            if (clientSocket.Available > 8)
                            {
                                // First we read the header.
                                byte[] header = new byte[8];
                                int numberOfReceivedBytes = clientSocket.Receive(header, 8, 0);

                                Debug.Assert(numberOfReceivedBytes == 8, "Header receival problem");

                                EnumTopic topic = (EnumTopic)BitConverter.ToInt32(header, 0);
                                EnumActivityType activity = (EnumActivityType)BitConverter.ToInt32(header, 0);
                                Int32 messageLength = BitConverter.ToInt32(header, 4);

                                Debug.WriteLine(String.Format("Number of bytes to receive: {0}", messageLength));

                                byte[] buffer = new byte[messageLength];
                                byte[] receivedData = new byte[messageLength];

                                Int32 totalNumberOfBytesReceived = 0;

                                StringBuilder receivedString = new StringBuilder();

                                while (totalNumberOfBytesReceived < messageLength)
                                {
                                    numberOfReceivedBytes = clientSocket.Receive(buffer,
                                        messageLength - totalNumberOfBytesReceived, 0);

                                    // Add the received data to the buffer.
                                    Array.Copy(buffer, 0, receivedData,
                                        totalNumberOfBytesReceived, numberOfReceivedBytes);

                                    Debug.WriteLine(String.Format("Received {0} bytes {1} {2}",
                                        numberOfReceivedBytes, totalNumberOfBytesReceived, messageLength));

                                    totalNumberOfBytesReceived += numberOfReceivedBytes;
                                }

                                receivedString.Append(System.Text.Encoding.Unicode.GetString(receivedData));

                                numberOfReceivedActivities++;
                                activityLogUpdater.ReportActivity(() =>
                                    {
                                        activityLogListView.Items.Add("\n" + DateTime.Now.ToShortDateString() + " " + DateTime.Now.ToShortTimeString() + " : " + receivedString);
                                    });
                                switch (topic)
                                {

                                    #region Particiapant
                                    case EnumTopic.Participant:
                                        {
                                            ParticipantActivity participantActivity = DeserializeObject<ParticipantActivity>(receivedString.ToString());
                                            if (participantActivity.Type == EnumActivityType.ParticipantUpdated)
                                            {
                                                if (participantActivity != null && participantActivity.Participant != null)
                                                {

                                                    ///If admin or any other participant is who is not a participant of  meeting
                                                    ///gets added to the Speaker / request List in SMD, check for the  particpantActivity.Participant.Seat.SeatData.SeatType
                                                    /// OR 10508.

                                                    particiapantInfo.AddRowtoParticipantAttendanceDetails(participantActivity.Participant);
                                                    attendanceRegistration.LoadAttendanceRegistratioData(particiapantInfo);

                                                    if (participantActivity.Participant.Seat != null && participantActivity.Participant.ParticipantData != null)
                                                    {
                                                        UpdateMicrophoneStatusInSpeakersTable(participantActivity);
                                                    }
                                                }
                                            }
                                            if (microPhoneStatus != null)
                                            {
                                                microPhoneStatus.LoadMicrophoneStatusData(discussionData);
                                            }
                                           // DisplayString("\n" + DateTime.Now.ToShortDateString() + " " + DateTime.Now.ToShortTimeString() + " : " + receivedString);

                                        }; break;
                                    #endregion
                                    #region Voting
                                    case EnumTopic.Voting:
                                        {
                                            //DisplayString("\n" + DateTime.Now.ToShortDateString() + " " + DateTime.Now.ToShortTimeString() + " : " + receivedString);

                                            // TODO: Enable if the matching activity is added to the project.

                                            VotingActivity votingActivity = DeserializeObject<VotingActivity>(receivedString.ToString());

                                            if (votingActivity != null)
                                            {
                                                if (votingActivity.Type == EnumActivityType.VotingStarted && votingActivity.Voting.VotingData != null)
                                                {
                                                    if (votingActivity.Voting.VotingData.RemainingVotingTime >= 1)
                                                    {
                                                        votingTimeLimitTimer.Interval = 1000;
                                                        votingTimeLimitTimer.Start();
                                                        votingLimitCurrentTime = DateTime.Now;
                                                        votingTimeLimitTimer.Elapsed += VotingTimer_Tick;
                                                    }
                                                    activeVotingId = votingActivity.Voting.Id;
                                                    votingStopped = false;
                                                    votingResults = new SMDVotingData();
                                                    votingResults.VotingData = votingActivity.Voting.VotingData;
                                                    ClearPreviousResults();
                                                    indvVotingResults.UpdateStaticData(votingResults);

                                                    groupResults.UpdateHeaderData(votingResults);
                                                    groupResults.ClearPreviousResults();
                                                    totalResults.UpdateHeaderData(votingResults);
                                                        
                                                    if (votingActivity.Voting.Answers != null && votingActivity.Voting.Answers.Count > 0)
                                                    {
                                                        votingResults.Answers = votingActivity.Voting.Answers;
                                                        //indvVotingResults.CreateDtIntermediateTable();
                                                        ClearPreviousResults();
                                                        groupResults.CreateDtIntermediateTable();
                                                        totalResults.LoadVotingDataOnVotingStart(votingResults, votingStopped, 0);
                                                        indvVotingResults.UpdateStaticData(votingResults);
                                                    }
                                                }

                                                else if (votingActivity.Type == EnumActivityType.VotingInterimResult)
                                                {
                                                    if (!votingStopped)
                                                    {
                                                        votingResults.IndividualResults = votingActivity.Voting.VotingResults.IndividualResults;
                                                        votingResults.TotalResults = votingActivity.Voting.VotingResults.VotingTotalResults;
                                                        if (indvVotingResults != null)
                                                        {
                                                            if (votingResults.VotingData != null)
                                                            {
                                                                VotingInterimResultActivity = Task.Factory.StartNew(() =>
                                                                    {
                                                                        Task.Factory.StartNew(() =>
                                                                            {
                                                                                AddOrUpdateInterimResults(votingResults);
                                                                                indvVotingResults.LoadVotingData(votingResults, false);
                                                                                UpdatedDataSourceForIndvResults();
                                                                            });
                                                                        Task.Factory.StartNew(() =>
                                                                            {
                                                                                groupResults.AddOrUpdateInterimResults(votingResults);
                                                                                groupResults.LoadVotingData(votingResults, false);
                                                                            });

                                                                        Task.Factory.StartNew(() =>
                                                                            {
                                                                                totalResults.LoadVotingData(votingResults, votingStopped, interMediateResults);
                                                                            });
                                                                        
                                                                        interMediateResults++;
                                                                    });
                                                            }
                                                            else
                                                            {
                                                                DisplayString("\n" + DateTime.Now.ToShortDateString() + " " + DateTime.Now.ToShortTimeString() + " : Voting Interim Result Activity: Voting metadata not available");
                                                            }
                                                            
                                                        }
                                                    }
                                                }

                                                else if (votingActivity.Type == EnumActivityType.VotingStopped)
                                                {
                                                    votingStopped = true;
                                                    votingTimeLimitTimer.Stop();
                                                    if (votingActivity.Voting.VotingResults != null)
                                                    {
                                                        if (votingActivity.Voting.VotingResults.IndividualResults != null && votingActivity.Voting.VotingResults.IndividualResults.Count > 0)
                                                        {
                                                            votingResults.IndividualResults = votingActivity.Voting.VotingResults.IndividualResults;
                                                        }
                                                    }

                                                    if (votingActivity.Voting.VotingResults != null)
                                                    {
                                                        if (votingActivity.Voting.VotingResults.VotingTotalResults != null && votingActivity.Voting.VotingResults.VotingTotalResults.VotingAnswerResults.Count > 0)
                                                        {
                                                            votingResults.TotalResults = votingActivity.Voting.VotingResults.VotingTotalResults;
                                                        }
                                                    }
                                                    if (votingResults != null)
                                                    {
                                                        if (votingResults.VotingData != null)
                                                        {
                                                            AddOrUpdateInterimResults(votingResults);
                                                            votingResults.VotingData.RemainingVotingTime = 0;
                                                            indvVotingResults.UpdateRemainingVotingTime(votingResults.VotingData.RemainingVotingTime);
                                                            totalResults.UpdateRemainingVotingTime(votingResults);
                                                            groupResults.UpdateRemainingVotingTime(votingResults);

                                                            indvVotingResults.LoadVotingData(votingResults, votingStopped);
                                                            UpdatedDataSourceForIndvResults();
                                                            groupResults.LoadVotingData(votingResults, votingStopped);
                                                            if (votingResults.IndividualResults != null)
                                                            {
                                                                interMediateResults = votingResults.IndividualResults.Count;
                                                            }
                                                            totalResults.LoadVotingData(votingResults, votingStopped, interMediateResults);
                                                        }
                                                        else
                                                        {
                                                            DisplayString("\n" + DateTime.Now.ToShortDateString() + " " + DateTime.Now.ToShortTimeString() + " : Voting Stopped Activity: Voting metadata not available");
                                                        }
                                                    }
                                                }
                                                else if (votingActivity.Type == EnumActivityType.VotingOnHold)
                                                {
                                                    votingHold = true;
                                                    votingTimeLimitTimer.Elapsed -= VotingTimer_Tick;
                                                    votingTimeLimitTimer.Stop();
                                                    if (votingActivity.Voting.VotingResults != null)
                                                    {
                                                        if (votingActivity.Voting.VotingResults.IndividualResults != null && votingActivity.Voting.VotingResults.IndividualResults.Count > 0)
                                                        {
                                                            votingResults.IndividualResults = votingActivity.Voting.VotingResults.IndividualResults;
                                                        }
                                                    }
                                                                                                        
                                                    if (votingActivity.Voting.VotingResults != null)
                                                    {
                                                        if (votingActivity.Voting.VotingResults.VotingTotalResults != null && votingActivity.Voting.VotingResults.VotingTotalResults.VotingAnswerResults.Count > 0)
                                                        {
                                                            votingResults.TotalResults = votingActivity.Voting.VotingResults.VotingTotalResults;
                                                        }
                                                    }
                                                    if (votingResults != null)
                                                    {
                                                        if (votingResults.VotingData != null)
                                                        {
                                                            AddOrUpdateInterimResults(votingResults);
                                                            groupResults.AddOrUpdateInterimResults(votingResults);
                                                            votingResults.VotingData.RemainingVotingTime = 0;
                                                            indvVotingResults.UpdateRemainingVotingTime(votingResults.VotingData.RemainingVotingTime);
                                                            totalResults.UpdateRemainingVotingTime(votingResults);
                                                            groupResults.UpdateRemainingVotingTime(votingResults);

                                                            indvVotingResults.LoadVotingData(votingResults, false);
                                                            UpdatedDataSourceForIndvResults();
                                                            groupResults.LoadVotingData(votingResults, votingStopped);
                                                            if (votingResults.IndividualResults != null)
                                                            {
                                                                interMediateResults = votingResults.IndividualResults.Count;
                                                            }
                                                            totalResults.LoadVotingData(votingResults, votingHold, interMediateResults);
                                                        }
                                                        else
                                                        {
                                                            DisplayString("\n" + DateTime.Now.ToShortDateString() + " " + DateTime.Now.ToShortTimeString() + " : Voting OnHold Activity: Voting metadata not available");
                                                        }
                                                    }
                                                }
                                                else if (votingActivity.Type == EnumActivityType.VotingResumed)
                                                {
                                                    if (votingResults != null && votingResults.VotingData != null)
                                                    {
                                                        votingResults.VotingData.RemainingVotingTime = votingActivity.Voting.VotingData.RemainingVotingTime;
                                                        votingLimitCurrentTime = DateTime.Now;
                                                        votingTimeLimitTimer.Elapsed += VotingTimer_Tick;
                                                        votingTimeLimitTimer.Start();
                                                    }
                                                    else
                                                    {
                                                        DisplayString("\n" + DateTime.Now.ToShortDateString() + " " + DateTime.Now.ToShortTimeString() + " : Voting Resumed Activity: Voting metadata not available");
                                                    }
                                                }
                                                else if (votingActivity.Type == EnumActivityType.VotingDataUpdated)
                                                {
                                                    if (votingResults != null && votingResults.VotingData != null)
                                                    {
                                                        if (activeVotingId == votingActivity.Voting.Id)
                                                        {
                                                            votingResults.VotingData = votingActivity.Voting.VotingData;
                                                            indvVotingResults.UpdateStaticData(votingResults);
                                                            groupResults.UpdateHeaderData(votingResults);
                                                            totalResults.UpdateHeaderData(votingResults);
                                                        }
                                                    }
                                                    else
                                                    {
                                                        DisplayString("\n" + DateTime.Now.ToShortDateString() + " " + DateTime.Now.ToShortTimeString() + " : Voting Data Updated Activity: Voting metadata not available");
                                                    }
                                                }
                                            }
                                        }; break;
                                    #endregion

                                    #region Session
                                    case EnumTopic.Session:
                                        {
                                            //DisplayString("\n" + DateTime.Now.ToShortDateString() + " " + DateTime.Now.ToShortTimeString() + " : " + receivedString);
                                            SessionActivity sessionActivity = DeserializeObject<SessionActivity>(receivedString.ToString());
                                            if (sessionActivity != null)
                                            {
                                                if (sessionActivity.Type == EnumActivityType.SessionStarted)
                                                {
                                                    votingResults = new SMDVotingData();
                                                    activeSessionId = sessionActivity.Session.Id;
                                                    if (sessionActivity.Session.SessionData != null)
                                                    {
                                                        session.SessionData = sessionActivity.Session.SessionData;
                                                        currentAgenda.LoadAgendaData(session);
                                                        microPhoneStatus.LoadMicrophoneStatusData(null);
                                                        UpdateGroupSpeechTimeListOnSessionStart(sessionActivity);
                                                    }
                                                }
                                                else if (sessionActivity.Type == EnumActivityType.SessionStopped)
                                                {
                                                    //ClearRequestList();
                                                    ClearSpeakersList();
                                                    ClearGroupSpeechTimes();
                                                    currentAgenda.LoadDefaultAgendaData();
                                                    microPhoneStatus.LoadMicrophoneStatusData(null);
                                                }
                                                else if (sessionActivity.Type == EnumActivityType.SessionDataUpdated)
                                                {
                                                    if (sessionActivity.Session.Id == activeSessionId)
                                                    {
                                                        session.SessionData = sessionActivity.Session.SessionData;
                                                        currentAgenda.LoadAgendaData(session);
                                                    }
                                                }
                                            }
                                        }; break;
                                    #endregion

                                    #region System
                                    case EnumTopic.System:
                                        {
                                            //// A system activity is received, so we 
                                            //// should deserialize it from the received 
                                            //// string.
                                            SystemActivity systemActivity = DeserializeObject<SystemActivity>(receivedString.ToString());
                                            if (systemActivity.Type == EnumActivityType.SystemStopped)
                                            {
                                                terminate.Set();
                                                connectionStatus = false;
                                                UpdateUserInterfaceState();
                                            }
                                           // DisplayString("\n" + DateTime.Now.ToShortDateString() + " " + DateTime.Now.ToShortTimeString() + " : " + receivedString);
                                        }; break;
                                    #endregion
                                    #region Meeting
                                    case EnumTopic.Meeting:
                                        {
                                            MeetingActivity meetingActivity = DeserializeObject<MeetingActivity>(receivedString.ToString());
                                            if (meetingActivity.Type == EnumActivityType.MeetingStopped)
                                            {
                                                /// Added to fix the issue where the Total resuts of the previous voting was getting displayed in the following scenario:
                                                /// Start meeting - start voting - Vote - Stop meeting-Start meeting-start voting and Press F2.
                                                   
                                                ClearSpeakersList();
                                                ClearRequestList();
                                                ClearIndvResultsDataSource();
                                                votingStopped = true;

                                                currentAgenda.LoadDefaultAgendaData();
                                            }
                                            else if (meetingActivity.Type == EnumActivityType.MeetingStarted)
                                            {
                                                votingResults = new SMDVotingData();
                                                particiapantInfo.ParticipantApptendanceDetails.Clear();
                                            }
                                            //DisplayString("\n" + DateTime.Now.ToShortDateString() + " " + DateTime.Now.ToShortTimeString() + " : " + receivedString);
                                        }; break;
                                    #endregion;
                                    #region Seat
                                    case EnumTopic.Seat:
                                        {
                                            SeatActivity seatActivity = DeserializeObject<SeatActivity>(receivedString.ToString());
                                            if (seatActivity.Type == EnumActivityType.SeatUpdated)
                                            {
                                                if (seatActivity.Seat != null && seatActivity.Seat.SeatData != null)
                                                {
                                                    if (seatActivity.Seat.SeatData.SeatType == "Chairman")
                                                    {
                                                        if (seatActivity.Seat.SeatData.MicrophoneActive)
                                                        {
                                                            AddChairmanOrNoteBookerToSpeakerTable(seatActivity);
                                                        }
                                                        else
                                                        {
                                                            RemoveCharimanMicPhoneStatus(seatActivity.Seat.Id);
                                                        }
                                                    }
                                                    else if (seatActivity.Seat.SeatData.SeatType == "Delegate")
                                                    {
                                                        if (seatActivity.Seat.SeatData.IsSpecialStation)
                                                        {
                                                            if (seatActivity.Seat.SeatData.MicrophoneActive)
                                                            {
                                                                AddChairmanOrNoteBookerToSpeakerTable(seatActivity);
                                                            }
                                                            else
                                                            {
                                                                RemoveCharimanMicPhoneStatus(seatActivity.Seat.Id);
                                                            }
                                                            
                                                        }
                                                        else if (!seatActivity.Seat.SeatData.IsSpecialStation)
                                                        {
                                                            bool microphoneActiveAndRunning = false;
                                                            if (!seatActivity.Seat.Participant.ParticipantData.SpeechTimerOnHold && seatActivity.Seat.SeatData.MicrophoneActive)
                                                            {
                                                                microphoneActiveAndRunning = true;
                                                            }

                                                            UpdateMicrophoneStatusInSpeakersTable(seatActivity.Seat.Id, microphoneActiveAndRunning, seatActivity.Seat.Participant.ParticipantData.RemainingSpeechTime);
                                                            UpdateGroupSpeechListOnSeatActivityMicOnOff(seatActivity);
                                                        }
                                                        
                                                    }
                                                }
                                                if (microPhoneStatus != null)
                                                {
                                                    microPhoneStatus.LoadMicrophoneStatusData(discussionData);
                                                }
                                            }
                                           // DisplayString("\n" + DateTime.Now.ToShortDateString() + " " + DateTime.Now.ToShortTimeString() + " : " + receivedString);
                                        }; break;
                                    #endregion
                                    #region Discussion
                                    case EnumTopic.Discussion:
                                        {
                                            DiscussionActivity discussionActivity = DeserializeObject<DiscussionActivity>(receivedString.ToString());
                                            //DisplayString("\n" + DateTime.Now.ToShortDateString() + " " + DateTime.Now.ToShortTimeString() + " : " + receivedString);
                                            if (discussionActivity != null)
                                            {
                                                    
                                                    
                                                if (discussionActivity.Type == EnumActivityType.ActiveListUpdated || discussionActivity.Type == EnumActivityType.ActiveResponseListUpdated)
                                                {
                                                    if (!microphoneStatusTimer.Enabled)
                                                    {
                                                        microphoneStatusTimer.Enabled = true;
                                                        microphoneStatusTimer.Interval = 1000;
                                                        microphoneStatusTimer.AutoReset = true;
                                                        microphoneStatusTimer.Start();
                                                        microphoneStatusCurrentTime = DateTime.Now;
                                                        Oscillator.OneHzTick += new EventHandler(MicrophoneStatusTimer_Tick);

                                                        groupSpeechLimitTimer.Enabled = true;
                                                        groupSpeechLimitTimer.Interval = 1000;
                                                        groupSpeechLimitTimer.AutoReset = true;
                                                        groupSpeechLimitTimer.Start();
                                                        groupSpeechCurrentTime = DateTime.Now;
                                                        groupSpeechLimitTimer.Elapsed += GroupSpeechTimer_Tick;
                                                        // microphoneStatusTimer.Elapsed += MicrophoneStatusTimer_Tick;
                                                            
                                                        votingTimeLimitTimer.Interval = 1000;
                                                        votingTimeLimitTimer.Start();
                                                        votingLimitCurrentTime = DateTime.Now;

                                                            
                                                    }
                                                }

                                                if (discussionActivity.Type == EnumActivityType.ActiveListUpdated && discussionActivity.Discussion.ActiveList != null)
                                                {
                                                    discussionData.ActiveList = discussionActivity.Discussion.ActiveList;
                                                    UpdateMicrophoneSpeakerDataTable(discussionActivity.Type, true);
                                                }

                                                if (discussionActivity.Discussion.RequestList != null)
                                                {
                                                    discussionData.RequestList = discussionActivity.Discussion.RequestList;
                                                    UpdateMicrophoneRequestList(false);
                                                }
                                                if (discussionActivity.Discussion.SpecialEquipmentList != null)
                                                {
                                                    discussionData.SpecialEquipment = discussionActivity.Discussion.SpecialEquipmentList;
                                                    UpdateChairmanToSpeakerDataTable();
                                                }

                                                if (discussionActivity.Discussion.ActiveResponseList != null)
                                                {
                                                    discussionData.ActiveResponseList = discussionActivity.Discussion.ActiveResponseList;
                                                    UpdateMicrophoneSpeakerDataTable(discussionActivity.Type, false);
                                                }
                                                if (microPhoneStatus != null && discussionActivity.Discussion.ResponseList != null)
                                                {
                                                    discussionData.ResponseList = discussionActivity.Discussion.ResponseList;
                                                    UpdateMicrophoneRequestList(true);
                                                }
                                                if (microPhoneStatus != null)
                                                {
                                                    microPhoneStatus.LoadMicrophoneStatusData(discussionData);
                                                }
                                            }
                                        }; break;
                                    #endregion
                                    #region Desk
                                    case EnumTopic.Desk:
                                        {

                                        }; break;
                                    #endregion
                                    #region Interpretation
                                    case EnumTopic.Interpretation:
                                        {

                                        }; break;
                                    #endregion
                                    #region Booth
                                    case EnumTopic.Booth:
                                        {

                                        }; break;
                                    #endregion
                                    #region ServiceCall
                                    case EnumTopic.ServiceCall:
                                        {

                                        }; break;
                                    #endregion
                                    #region TestSystem
                                    case EnumTopic.TestSystem:
                                        {

                                        }; break;
                                    #endregion
                                    #region Default
                                    default:
                                        {

                                        }; break;
                                    #endregion

                                }
                            }
                        }
                    }
                }
                else
                {
                    DisplayString("An invalid IP address was entered.");
                    activityLogListView.Columns[0].Width = activityLogListView.Width - 5;
                }
            }
            catch (SocketException e)
            {
                DisplayString("Exception occurred: " + e.StackTrace.ToString());
            }
            catch (Exception e)
            {
                DisplayString("Exception occurred: " + e.StackTrace.ToString());
            }
            finally
            {
                connectionStatus = false;
                clientSocket.Close();
            }

        }
        /// <summary>
        /// Reconstruct an object from an XML string.
        /// </summary>
        /// <param name="xml">The string that contains the xml</param>
        /// <returns>The object that was deserialized from the passed xml 
        /// string</returns>
        public T DeserializeObject<T>(string xml)
        {
            XmlSerializer xs = new XmlSerializer(typeof(T));
            MemoryStream memoryStream = new MemoryStream(StringToUTF8ByteArray(xml));
            XmlTextWriter xmlTextWriter = new XmlTextWriter(memoryStream, Encoding.UTF8);
            return (T)xs.Deserialize(memoryStream);
        }

        /// <summary>
        /// Display the passed activity in the user interface.
        /// </summary>
        /// <param name="activity">The activity to display</param>
        private void DisplayActivity(Activity activity)
        {
            DisplayString(DateTime.Now.ToShortDateString() + " " + activity.ToString());
        }

        /// <summary>
        /// Converts the String to UTF8 Byte array which is used in the 
        /// serialization.
        /// </summary>
        /// <param name="pXmlString">The string that needs to be converted</param>
        /// <returns>The bytearray</returns>
        private static Byte[] StringToUTF8ByteArray(string pXmlString)
        {
            UTF8Encoding encoding = new UTF8Encoding();
            byte[] byteArray = encoding.GetBytes(pXmlString);
            return byteArray;
        }
        /// <summary>
        /// Display a string in the listbox represented in the user interface.
        /// Execution is transferred on the UI thread if necessary.
        /// </summary>
        /// <param name="log">The string that we need to display</param>
        private void DisplayString(string log)
        {
            try
            {
                if (InvokeRequired)
                {
                    BeginInvoke(new DisplayStringInvoker(DisplayString), log);
                }
                else
                {
                    // Add the item to the list ...
                    activityLogListView.Items.Add(log);
                    activityLogListView.AutoResizeColumn(0, ColumnHeaderAutoResizeStyle.ColumnContent);
                    // ... and select it.
                }
            }
            catch (Exception ex)
            {
                DisplayString(ex.StackTrace.ToString());
            }
        }

        /// <summary>
        /// The client is being closed, so let's terminate the connection.
        /// </summary>
        protected override void OnClosing(CancelEventArgs e)
        {
            //stop the connection.
            terminate.Set();
            base.OnClosing(e);
        }

        private delegate void EnableDisbleControls(ToolStripMenuItem c, bool b);

        private void EnableDisableControl(ToolStripMenuItem cntr, bool value)
        {
            if (!IsDisposed)
            {
                if (this.InvokeRequired)
                {
                    BeginInvoke(new EnableDisbleControls(EnableDisableControl), cntr, value);
                }
                else
                {
                    cntr.Enabled = value;
                }
            }
        }

        private void btnClearLog_Click(object sender, EventArgs e)
        {
            if (activityLogListView.Items.Count > 0)
            {
                activityLogListView.Items.Clear();
            }
            else
            {
                MessageBox.Show("There are no activity logs to clear");
            }
        }

        
        private delegate void ShowFormDelegate();
        /// <summary>
        /// Function to Display the IndividualVoting Results.
        /// </summary>
        private void ShowIndividualVotingResults()
        {
            if (this.InvokeRequired)
            {
                this.BeginInvoke(new ShowFormDelegate(ShowIndividualVotingResults));
            }
            else
            {
                if (votingStopped)
                {
                    if (indvVotingResults != null)
                    {
                        indvVotingResults.Dispose();
                        indvVotingResults = null;
                    }
                    if (connectionStatus)
                    {
                        FindScreens();
                        #region CloseForms
                        groupResults.Close();
                        totalResults.Close();
                        microPhoneStatus.Close();
                        currentAgenda.Close();
                        attendanceRegistration.Close();
                        groupSpeechTimerForm.Close();
                        #endregion
                        if (votingResults != null && votingResults.IndividualResults != null)
                        {
                            indvVotingResults = new IndividualResults(votingResults);
                            if (resultDisplayScreen.Length > 1)
                            {
                                indvVotingResults.Left = resultDisplayScreen[1].WorkingArea.Left;
                                indvVotingResults.Top = resultDisplayScreen[1].WorkingArea.Top;
                                indvVotingResults.StartPosition = FormStartPosition.Manual;
                            }
                            indvVotingResults.UpdateStaticData(votingResults);
                            indvVotingResults.Show();
                            indvVotingResults.LoadVotingData(votingResults, votingStopped);
                            UpdatedDataSourceForIndvResults();   
                        }
                        else
                        {
                            if (votingResults == null)
                            {
                                DisplayString(DateTime.Now.ToShortDateString() + " " + DateTime.Now.ToShortTimeString() + " : " + "Individual voting reuslt doesnot have any meeting data.");
                                indvVotingResults = new IndividualResults(null);
                                if (resultDisplayScreen.Length > 1)
                                {
                                    indvVotingResults.Left = resultDisplayScreen[1].WorkingArea.Left;
                                    indvVotingResults.Top = resultDisplayScreen[1].WorkingArea.Top;
                                    indvVotingResults.StartPosition = FormStartPosition.Manual;
                                }
                                indvVotingResults.UpdateStaticData(votingResults);
                                indvVotingResults.Show();
                            }
                            else
                            {
                                indvVotingResults = new IndividualResults(votingResults);
                                if (resultDisplayScreen.Length > 1)
                                {
                                    indvVotingResults.Left = resultDisplayScreen[1].WorkingArea.Left;
                                    indvVotingResults.Top = resultDisplayScreen[1].WorkingArea.Top;
                                    indvVotingResults.StartPosition = FormStartPosition.Manual;
                                }
                                indvVotingResults.UpdateStaticData(votingResults);
                                indvVotingResults.Show();
                                
                            }
                        }
                    }
                    else
                    {
                        if (votingResults != null)
                        {
                            votingResults = null;
                        }
                        DisplayString(DateTime.Now.ToShortDateString() + " " + DateTime.Now.ToShortTimeString() + " : " + "Connection not establishied with the DCN-SW server.");
                    }
                }
                else
                {
                    DataTable dtTempIndvResults = new DataTable();
                    if (indvVotingResults != null)
                    {
                        //if (indvVotingResults.DtIntermediateIndvResults != null)
                        //{
                        //    dtTempIndvResults = indvVotingResults.DtIntermediateIndvResults;
                        //}

                        //if (indvVotingResults.IndvResultsDataSource != null)
                        //{
                        //    indvResultsDataSource = indvVotingResults.IndvResultsDataSource;
                        //}
                        indvVotingResults.Dispose();

                    }
                    if (connectionStatus)
                    {
                        FindScreens();
                        #region CloseForms
                        groupResults.Close();
                        totalResults.Close();
                        microPhoneStatus.Close();
                        currentAgenda.Close();
                        attendanceRegistration.Close();
                        // Have to include group speech timers form.
                        #endregion
                        if (votingResults != null && votingResults.IndividualResults != null)
                        {
                            if (indvVotingResults.IsDisposed)
                            {
                                indvVotingResults = new IndividualResults(votingResults);
                                indvVotingResults.DtIntermediateIndvResults = dtTempIndvResults;
                                UpdatedDataSourceForIndvResults(); 
                            }
                            if (resultDisplayScreen.Length > 1)
                            {
                                indvVotingResults.Left = resultDisplayScreen[1].WorkingArea.Left;
                                indvVotingResults.Top = resultDisplayScreen[1].WorkingArea.Top;
                                indvVotingResults.StartPosition = FormStartPosition.Manual;
                            }
                            indvVotingResults.UpdateStaticData(votingResults);
                            indvVotingResults.Show();
                            indvVotingResults.LoadVotingData(votingResults, votingStopped);
                            UpdatedDataSourceForIndvResults();
                            indvVotingResults.UpdateGridViewImage();
                        }
                        else
                        {
                            if (votingResults == null)
                            {
                                DisplayString(DateTime.Now.ToShortDateString() + " " + DateTime.Now.ToShortTimeString() + " : " + "Individual voting reuslt doesnot have any meeting data.");
                                indvVotingResults = new IndividualResults(null);
                                if (resultDisplayScreen.Length > 1)
                                {
                                    indvVotingResults.Left = resultDisplayScreen[1].WorkingArea.Left;
                                    indvVotingResults.Top = resultDisplayScreen[1].WorkingArea.Top;
                                    indvVotingResults.StartPosition = FormStartPosition.Manual;
                                }
                                indvVotingResults.UpdateStaticData(votingResults);
                                indvVotingResults.Show();
                            }
                            else
                            {
                                indvVotingResults = new IndividualResults(votingResults);
                                if (resultDisplayScreen.Length > 1)
                                {
                                    indvVotingResults.Left = resultDisplayScreen[1].WorkingArea.Left;
                                    indvVotingResults.Top = resultDisplayScreen[1].WorkingArea.Top;
                                    indvVotingResults.StartPosition = FormStartPosition.Manual;
                                }
                                indvVotingResults.UpdateStaticData(votingResults);
                                indvVotingResults.Show();
                            }
                        }
                    }
                    else
                    {
                        if (votingResults != null)
                        {
                            votingResults = null;
                        }
                        DisplayString(DateTime.Now.ToShortDateString() + " " + DateTime.Now.ToShortTimeString() + " : " + "Connection not establishied with the DCN-SW server.");
                    }
                }
            }
        }

        /// <summary>
        /// Function to Display the Group Voting Results.
        /// </summary>
        private void ShowGroupVotingResuts()
        {
            if (this.InvokeRequired)
            {
                this.BeginInvoke(new ShowFormDelegate(ShowGroupVotingResuts));
            }
            else
            {
                if (votingStopped)
                {
                    if (groupResults != null)
                    {
                        groupResults.Dispose();
                        groupResults = null;
                    }
                    if (connectionStatus)
                    {
                        FindScreens();
                        #region CloseForms
                        indvVotingResults.Close();
                        totalResults.Close();
                        microPhoneStatus.Close();
                        currentAgenda.Close();
                        attendanceRegistration.Close();
                        groupSpeechTimerForm.Close();
                        #endregion
                        if (votingResults != null)
                        {

                            if (votingResults.IndividualResults != null)
                            {
                                groupResults = new GroupResultsForm(votingResults);
                                if (resultDisplayScreen.Length > 1)
                                {
                                    groupResults.Left = resultDisplayScreen[1].WorkingArea.Left;
                                    groupResults.Top = resultDisplayScreen[1].WorkingArea.Top;
                                    groupResults.StartPosition = FormStartPosition.Manual;
                                }
                                groupResults.Show();
                                groupResults.LoadVotingData(votingResults, votingStopped);
                            }
                            else
                            {
                                if (votingResults == null)
                                {
                                    DisplayString(DateTime.Now.ToShortDateString() + " " + DateTime.Now.ToShortTimeString() + " : " + "Group voting reuslt doesnot have any meeting data.");
                                    groupResults = new GroupResultsForm(null);
                                    if (resultDisplayScreen.Length > 1)
                                    {
                                        groupResults.Left = resultDisplayScreen[1].WorkingArea.Left;
                                        groupResults.Top = resultDisplayScreen[1].WorkingArea.Top;
                                        groupResults.StartPosition = FormStartPosition.Manual;
                                    }
                                    groupResults.Show();
                                }
                                else
                                {
                                    groupResults = new GroupResultsForm(votingResults);
                                    if (resultDisplayScreen.Length > 1)
                                    {
                                        groupResults.Left = resultDisplayScreen[1].WorkingArea.Left;
                                        groupResults.Top = resultDisplayScreen[1].WorkingArea.Top;
                                        groupResults.StartPosition = FormStartPosition.Manual;
                                    }
                                    groupResults.UpdateHeaderData(votingResults);
                                    groupResults.Show();
                                }
                            }
                        }
                        else
                        {
                            DisplayString(DateTime.Now.ToShortDateString() + " " + DateTime.Now.ToShortTimeString() + " : " + "Voting Results object is null");
                        }
                    }
                    else
                    {
                        if (votingResults != null)
                        {
                            votingResults = null;
                        }
                        DisplayString(DateTime.Now.ToShortDateString() + " " + DateTime.Now.ToShortTimeString() + " : " + "Connection not establishied with the DCN-SW server.");
                    }
                }
                else
                {
                    DataTable dtTempIndvResults = new DataTable();
                    if (groupResults != null)
                    {
                        if (groupResults.DtIntermediateIndvResults != null)
                        {
                            dtTempIndvResults = groupResults.DtIntermediateIndvResults;
                        }
                        groupResults.Dispose();
                    }
                    if (connectionStatus)
                    {
                        FindScreens();
                        #region CloseForms
                        indvVotingResults.Close();
                        totalResults.Close();
                        microPhoneStatus.Close();
                        currentAgenda.Close();
                        attendanceRegistration.Close();
                        // Have to include group speech timers form.
                        #endregion
                        if (votingResults != null && votingResults.IndividualResults != null)
                        {
                            if (groupResults.IsDisposed)
                            {
                                groupResults = new GroupResultsForm(votingResults);
                                groupResults.DtIntermediateIndvResults = dtTempIndvResults;
                            }
                            if (resultDisplayScreen.Length > 1)
                            {
                                groupResults.Left = resultDisplayScreen[1].WorkingArea.Left;
                                groupResults.Top = resultDisplayScreen[1].WorkingArea.Top;
                                groupResults.StartPosition = FormStartPosition.Manual;
                            }
                            groupResults.Show();
                            groupResults.LoadVotingData(votingResults, votingStopped);
                        }
                        else
                        {
                            if (votingResults == null)
                            {
                                DisplayString(DateTime.Now.ToShortDateString() + " " + DateTime.Now.ToShortTimeString() + " : " + "Group voting reuslt doesnot have any meeting data.");
                                groupResults = new GroupResultsForm(null);
                                if (resultDisplayScreen.Length > 1)
                                {
                                    groupResults.Left = resultDisplayScreen[1].WorkingArea.Left;
                                    groupResults.Top = resultDisplayScreen[1].WorkingArea.Top;
                                    groupResults.StartPosition = FormStartPosition.Manual;
                                }
                                groupResults.Show();
                            }
                            else
                            {
                                groupResults = new GroupResultsForm(votingResults);
                                if (resultDisplayScreen.Length > 1)
                                {
                                    groupResults.Left = resultDisplayScreen[1].WorkingArea.Left;
                                    groupResults.Top = resultDisplayScreen[1].WorkingArea.Top;
                                    groupResults.StartPosition = FormStartPosition.Manual;
                                }
                                groupResults.UpdateHeaderData(votingResults);
                                groupResults.Show();
                               
                            }
                        }
                    }
                    else
                    {
                        if (votingResults != null)
                        {
                            votingResults = null;
                        }
                        DisplayString(DateTime.Now.ToShortDateString() + " " + DateTime.Now.ToShortTimeString() + " : " + "Connection not establishied with the DCN-SW server.");
                    }
                }
            }
        }

        /// <summary>
        /// Function to Displat the Total Voting Results.
        /// </summary>
        private void ShowTotalVotingResults()
        {
            if (this.InvokeRequired)
            {
                this.BeginInvoke(new ShowFormDelegate(ShowTotalVotingResults));
            }
            else
            {

                if (totalResults != null)
                {
                    totalResults.Dispose();
                    totalResults = null;
                }

                if (connectionStatus)
                {
                    FindScreens();
                    #region CloseForms
                    indvVotingResults.Close();
                    groupResults.Close();
                    microPhoneStatus.Close();
                    currentAgenda.Close();
                    attendanceRegistration.Close();
                    groupSpeechTimerForm.Close();
                    #endregion
                    if (votingResults != null && votingResults.TotalResults != null)
                    {
                        totalResults = new TotalVotingResultsForm(votingResults);
                        if (resultDisplayScreen.Length > 1)
                        {
                            totalResults.Left = resultDisplayScreen[1].WorkingArea.Left;
                            totalResults.Top = resultDisplayScreen[1].WorkingArea.Top;
                            totalResults.StartPosition = FormStartPosition.Manual;
                        }
                        totalResults.Show();
                        totalResults.LoadVotingData(votingResults, votingStopped, interMediateResults);
                    }
                    else
                    {
                        if (votingResults == null)
                        {
                            DisplayString(DateTime.Now.ToShortDateString() + " " + DateTime.Now.ToShortTimeString() + " : " + "Group voting reuslt doesnot have any meeting data.");
                            totalResults = new TotalVotingResultsForm(null);
                            if (resultDisplayScreen.Length > 1)
                            {
                                totalResults.Left = resultDisplayScreen[1].WorkingArea.Left;
                                totalResults.Top = resultDisplayScreen[1].WorkingArea.Top;
                                totalResults.StartPosition = FormStartPosition.Manual;
                            }
                            totalResults.Show();
                        }
                        else
                        {
                            totalResults = new TotalVotingResultsForm(votingResults);
                            if (resultDisplayScreen.Length > 1)
                            {
                                totalResults.Left = resultDisplayScreen[1].WorkingArea.Left;
                                totalResults.Top = resultDisplayScreen[1].WorkingArea.Top;
                                totalResults.StartPosition = FormStartPosition.Manual;
                            }
                            //totalResults.UpdateHeaderData(votingResults);
                            totalResults.LoadVotingDataOnVotingStart(votingResults, votingStopped, 0);
                            totalResults.Show();
                           
                        }
                    }
                }
                else
                {
                    if (votingResults != null)
                    {
                        votingResults = null;
                    }
                    DisplayString(DateTime.Now.ToShortDateString() + " " + DateTime.Now.ToShortTimeString() + " : " + "Connection not establishied with the DCN-SW server.");
                }
            }
        }

        /// <summary>
        /// Function to Display the Microphone Status.
        /// </summary>
        private void ShowMicrophoneStatus()
        {
            if (this.InvokeRequired)
            {
                this.BeginInvoke(new ShowFormDelegate(ShowMicrophoneStatus));
            }
            else
            {

                if (microPhoneStatus != null)
                {
                    microPhoneStatus.Dispose();
                    microPhoneStatus = null;
                }

                if (connectionStatus)
                {
                    FindScreens();
                    #region CloseForms
                    indvVotingResults.Close();
                    groupResults.Close();
                    totalResults.Close();
                    currentAgenda.Close();
                    attendanceRegistration.Close();
                    groupSpeechTimerForm.Close();
                    #endregion
                    if (discussionData != null)
                    {
                        if (discussionData != null)
                        {
                            microPhoneStatus = new MicrophoneStatusForm();
                            if (resultDisplayScreen.Length > 1)
                            {
                                microPhoneStatus.Left = resultDisplayScreen[1].WorkingArea.Left;
                                microPhoneStatus.Top = resultDisplayScreen[1].WorkingArea.Top;
                                microPhoneStatus.StartPosition = FormStartPosition.Manual;
                            }
                            microPhoneStatus.Show();
                            microPhoneStatus.LoadMicrophoneStatusData(discussionData);
                            if (discussionData.ActiveList != null)
                            {
                                lock (microPhoneStatus.microphoneStatusLock)
                                {
                                    microPhoneStatus.speakerBindingSource.DataSource = speakerList;
                                }
                            }
                            if (discussionData.RequestList != null)
                            {
                                lock (requestlist)
                                {
                                    microPhoneStatus.requestsBindingSource.DataSource = requestlist;
                                }
                            }
                            if (discussionData.SpecialEquipment != null)
                            {
                                lock (microPhoneStatus.microphoneStatusLock)
                                {
                                    microPhoneStatus.speakerBindingSource.DataSource = speakerList;
                                }
                            }
                            if (discussionData.ActiveResponseList != null)
                            {
                                lock (microPhoneStatus.microphoneStatusLock)
                                {
                                    microPhoneStatus.speakerBindingSource.DataSource = speakerList;
                                }
                            }
                            if (discussionData.ResponseList != null)
                            {
                                lock (requestlist)
                                {
                                    microPhoneStatus.requestsBindingSource.DataSource = requestlist;
                                }
                            }

                        }
                        else
                        {
                            DisplayString(DateTime.Now.ToShortDateString() + " " + DateTime.Now.ToShortTimeString() + " : " + "Discussion Object is null");
                        }
                    }
                }
                else
                {
                    if (discussionData != null)
                    {
                        discussionData = null;
                    }
                    DisplayString(DateTime.Now.ToShortDateString() + " " + DateTime.Now.ToShortTimeString() + " : " + "Connection not establishied with the DCN-SW server.");
                }
            }

        }

        /// <summary>
        /// Function to Display the Group Speech Timers.
        /// </summary>
        private void ShowGroupSpeechTimers()
        {
            if (this.InvokeRequired)
            {
                this.BeginInvoke(new ShowFormDelegate(ShowGroupSpeechTimers));
            }
            else
            {
                if (groupSpeechTimerForm != null)
                {
                    groupSpeechTimerForm.Dispose();
                    groupSpeechTimerForm = null;
                }
                if (connectionStatus)
                {
                    FindScreens();
                    #region CloseForms
                    indvVotingResults.Close();
                    groupResults.Close();
                    totalResults.Close();
                    currentAgenda.Close();
                    attendanceRegistration.Close();
                    microPhoneStatus.Close();
                    #endregion

                    groupSpeechTimerForm = new GroupSpeechTimerForm();
                    if (resultDisplayScreen.Length > 1)
                    {
                        groupSpeechTimerForm.Left = resultDisplayScreen[1].WorkingArea.Left;
                        groupSpeechTimerForm.Top = resultDisplayScreen[1].WorkingArea.Top;
                        groupSpeechTimerForm.StartPosition = FormStartPosition.Manual;
                    }
                    groupSpeechTimerForm.Show();
                    groupSpeechTimerForm.groupSpeechTimeBindingSource.DataSource = groupSpeechTimeList;
                    groupSpeechTimerForm.LoadGroupSpeechData();
                
                }
            }

        }

        private void FindScreens()
        {
            resultDisplayScreen = Screen.AllScreens;
        }
        /// <summary>
        /// Function to Display the details of Current Agenda.
        /// </summary>
        private void ShowAgenda()
        {

            if (this.InvokeRequired)
            {
                this.BeginInvoke(new ShowFormDelegate(ShowAgenda));
            }
            else
            {
                if (currentAgenda != null)
                {
                    currentAgenda.Dispose();
                    currentAgenda = null;
                }
                if (connectionStatus)
                {
                    FindScreens();
                    #region CloseForms
                    indvVotingResults.Close();
                    groupResults.Close();
                    totalResults.Close();
                    microPhoneStatus.Close();
                    attendanceRegistration.Close();
                    groupSpeechTimerForm.Close();
                    #endregion
                    if (session != null)
                    {
                        if (session != null)
                        {
                            currentAgenda = new AgendaForm();
                            if (resultDisplayScreen.Length > 1)
                            {
                                currentAgenda.Left = resultDisplayScreen[1].WorkingArea.Left;
                                currentAgenda.Top = resultDisplayScreen[1].WorkingArea.Top;
                                currentAgenda.StartPosition = FormStartPosition.Manual;
                            }
                            currentAgenda.Show();
                            currentAgenda.LoadAgendaData(session);
                        }
                        else
                        {
                            DisplayString(DateTime.Now.ToShortDateString() + " " + DateTime.Now.ToShortTimeString() + " : " + "Session doesnot have any meeting data.");
                        }
                    }
                    else
                    {
                        DisplayString(DateTime.Now.ToShortDateString() + " " + DateTime.Now.ToShortTimeString() + " : " + "Session object is null");
                    }
                }
                else
                {
                    if (session != null)
                    {
                        session = null;
                    }
                    DisplayString(DateTime.Now.ToShortDateString() + " " + DateTime.Now.ToShortTimeString() + " : " + "Connection not establishied with the DCN-SW server.");
                }
            }
        }
        /// <summary>
        /// Function to Display the details of Attendance Registration.
        /// </summary>
        private void ShowAttendanceregistration()
        {
            if (this.InvokeRequired)
            {
                this.BeginInvoke(new ShowFormDelegate(ShowAttendanceregistration));
            }
            else
            {

                if (attendanceRegistration != null)
                {
                    attendanceRegistration.Dispose();
                    attendanceRegistration = null;
                }
                if (connectionStatus)
                {
                    FindScreens();
                    #region CloseForms
                    indvVotingResults.Close();
                    groupResults.Close();
                    totalResults.Close();
                    microPhoneStatus.Close();
                    currentAgenda.Close();
                    groupSpeechTimerForm.Close();
                    #endregion
                    if (particiapantInfo != null)
                    {
                        attendanceRegistration = new AttendanceRegistrationForm();
                        if (resultDisplayScreen.Length > 1)
                        {
                            attendanceRegistration.Left = resultDisplayScreen[1].WorkingArea.Left;
                            attendanceRegistration.Top = resultDisplayScreen[1].WorkingArea.Top;
                            attendanceRegistration.StartPosition = FormStartPosition.Manual;
                        }
                        attendanceRegistration.Show();
                        attendanceRegistration.LoadAttendanceRegistratioData(particiapantInfo);
                    }
                    else
                    {
                        DisplayString(DateTime.Now.ToShortDateString() + " " + DateTime.Now.ToShortTimeString() + " : " + "Attendance Registration data is not available.");
                    }
                }
                else
                {
                    if (particiapantInfo != null)
                    {
                        particiapantInfo = null;
                    }
                    DisplayString(DateTime.Now.ToShortDateString() + " " + DateTime.Now.ToShortTimeString() + " : " + "Connection not establishied with the DCN-SW server.");
                }
            }
        }



        private delegate void UpdateMicrophoneSpeakerDataTableDelegate(EnumActivityType activityType, bool IsActive);
        private void UpdateMicrophoneSpeakerDataTable(EnumActivityType activityType, bool IsActive)
        {
            if (this.InvokeRequired)
            {
                this.BeginInvoke(new UpdateMicrophoneSpeakerDataTableDelegate(UpdateMicrophoneSpeakerDataTable), activityType, IsActive);
            }
            else
            {
                try
                {
                    Speaker tempSpeaker;
                    List<Groups> tempGroupList = new List<Groups>();
                    Groups tempGroup;
                    tempGroup = new Groups();
                    lock (microPhoneStatus.microphoneStatusLock)
                    {
                        if (speakerList.Count > 0)
                        {
                            var clearingList = speakerList.Where(x => x.SeatType != "Chairman" && x.IsActive == IsActive).ToList();
                            foreach (var sp in clearingList)
                            {
                                if (sp.SeatType != "Chairman" && sp.IsActive == IsActive)
                                {
                                    speakerList.Remove(sp);
                                }
                            }
                        }

                        if (activityType == EnumActivityType.ActiveListUpdated && discussionData.ActiveList != null)
                        {
                            #region speakerList

                            foreach (ParticipantContainer participant in discussionData.ActiveList.Participants)
                            {
                                tempGroup = null;
                                tempGroup = new Groups();
                                tempSpeaker = new Speaker();
                                if (participant.ParticipantData != null)
                                {
                                    tempSpeaker.Speakers = participant.ParticipantData.FirstName + " " + participant.ParticipantData.MiddleName + " " + participant.ParticipantData.LastName;
                                    RemainingTime = new TimeSpan(participant.ParticipantData.RemainingSpeechTime * TimeSpan.TicksPerSecond);
                                    if (participant.ParticipantData.RemainingSpeechTime != -1)
                                    {
                                        DisplayTime = string.Format(CultureInfo.InvariantCulture, SpeechTimeFormatString, Math.Abs(RemainingTime.Minutes), Math.Abs(RemainingTime.Seconds));
                                    }
                                    else
                                    {
                                        DisplayTime = String.Empty;
                                    }
                                    tempSpeaker.Remaining_Time = DisplayTime;
                                    tempSpeaker.RemainingSpeechTime = participant.ParticipantData.RemainingSpeechTime * 1000;
                                    tempSpeaker.InitialRemSpeechTime = participant.ParticipantData.RemainingSpeechTime * 1000;
                                    tempGroup.GroupName = participant.Group.Name;
                                    tempGroup.GroupSpeechTimeLimit = participant.Group.RemainingGroupSpeechTime * 1000;
                                    tempGroup.ActiveSpeakerCount++;
                                    tempGroup.StopWatchState = (STOPWATCH_STATE)participant.Group.StopWatchState;
                                    #region TempGroupList
                                    var grpToBeUpdated = tempGroupList.Where(x => x.GroupName == tempGroup.GroupName).ToList();
                                    if (grpToBeUpdated.Count > 0)
                                    {
                                        foreach (var tempgrp in grpToBeUpdated)
                                        {
                                            if (participant.Seat.SeatData.MicrophoneActive)
                                            {
                                                tempgrp.ActiveSpeakerCount++;
                                                tempgrp.GroupSpeechTimeLimit = tempGroup.GroupSpeechTimeLimit;
                                            }
                                        }
                                    }
                                    else
                                    {
                                        if (participant.Seat.SeatData.MicrophoneActive)
                                        {
                                            tempGroupList.Add(tempGroup);
                                        }
                                    }
                                    #endregion

                                }
                                else
                                {
                                    tempSpeaker.RemainingSpeechTime = -1000;
                                    tempSpeaker.InitialRemSpeechTime = -1000;
                                }

                                tempSpeaker.ParticcipantId = participant.Id;

                                if (participant.Seat != null && participant.Seat.SeatData != null)
                                {
                                    if (participant.ParticipantData == null)
                                    {
                                        tempSpeaker.Speakers = participant.Seat.SeatData.Name;
                                    }
                                    tempSpeaker.SeatId = participant.Seat.Id;
                                    if (participant.ParticipantData != null)
                                    {
                                        if (participant.Seat.SeatData.MicrophoneActive && !participant.ParticipantData.SpeechTimerOnHold)
                                        {
                                            tempSpeaker.MicrophoneActiveAndRunning = true;
                                        }
                                    }
                                    else
                                    {
                                        if (participant.Seat.SeatData.MicrophoneActive)
                                        {
                                            tempSpeaker.MicrophoneActiveAndRunning = true;
                                        }
                                    }
                                    tempSpeaker.SeatType = participant.Seat.SeatData.SeatType;
                                    tempSpeaker.IsActive = IsActive;
                                    tempSpeaker.IsSpecialStation = participant.Seat.SeatData.IsSpecialStation;
                                    speakerList.Insert(speakerList.Count, tempSpeaker);
                                }

                                
                            }

                            #region GroupList

                            foreach (Groups grp in tempGroupList)
                            {
                                var grpToBeUpdated = groupSpeechTimeList.Where(x => x.GroupName == grp.GroupName).ToList();
                                if (grpToBeUpdated.Count > 0)
                                {
                                    foreach (var tempgrp in grpToBeUpdated)
                                    {
                                        tempgrp.ActiveSpeakerCount = grp.ActiveSpeakerCount * (-10);
                                        tempgrp.StopWatchState = grp.StopWatchState;
                                    }
                                }
                            }

                            var grpsToBeUpdated = groupSpeechTimeList.Where(x => x.ActiveSpeakerCount > 0).ToList();
                            foreach (Groups grp in grpsToBeUpdated)
                            {
                                grp.ActiveSpeakerCount = 0;
                            }

                            grpsToBeUpdated = groupSpeechTimeList.Where(x => x.ActiveSpeakerCount < 0).ToList();
                            foreach (Groups grp in grpsToBeUpdated)
                            {
                                grp.ActiveSpeakerCount /= -10;
                            }

                            #endregion

                            microPhoneStatus.speakerBindingSource.DataSource = speakerList;
                            microPhoneStatus.dgvSpeakers.DataSource = microPhoneStatus.speakerBindingSource;
                            #endregion

                          
                        }

                        if (activityType == EnumActivityType.ActiveResponseListUpdated && discussionData.ActiveResponseList != null)
                        {
                            #region speakersList
                            foreach (ParticipantContainer participant in discussionData.ActiveResponseList.Participants)
                            {
                                tempSpeaker = new Speaker();
                                if (participant.ParticipantData != null)
                                {
                                    tempSpeaker.Speakers = participant.ParticipantData.FirstName + " " + participant.ParticipantData.MiddleName + " " + participant.ParticipantData.LastName;
                                    RemainingTime = new TimeSpan(participant.ParticipantData.RemainingSpeechTime * TimeSpan.TicksPerSecond);
                                    DisplayTime = string.Format(CultureInfo.InvariantCulture, SpeechTimeFormatString, Math.Abs(RemainingTime.Minutes), Math.Abs(RemainingTime.Seconds));
                                    tempSpeaker.Remaining_Time = DisplayTime;
                                    tempSpeaker.RemainingSpeechTime = participant.ParticipantData.RemainingSpeechTime * 1000;
                                    tempSpeaker.InitialRemSpeechTime = participant.ParticipantData.RemainingSpeechTime * 1000;
                                }
                                else
                                {
                                    tempSpeaker.RemainingSpeechTime = -1000;
                                    tempSpeaker.InitialRemSpeechTime = -1000;
                                }

                                tempSpeaker.ParticcipantId = participant.Id;

                                if (participant.Seat != null && participant.Seat.SeatData != null)
                                {
                                    if (participant.ParticipantData == null)
                                    {
                                        tempSpeaker.Speakers = participant.Seat.SeatData.Name;
                                    }
                                    tempSpeaker.SeatId = participant.Seat.Id;
                                    if (participant.ParticipantData != null)
                                    {
                                        if (participant.Seat.SeatData.MicrophoneActive && !participant.ParticipantData.SpeechTimerOnHold)
                                        {
                                            tempSpeaker.MicrophoneActiveAndRunning = true;
                                        }
                                    }
                                    else
                                    {
                                        if (participant.Seat.SeatData.MicrophoneActive)
                                        {
                                            tempSpeaker.MicrophoneActiveAndRunning = true;
                                        }
                                    }
                                    tempSpeaker.SeatType = participant.Seat.SeatData.SeatType;
                                    tempSpeaker.IsActive = IsActive;
                                    tempSpeaker.IsSpecialStation = participant.Seat.SeatData.IsSpecialStation;
                                    speakerList.Insert(speakerList.Count, tempSpeaker);
                                }
                                

                            }
                            #endregion
                        }
                    }
                }
                catch (Exception ex)
                {
                    DisplayString(ex.StackTrace);
                }
            }
        }


        private delegate void AddChairmanToSpeakerTableDelegate(SeatActivity seatActivity);
        private void AddChairmanOrNoteBookerToSpeakerTable(SeatActivity seatActivity)
        {
            if(this.InvokeRequired)
            {
                this.BeginInvoke(new AddChairmanToSpeakerTableDelegate(AddChairmanOrNoteBookerToSpeakerTable), seatActivity);
            }
            else
            {
                lock (microPhoneStatus.microphoneStatusLock) 
                {
                    Speaker tempSpeaker = new Speaker();
                    if (seatActivity.Seat.Participant.ParticipantData.FirstName != "" && seatActivity.Seat.Participant.ParticipantData.MiddleName != "")
                    {
                        tempSpeaker.Speakers = seatActivity.Seat.Participant.ParticipantData.FirstName + " " + seatActivity.Seat.Participant.ParticipantData.MiddleName + " " + seatActivity.Seat.Participant.ParticipantData.LastName;
                    }
                    else
                    {
                        tempSpeaker.Speakers = seatActivity.Seat.Participant.ParticipantData.FirstName;
                        tempSpeaker.Speakers += seatActivity.Seat.Participant.ParticipantData.MiddleName;
                        tempSpeaker.Speakers += seatActivity.Seat.Participant.ParticipantData.LastName;
                    }


                    RemainingTime = new TimeSpan(seatActivity.Seat.Participant.ParticipantData.RemainingSpeechTime * TimeSpan.TicksPerSecond);
                    DisplayTime = string.Format(CultureInfo.InvariantCulture, SpeechTimeFormatString, Math.Abs(RemainingTime.Minutes), Math.Abs(RemainingTime.Seconds));
                    tempSpeaker.Remaining_Time = String.Empty;
                    tempSpeaker.RemainingSpeechTime = seatActivity.Seat.Participant.ParticipantData.RemainingSpeechTime * 1000;
                    tempSpeaker.ParticcipantId = seatActivity.Seat.Participant.Id;
                    tempSpeaker.SeatId = seatActivity.Seat.Id;
                    tempSpeaker.MicrophoneActiveAndRunning = seatActivity.Seat.SeatData.MicrophoneActive;
                    tempSpeaker.SeatType = seatActivity.Seat.SeatData.SeatType;
                    tempSpeaker.IsSpecialStation = seatActivity.Seat.SeatData.IsSpecialStation;
                    speakerList.Add(tempSpeaker);
                    microPhoneStatus.speakerBindingSource.DataSource = speakerList;
                }
            }
        }
        private delegate void UpdateChairmantoSpeakerDataTableDelegate();
        private void UpdateChairmanToSpeakerDataTable()
        {
            if (this.InvokeRequired)
            {
                this.BeginInvoke(new UpdateChairmantoSpeakerDataTableDelegate(UpdateChairmanToSpeakerDataTable));
            }
            else
            {
                if (discussionData.SpecialEquipment != null)
                {
                    foreach (ParticipantContainer participant in discussionData.SpecialEquipment.Participants)
                    {
                        if (participant.Seat != null && participant.Seat.SeatData.MicrophoneActive)
                        {
                            lock (microPhoneStatus.microphoneStatusLock)
                            {
                                Speaker tempSpeaker = new Speaker();
                                tempSpeaker.Speakers = participant.ParticipantData.FirstName + " " + participant.ParticipantData.MiddleName + " " + participant.ParticipantData.LastName;
                                RemainingTime = new TimeSpan(participant.ParticipantData.RemainingSpeechTime * TimeSpan.TicksPerSecond);
                                DisplayTime = string.Format(CultureInfo.InvariantCulture, SpeechTimeFormatString, Math.Abs(RemainingTime.Minutes), Math.Abs(RemainingTime.Seconds));
                                tempSpeaker.Remaining_Time = String.Empty;
                                tempSpeaker.RemainingSpeechTime = participant.ParticipantData.RemainingSpeechTime * 1000;
                                tempSpeaker.ParticcipantId = participant.Id;
                                tempSpeaker.SeatId = participant.Seat.Id;
                                tempSpeaker.MicrophoneActiveAndRunning = participant.Seat.SeatData.MicrophoneActive;
                                tempSpeaker.SeatType = participant.Seat.SeatData.SeatType;
                                tempSpeaker.IsSpecialStation = participant.Seat.SeatData.IsSpecialStation;

                                if (speakerList.Count > 0)
                                {
                                    // To check whther the Notebooker is alreadty present. If present don't add.
                                    var chairmanList = speakerList.Where(x=> x.SeatId == participant.Seat.Id).ToList();
                                    if (chairmanList.Count == 0)
                                    {
                                        speakerList.Add(tempSpeaker);
                                    }
                                }
                                else
                                {
                                    speakerList.Add(tempSpeaker);
                                }
                            }
                        }
                    }
                }
            }
        }

        private delegate void UpdateMicrophoneStatusInSpeakersTableDelegate(ParticipantActivity particpantActivity);
        private void UpdateMicrophoneStatusInSpeakersTable(ParticipantActivity particpantActivity)
        {
            if (this.InvokeRequired)
            {
                this.BeginInvoke(new UpdateMicrophoneStatusInSpeakersTableDelegate(UpdateMicrophoneStatusInSpeakersTable), particpantActivity);
            }
            else
            {
                try
                {
                    string filterExpression = "SeatId = '" + particpantActivity.Participant.Seat.Id + "'";
                    bool micStatus = false;
                    if (particpantActivity.Participant.Seat.SeatData.MicrophoneActive && !particpantActivity.Participant.ParticipantData.SpeechTimerOnHold)
                    {
                        micStatus = true;
                    }

                    lock (microPhoneStatus.microphoneStatusLock)
                    {
                        if (speakerList.Count > 0)
                        {
                            foreach (Speaker speaker in speakerList)
                            {
                                if (speaker.SeatId == particpantActivity.Participant.Seat.Id)
                                {
                                    speaker.Speakers = particpantActivity.Participant.ParticipantData.FirstName + " " + particpantActivity.Participant.ParticipantData.MiddleName + " " + particpantActivity.Participant.ParticipantData.LastName;
                                    RemainingTime = new TimeSpan(particpantActivity.Participant.ParticipantData.RemainingSpeechTime * TimeSpan.TicksPerSecond);
                                    DisplayTime = string.Format(CultureInfo.InvariantCulture, SpeechTimeFormatString, Math.Abs(RemainingTime.Minutes), Math.Abs(RemainingTime.Seconds));
                                    if (particpantActivity.Participant.ParticipantData.RemainingSpeechTime <= -1)
                                    {
                                        speaker.Remaining_Time = "-" + DisplayTime;
                                    }
                                    else
                                    {
                                        speaker.Remaining_Time = DisplayTime;
                                    }
                                    speaker.RemainingSpeechTime = particpantActivity.Participant.ParticipantData.RemainingSpeechTime * 1000;
                                    speaker.ParticcipantId = particpantActivity.Participant.Id;
                                    speaker.MicrophoneActiveAndRunning = micStatus;
                                    speaker.SeatType = particpantActivity.Participant.Seat.SeatData.SeatType;
                                    speaker.InitialRemSpeechTime = particpantActivity.Participant.ParticipantData.RemainingSpeechTime;
                                    var grpToBeUpdated = groupSpeechTimeList.Where(x => x.GroupName == particpantActivity.Participant.Group.Name);
                                    foreach (Groups grp in grpToBeUpdated)
                                    {
                                        grp.StopWatchState = particpantActivity.Participant.Group.StopWatchState;
                                    }

                                }
                            }
                        }
                    }
                }
                catch (Exception ex)
                {
                    DisplayString(ex.StackTrace.ToString());
                }
            }
        }
       
        private delegate void UpdateGroupSpeechListOnSeatActivityMicOnOffDelagte(SeatActivity seatActivity);
        private void UpdateGroupSpeechListOnSeatActivityMicOnOff(SeatActivity seatActivity)
        {
            if (this.InvokeRequired)
            {
                this.BeginInvoke(new UpdateGroupSpeechListOnSeatActivityMicOnOffDelagte(UpdateGroupSpeechListOnSeatActivityMicOnOff), seatActivity);
            }
            else
            {
                if (!seatActivity.Seat.IsReposnding)
                {
                    if (seatActivity.Seat.SeatData.MicrophoneActive == false)
                    {
                        if (seatActivity.Seat.Participant.Group != null)
                        {
                            var grpToBeUpdated = groupSpeechTimeList.Where(x => x.GroupName == seatActivity.Seat.Participant.Group.Name);
                            foreach (Groups grp in grpToBeUpdated)
                            {
                                if (grp.ActiveSpeakerCount > 0)
                                {
                                    grp.ActiveSpeakerCount--;
                                    if (seatActivity.Seat.Participant.Group.RemainingGroupSpeechTime == -1)
                                    {
                                        grp.GroupSpeechTimeLimit = INFINITE_SPEECH_TIME;
                                        grp.StopWatchState = seatActivity.Seat.Participant.Group.StopWatchState;
                                    }
                                    else
                                    {
                                        grp.GroupSpeechTimeLimit = seatActivity.Seat.Participant.Group.RemainingGroupSpeechTime * 1000;
                                        grp.StopWatchState = seatActivity.Seat.Participant.Group.StopWatchState;
                                    }
                                }
                            }
                        }
                        else
                        {
                            activityLogUpdater.ReportActivity(() =>
                            {
                                activityLogListView.Items.Add("Group was null");
                            });
                        }

                    }
                    else if (seatActivity.Seat.SeatData.MicrophoneActive == true)
                    {
                        if (seatActivity.Seat.Participant.Group != null)
                        {
                            var grpToBeUpdated = groupSpeechTimeList.Where(x => x.GroupName == seatActivity.Seat.Participant.Group.Name);
                            foreach (Groups grp in grpToBeUpdated)
                            {
                                grp.ActiveSpeakerCount++;
                                if (seatActivity.Seat.Participant.Group.RemainingGroupSpeechTime == -1)
                                {
                                    grp.GroupSpeechTimeLimit = INFINITE_SPEECH_TIME;
                                    grp.StopWatchState = seatActivity.Seat.Participant.Group.StopWatchState;
                                }
                                else
                                {
                                    grp.GroupSpeechTimeLimit = seatActivity.Seat.Participant.Group.RemainingGroupSpeechTime * 1000;
                                    grp.StopWatchState = seatActivity.Seat.Participant.Group.StopWatchState;
                                }

                            }
                        }
                        else
                        {
                            activityLogUpdater.ReportActivity(() =>
                            {
                                activityLogListView.Items.Add("Group was null");
                            });
                        }
                    }
                }
            }
        }

      




        private delegate void UpdateMicrophoneStatusInSpeakersTableForSeatDelegate(int seatId, bool micStatus, int remTime);
        private void UpdateMicrophoneStatusInSpeakersTable(int seatId, bool micStatus, int remTime)
        {
            if (this.InvokeRequired)
            {
                this.BeginInvoke(new UpdateMicrophoneStatusInSpeakersTableForSeatDelegate(UpdateMicrophoneStatusInSpeakersTable), seatId, micStatus, remTime);
            }
            else
            {
                try
                {
                    foreach (Speaker speaker in speakerList)
                    {
                        if (speaker.SeatId == seatId)
                        {
                            if (speaker.MicrophoneActiveAndRunning != micStatus)
                            {
                                speaker.RemainingSpeechTime = remTime * 1000;
                                speaker.InitialRemSpeechTime = remTime * 1000;
                                speaker.MicrophoneActiveAndRunning = micStatus;
                            }
                        }
                    }
                }
                catch (Exception ex)
                {
                    DisplayString("Exception occurred: " + ex.Message.ToString());
                }
            }
        }

        private delegate void RemoveChairmanMicStatusDelegate(int seatId);
        private void RemoveCharimanMicPhoneStatus(int seatId)
        {
            if (this.InvokeRequired)
            {
                this.BeginInvoke(new RemoveChairmanMicStatusDelegate(RemoveCharimanMicPhoneStatus), seatId);
            }
            else
            {
                try
                {
                    lock (microPhoneStatus.microphoneStatusLock)
                    {
                        var removeChairman = speakerList.Where(x => x.SeatId == seatId).ToList();
                        foreach (var sp in removeChairman)
                        {
                            speakerList.Remove(sp);
                        }
                    }
                }
                catch (Exception e)
                {
                    DisplayString("Exception occurred: " + e.Message);
                    DisplayString(e.StackTrace);
                }
            }
        }

        private delegate void UpdateMicrophoneRequestListDelegate(bool IsResponse);
        private void UpdateMicrophoneRequestList(bool IsResponse)
        {
            if (this.InvokeRequired)
            {
                this.BeginInvoke(new UpdateMicrophoneRequestListDelegate(UpdateMicrophoneRequestList), IsResponse);
            }
            else
            {
                if (requestlist.Count > 0)
                {
                    lock (requestlist)
                    {
                        var clearRequests = requestlist.Where(x => x.SeatType != "Chairman" && x.IsResponse == IsResponse).ToList();
                        foreach (var request in clearRequests)
                        {
                            requestlist.Remove(request);
                        }
                    }
                }

                if (!IsResponse)
                {
                    foreach (ParticipantContainer participant in discussionData.RequestList.Participants)
                    {
                        Requests tempRequest = new Requests();
                        lock (requestlist)
                        {

                            tempRequest.Request = participant.ParticipantData.FirstName + " " + participant.ParticipantData.MiddleName + " " + participant.ParticipantData.LastName;
                            if (participant.Seat != null)
                            {
                                tempRequest.SeatId = participant.Seat.Id.ToString();
                                tempRequest.SeatType = participant.Seat.SeatData.SeatType;
                            }
                            tempRequest.IsResponse = IsResponse;
                            requestlist.Insert(requestlist.Count, tempRequest);
                        }
                    }
                    microPhoneStatus.requestsBindingSource.DataSource = requestlist;
                }
                
                    ///For Response List
                else if (IsResponse)
                {
                    foreach (ParticipantContainer participant in discussionData.ResponseList.Participants)
                    {
                        Requests tempRequest = new Requests();
                        lock (requestlist)
                        {
                            tempRequest.Request = participant.ParticipantData.FirstName + " " + participant.ParticipantData.MiddleName + " " + participant.ParticipantData.LastName;
                            if (participant.Seat != null)
                            {
                                tempRequest.SeatId = participant.Seat.Id.ToString();
                                tempRequest.SeatType = participant.Seat.SeatData.SeatType;
                            }
                            tempRequest.IsResponse = IsResponse;
                            requestlist.Insert(requestlist.Count, tempRequest);
                        }
                    }
                }
            }
        }

        private delegate void UpdateGroupSpeechTimeListOnSessionStartDelegate(SessionActivity sessionActivity);
        private void UpdateGroupSpeechTimeListOnSessionStart(SessionActivity sessionActivity)
        {
            if (this.InvokeRequired)
            {
                this.BeginInvoke(new UpdateGroupSpeechTimeListOnSessionStartDelegate(UpdateGroupSpeechTimeListOnSessionStart), sessionActivity);
            }
            else
            {
                foreach (GroupContainer grpContainer in sessionActivity.Session.Groups)
                {
                    Groups grp = null;
                    grp = new Groups();

                    /// The following condition has to be checked as "-" is the name of the default group and Speeh time limit is
                    /// not applicable to this group.
                    
                    if (grpContainer.Name != "-")
                    {
                        grp.GroupName = grpContainer.Name;
                        if (grpContainer.RemainingGroupSpeechTime == -1)
                        {
                            grp.GroupSpeechTimeLimit = INFINITE_SPEECH_TIME;
                        }
                        else
                        {
                            grp.GroupSpeechTimeLimit = grpContainer.RemainingGroupSpeechTime * 1000;
                        }
                        grp.ActiveSpeakerCount = 0;
                        grp.StopWatchState = grpContainer.StopWatchState;
                        groupSpeechTimeList.Add(grp);
                    }
                }
                groupSpeechTimerForm.groupSpeechTimeBindingSource.DataSource = groupSpeechTimeList;
            }

        }
        private void SmdTimer_Tick(Object sender, EventArgs e)
        {
            if (this.InvokeRequired)
            {
                this.BeginInvoke(new EventHandler(SmdTimer_Tick), sender, e);
            }
            else
            {
                if (indvVotingResults != null)
                {
                    indvVotingResults.UpdateHeaderDateAndTime();
                }
                if (groupResults != null)
                {
                    groupResults.UpdateHeaderDateAndTime();
                }
                if (totalResults != null)
                {
                    totalResults.UpdateHeaderDateAndTime();
                }
                if (microPhoneStatus != null)
                {
                    microPhoneStatus.UpdateHeaderDateAndTime();
                }
                if (attendanceRegistration != null)
                {
                    attendanceRegistration.UpdateHeaderDateAndTime();
                }
                if (currentAgenda != null)
                {
                    currentAgenda.UpdateHeaderDateAndTime();
                }
                if (groupSpeechTimerForm != null)
                {
                    groupSpeechTimerForm.UpdateHeaderDateAndTime();
                }
                // To do : Update the above to GroupSpeecgTimers.
            }
        }

        private void GroupSpeechTimer_Tick(object sender, EventArgs e)
        {
            if (this.InvokeRequired)
            {
                this.BeginInvoke(new EventHandler(GroupSpeechTimer_Tick), sender, e);
            }
            else
            {
                TimeSpan diffTime = DateTime.Now.Subtract(groupSpeechCurrentTime);
                groupSpeechCurrentTime = DateTime.Now;
                lock (groupSpeechTimeList)
                {
                    foreach (Groups grp in groupSpeechTimeList)
                    {
                        if (grp.StopWatchState == STOPWATCH_STATE.STOPWATCH_RUNNING && grp.ActiveSpeakerCount > 0 && grp.GroupSpeechTimeLimit != INFINITE_SPEECH_TIME)
                        {
                            long remainingTime = grp.GroupSpeechTimeLimit;
                            grp.GroupSpeechTimeLimit = (remainingTime -(long)diffTime.TotalMilliseconds);
                        }
                    }
                }
            }
        }
        private void VotingTimer_Tick(Object sender, EventArgs e)
        {
            if (this.InvokeRequired)
            {
                this.BeginInvoke(new EventHandler(VotingTimer_Tick), sender, e);
            }
            else
            {
                votingTimeLimitTimer.Stop();
                TimeSpan diffTime = DateTime.Now.Subtract(votingLimitCurrentTime);
                votingTimeLimitTimer.Start();
                votingLimitCurrentTime = DateTime.Now;
                string remainingTime = String.Empty;

                if (votingResults != null)
                {
                    if (votingResults.VotingData != null)
                    {
                        if (!votingStopped)
                        {
                            if (votingResults.VotingData.RemainingVotingTime > 0)
                            {
                                votingResults.VotingData.RemainingVotingTime = votingResults.VotingData.RemainingVotingTime - (int)diffTime.TotalSeconds;
                                if (votingResults.VotingData.RemainingVotingTime > 0)
                                {
                                    votingTimeLimitTimer.Start();
                                }
                                indvVotingResults.UpdateRemainingVotingTime(votingResults.VotingData.RemainingVotingTime);
                                totalResults.UpdateRemainingVotingTime(votingResults);
                                groupResults.UpdateRemainingVotingTime(votingResults);

                            }
                        }
                        else if (votingStopped == true)
                        {
                            indvVotingResults.UpdateRemainingVotingTime(votingResults.VotingData.RemainingVotingTime);
                            totalResults.UpdateRemainingVotingTime(votingResults);
                            groupResults.UpdateRemainingVotingTime(votingResults);
                        }
                    }
                }
            }
        }
        private void MicrophoneStatusTimer_Tick(Object sender, EventArgs e)
        {
            if (this.InvokeRequired)
            {
                this.BeginInvoke(new EventHandler(MicrophoneStatusTimer_Tick), sender, e);
            }
            else
            {
                TimeSpan diffTime = DateTime.Now.Subtract(microphoneStatusCurrentTime);
                microphoneStatusCurrentTime = DateTime.Now;
                lock (speakerList)
                {
                    if (connectionStatus)
                    {
                        foreach (Speaker s in speakerList)
                        {
                            if ((s.MicrophoneActiveAndRunning.ToString() == "True")&&
                                (s.SeatType != "Chairman") && 
                                (s.IsSpecialStation != true) &&
                                (s.ParticcipantId != 0 && s.InitialRemSpeechTime != -1000))
                            {
                                long remainingTime = s.RemainingSpeechTime;
                                s.RemainingSpeechTime = (remainingTime - (long)diffTime.TotalMilliseconds);
                            }
                        }
                    }
                }
            }
        }

        delegate void ChangeVisibilityOfDataGridViewDelegate(DataGridView gridControl, bool isVisible);
        private void ChangeVisibilityofGridView(DataGridView gridViewControl, bool isVisible)
        {
            ChangeVisibilityOfDataGridViewDelegate obj = new ChangeVisibilityOfDataGridViewDelegate(ChangeVisibilityofGridView);
            if (gridViewControl.InvokeRequired)
            {
                if (gridViewControl.IsHandleCreated)
                {
                    gridViewControl.BeginInvoke(obj, new object[] { gridViewControl, isVisible });
                }
            }

            else
            {
                gridViewControl.Visible = isVisible;
            }
        }

        private delegate void ClearRequestListDelegate();
        private void ClearRequestList()
        {
            if (this.InvokeRequired)
            {
                this.BeginInvoke(new ClearRequestListDelegate(ClearRequestList));
            }
            else
            {
                requestlist.Clear();
            }
        }

        private delegate void ClearGroupSpeechTimeListDelegate();
        private void ClearGroupSpeechTimes()
        {
            if (this.InvokeRequired)
            {
                this.BeginInvoke(new ClearGroupSpeechTimeListDelegate(ClearGroupSpeechTimes));
            }
            else
            {
                groupSpeechTimeList.Clear();
            }
        }
        private delegate void ClearIndvResultsDataSourceDelegate();
        private void ClearIndvResultsDataSource()
        {
            if (this.InvokeRequired)
            {
                this.BeginInvoke(new ClearIndvResultsDataSourceDelegate(ClearIndvResultsDataSource));
            }
            else
            {
                indvResultsDataSource.Clear();
            }
        }
        private delegate void ClearSpeakersListDelegate();
        private void ClearSpeakersList()
        {
            if (this.InvokeRequired)
            {
                this.BeginInvoke(new ClearSpeakersListDelegate(ClearSpeakersList));
            }
            else
            {
                speakerList.Clear();
            }
        }

        private void radioBtnIndvResults_CheckedChanged(object sender, EventArgs e)
        {
            if (radioBtnIndvResults.Checked)
            {
                if (connectionStatus)
                {
                    radioBtnAttendanceRegn.Checked = false;
                    radioBtnGroupResults.Checked = false;
                    radioBtnSpeakers.Checked = false;
                    radioBtnTotalResults.Checked = false;
                    radioBtnAgenda.Checked = false;
                    radioBtnGroupSpeechTime.Checked = false;
                    ShowIndividualVotingResults();
                }
                else
                {
                    DisplayString(DateTime.Now.ToShortDateString() + " " + DateTime.Now.ToShortTimeString() + " : " + "Connection not establishied with the DCN-SW server.");
                    activityLogListView.Columns[0].Width = activityLogListView.Width - 5;
                    radioBtnIndvResults.Checked = false;
                }
            }
            else
            {
                if (indvVotingResults != null)
                {
                    indvVotingResults.Close();
                }
            }
        }

        private void radioBtnGroupResults_CheckedChanged(object sender, EventArgs e)
        {
            if (radioBtnGroupResults.Checked)
            {
                if (connectionStatus)
                {
                    radioBtnAttendanceRegn.Checked = false;
                    radioBtnAgenda.Checked = false;
                    radioBtnSpeakers.Checked = false;
                    radioBtnTotalResults.Checked = false;
                    radioBtnIndvResults.Checked = false;
                    radioBtnGroupSpeechTime.Checked = false;
                    //if (!votingHold)
                    //{
                        ShowGroupVotingResuts();

                    //}
                    //else
                    //{
                    //    ShowGroupVotingResultsOnHold();
                    //}
                }
                else
                {
                    DisplayString(DateTime.Now.ToShortDateString() + " " + DateTime.Now.ToShortTimeString() + " : " + "Connection not establishied with the DCN-SW server.");
                    activityLogListView.Columns[0].Width = activityLogListView.Width - 5;
                    radioBtnGroupResults.Checked = false;
                }
            }
            else
            {
                if (groupResults != null)
                {
                    groupResults.Close();
                }
            }
        }

        private void radioBtnTotalResults_CheckedChanged(object sender, EventArgs e)
        {
            if (radioBtnTotalResults.Checked)
            {
                if (connectionStatus)
                {
                    radioBtnAttendanceRegn.Checked = false;
                    radioBtnGroupResults.Checked = false;
                    radioBtnSpeakers.Checked = false;
                    radioBtnAgenda.Checked = false;
                    radioBtnIndvResults.Checked = false;
                    radioBtnGroupSpeechTime.Checked = false;
                    ShowTotalVotingResults();
                }
                else
                {
                    DisplayString(DateTime.Now.ToShortDateString() + " " + DateTime.Now.ToShortTimeString() + " : " + "Connection not establishied with the DCN-SW server.");
                    radioBtnTotalResults.Checked = false;
                }
            }
            else
            {
                if (totalResults != null)
                {
                    totalResults.Close();
                }
            }
        }

        private void radioBtnSpeakers_CheckedChanged(object sender, EventArgs e)
        {
            if (radioBtnSpeakers.Checked)
            {
                if (connectionStatus)
                {
                    radioBtnAttendanceRegn.Checked = false;
                    radioBtnGroupResults.Checked = false;
                    radioBtnAgenda.Checked = false;
                    radioBtnTotalResults.Checked = false;
                    radioBtnIndvResults.Checked = false;
                    radioBtnGroupSpeechTime.Checked = false;
                    ShowMicrophoneStatus();
                }
                else
                {
                    DisplayString(DateTime.Now.ToShortDateString() + " " + DateTime.Now.ToShortTimeString() + " : " + "Connection not establishied with the DCN-SW server.");
                    activityLogListView.Columns[0].Width = activityLogListView.Width - 5;
                    radioBtnSpeakers.Checked = false;
                }
            }
            else
            {
                if (microPhoneStatus != null)
                {
                    microPhoneStatus.Close();
                }
            }
        }

        private void radioBtnGroupSpeechTime_CheckedChanged(object sender, EventArgs e)
        {
            if (radioBtnGroupSpeechTime.Checked)
            {
                if (connectionStatus)
                {
                    radioBtnAttendanceRegn.Checked = false;
                    radioBtnGroupResults.Checked = false;
                    radioBtnAgenda.Checked = false;
                    radioBtnTotalResults.Checked = false;
                    radioBtnIndvResults.Checked = false;
                    radioBtnSpeakers.Checked = false;
                    ShowGroupSpeechTimers();
                }
                else
                {
                    DisplayString(DateTime.Now.ToShortDateString() + " " + DateTime.Now.ToShortTimeString() + " : " + "Connection not establishied with the DCN-SW server.");
                    activityLogListView.Columns[0].Width = activityLogListView.Width - 5;
                    radioBtnGroupSpeechTime.Checked = false;
                }
            }
        }

        private void radioBtnAgenda_CheckedChanged(object sender, EventArgs e)
        {
            if (radioBtnAgenda.Checked)
            {
                if (connectionStatus)
                {
                    radioBtnAttendanceRegn.Checked = false;
                    radioBtnGroupResults.Checked = false;
                    radioBtnSpeakers.Checked = false;
                    radioBtnTotalResults.Checked = false;
                    radioBtnIndvResults.Checked = false;
                    radioBtnGroupSpeechTime.Checked = false;
                    ShowAgenda();
                }
                else
                {
                    DisplayString(DateTime.Now.ToShortDateString() + " " + DateTime.Now.ToShortTimeString() + " : " + "Connection not establishied with the DCN-SW server.");
                    activityLogListView.Columns[0].Width = activityLogListView.Width - 5;
                    radioBtnAgenda.Checked = false;
                }
            }
            else
            {
                if (currentAgenda != null)
                {
                    currentAgenda.Close();
                }
            }
        }

        private void radioBtnAttendanceRegn_CheckedChanged(object sender, EventArgs e)
        {
            if (radioBtnAttendanceRegn.Checked)
            {
                if (connectionStatus)
                {
                    radioBtnAgenda.Checked = false;
                    radioBtnGroupResults.Checked = false;
                    radioBtnSpeakers.Checked = false;
                    radioBtnTotalResults.Checked = false;
                    radioBtnIndvResults.Checked = false;
                    radioBtnGroupSpeechTime.Checked = false;
                    ShowAttendanceregistration();
                }
                else
                {
                    DisplayString(DateTime.Now.ToShortDateString() + " " + DateTime.Now.ToShortTimeString() + " : " + "Connection not establishied with the DCN-SW server.");
                    activityLogListView.Columns[0].Width = activityLogListView.Width - 5;
                    radioBtnAttendanceRegn.Checked = false;
                }
            }
            else
            {
                if (attendanceRegistration != null)
                {
                    attendanceRegistration.Close();
                }
            }

        }

        private void CloseAllScreens()
        {
            if (indvVotingResults != null)
            {
                indvVotingResults.Close();
            }
            if (groupResults != null)
            {
                groupResults.Close();
            }
            if (totalResults != null)
            {
                totalResults.Close();
            }
            if (microPhoneStatus != null)
            {
                microPhoneStatus.Close();
            }
            if (attendanceRegistration != null)
            {
                attendanceRegistration.Close();
            }
            if (currentAgenda != null)
            {
                currentAgenda.Close();
            }
            if (groupSpeechTimerForm != null)
            {
                groupSpeechTimerForm.Close();
            }
            radioBtnGroupSpeechTime.Checked = false;
            radioBtnAgenda.Checked = false;
            radioBtnAttendanceRegn.Checked = false;
            radioBtnIndvResults.Checked = false;
            radioBtnGroupResults.Checked = false;
            radioBtnSpeakers.Checked = false;
            radioBtnTotalResults.Checked = false;
        }
        private void radioBtnClose_CheckedChanged(object sender, EventArgs e)
        {
            if (radioBtnClose.Checked)
            {
                CloseAllScreens();
                radioBtnClose.Checked = false;
            }
        }
      
        private void indvVotingResults_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (radioBtnIndvResults.Checked)
            {
                radioBtnIndvResults.Checked = false;
            }
        }
        public delegate void ClearPreviousResultsDelegate();
        public void ClearPreviousResults()
        {
            if (this.InvokeRequired)
            {
                ClearPreviousResultsDelegate obj = new ClearPreviousResultsDelegate(ClearPreviousResults);
                this.BeginInvoke(obj);
            }
            else
            {
                if (indvResultsDataSource != null)
                {
                    indvResultsDataSource.Clear();
                }
            }
        }
        //public delegate void AddorUpdateInterimResultsDelegate(SMDVotingData votingIndvResults);
        public void AddOrUpdateInterimResults(SMDVotingData votingIndvResults)
        {
            //if (this.InvokeRequired)
            //{
            //    AddorUpdateInterimResultsDelegate obj = new AddorUpdateInterimResultsDelegate(AddOrUpdateInterimResults);
            //    this.BeginInvoke(obj, votingIndvResults);
            //}
            //else
            //{
                try
                {
                    lock (indvResultsDataSource)
                    {
                        if (votingIndvResults != null)
                        {
                            if (votingIndvResults.IndividualResults != null && votingIndvResults.IndividualResults.Count > 0)
                            {

                                int ansId = 0;

                                IndvResults indvResult;


                                for (int iCount = 0; iCount < votingIndvResults.IndividualResults.Count; iCount++)
                                {
                                    indvResult = new IndvResults();
                                    ansId = votingIndvResults.IndividualResults[iCount].AnswerId;
                                    int ansIndex = votingIndvResults.Answers.FindIndex(delegate(AnswerContainer ansCont) { return ansCont.Id == ansId; });
                                    if (ansIndex > -1)
                                    {
                                        indvResult.Answer = votingIndvResults.Answers[ansIndex].AnswerText;
                                    }
                                    else
                                    {
                                        indvResult.Answer = "Not Voted";

                                    }
                                    
                                    indvResult.ParticipantId = votingIndvResults.IndividualResults[iCount].Participant.Id.ToString(CultureInfo.InvariantCulture);
                                    indvResult.ParticipantName = votingIndvResults.IndividualResults[iCount].Participant.ParticipantData.FirstName + " " + votingIndvResults.IndividualResults[iCount].Participant.ParticipantData.LastName;
                                    indvResult.ParticipantGroup = votingIndvResults.IndividualResults[iCount].Participant.Group.Name;

                                    if (indvResultsDataSource.Count > 0)
                                    {
                                        var modifyIndvResultList = indvResultsDataSource.Where(x => x.ParticipantId == indvResult.ParticipantId).ToList();
                                        if (modifyIndvResultList.Count == 0)
                                        {
                                            indvResultsDataSource.Add(indvResult);
                                        }
                                        else if (modifyIndvResultList.Count > 0)
                                        {
                                            foreach (var iResult in modifyIndvResultList)
                                            {
                                                indvResultsDataSource.Remove(iResult);
                                            }
                                            indvResultsDataSource.Add(indvResult);
                                        }
                                    }
                                    else if (indvResultsDataSource.Count == 0)
                                    {
                                        indvResultsDataSource.Add(indvResult);
                                    }
                                }
                            }
                        }
                        indvVotingResults.indvResultsBindingSource.DataSource = indvResultsDataSource;
                        indvVotingResults.UpdateGridViewImage();
                        indvVotingResults.UpdateTotalResults(votingIndvResults);
                    }
                }
                catch (Exception)
                {

                }
           // }
        }

        public delegate void UpdatedDataSourceForIndvResultsDelegate();
        public void UpdatedDataSourceForIndvResults()
        {
            if (this.InvokeRequired)
            {
                UpdatedDataSourceForIndvResultsDelegate obj = new UpdatedDataSourceForIndvResultsDelegate(UpdatedDataSourceForIndvResults);
                this.BeginInvoke(obj);
            }
            else
            {
                indvVotingResults.indvResultsBindingSource.DataSource = indvResultsDataSource;
            }

        }

       

    

        #endregion

        #region Fields

        ActivityLogUpdater activityLogUpdater;
        const long  INFINITE_SPEECH_TIME = 9223372036854775807;
        private int activeSessionId = -1;
        private int activeVotingId = -1;
        private BindingList<Requests> requestlist = new BindingList<Requests>();
        private BindingList<Speaker> speakerList = new BindingList<Speaker>();
        private BindingList<IndvResults> indvResultsDataSource = new BindingList<IndvResults>();
        private BindingList<Groups> groupSpeechTimeList = new BindingList<Groups>();

        const string SpeechTimeFormatString = "{0:00}:{1:00}";
        private TimeSpan RemainingTime = new TimeSpan();
        private string DisplayTime = string.Empty;
        private System.Timers.Timer smdTimer;
        private DateTime votingLimitCurrentTime;
        private DateTime groupSpeechCurrentTime;

        private GroupSpeechTimerForm groupSpeechTimerForm;

        private System.Timers.Timer groupSpeechLimitTimer;

        /// <summary>
        /// Timer for Voting Time.
        /// </summary>
        private System.Timers.Timer votingTimeLimitTimer;
        
        private Object mainLock;

        private Object microPhoneStatusLock;
        DataTable dtSpeakers = new DataTable();
        DataTable dtRequests = new DataTable();
        /// <summary>
        /// Hold the current time, which will used to determin the Time spanned.
        /// </summary>
        private DateTime microphoneStatusCurrentTime;
        /// <summary>
        /// Timer to decrease the Individual Speech time.
        /// </summary>
        System.Timers.Timer microphoneStatusTimer;
        /// Array to hold the details of the available screens.
        /// </summary>
        private Screen[] resultDisplayScreen;

        /// <summary>
        /// Context menu for copying the items in the Activity log listbox.
        /// </summary>
        private MenuItem miCopy;

        /// <summary>
        /// Form to display Attendance Registration details.
        /// </summary>
        private AttendanceRegistrationForm attendanceRegistration;

        /// <summary>
        /// Class that contains the data of a participant.
        /// </summary>
        private SMDParticipantData particiapantInfo;

        /// <summary>
        /// Form to display the microphone status.
        /// </summary>
        private MicrophoneStatusForm microPhoneStatus;

        private int interMediateResults = -1;
        /// <summary>
        /// Class that contains the data of a discussion.
        /// </summary>
        private SMDDiscussionData discussionData;
        /// <summary>
        /// Form to display the details of current Agenda.
        /// </summary>
        private AgendaForm currentAgenda;

        /// <summary>
        /// Class that contains the data of a session.
        /// </summary>
        private SMDSessionData session;
        /// <summary>
        /// 
        /// </summary>
        private SMDVotingData votingResults;
        /// <summary>
        /// Form to displays Individual Results
        /// </summary>
        private IndividualResults indvVotingResults;

        /// <summary>
        /// Form to display the Group Voting Results.
        /// </summary>
        private GroupResultsForm groupResults;

        /// <summary>
        /// Form to display the Total Voting Results.
        /// </summary>
        private TotalVotingResultsForm totalResults;

        /// <summary>
        /// The delegate used to transfer the execution of the DisplayString 
        /// onto the UI thread.
        /// </summary>
        /// <param name="log">The string that we need to display</param>
        private delegate void DisplayStringInvoker(string log);

        /// <summary>
        /// This signal is used to be able to terminate the communication 
        /// thread if necessary.
        /// </summary>
        private ManualResetEvent terminate = new ManualResetEvent(false);

        /// <summary>
        /// Indicates the a connection is (or is not) present with the server.
        /// </summary>
        private bool connectionStatus = false;

        /// <summary>
        /// Indiacates whether the voting has been stopped or not.
        /// </summary>
        private bool votingStopped = false;

        /// <summary>
        /// Indiacates whether the voting is on hold or not.
        /// </summary>
        private bool votingHold = false;
        #endregion

        private void chkBoxHideLogs_CheckStateChanged(object sender, EventArgs e)
        {
            if (chkBoxHideLogs.CheckState == CheckState.Checked)
            {
                activityLogListView.Visible = false;
                btnClearLog.Enabled = false;
                
            }
            else if (chkBoxHideLogs.CheckState == CheckState.Unchecked)
            {
                 activityLogListView.Visible = true;
                 btnClearLog.Enabled = true;
            }
        }


    }
}
