﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Xml;
using System.Xml.Serialization;

namespace DcnSwSmd
{
    /// <summary>
    /// This is a specific activity which represents the SystemStarted and the 
    /// SystemStopped activities.
    /// </summary>
    /// <remarks>
    /// This source code is provided for educational purposes only, without 
    /// warranties of any kind.
    /// </remarks>
    [XmlRootAttribute(ElementName = "SystemActivity", IsNullable = false)]
    public class SystemActivity : Activity
    {
         #region Constructor
        /// <summary>
        /// Constructor.
        /// </summary>
        public SystemActivity()
        {
            Topic = EnumTopic.System;
        }

        /// <summary>
        /// Constructor.
        /// </summary>
        /// <param name="activityType"></param>
        public SystemActivity(EnumActivityType activityType): this()
        {
            Type = activityType;
        }

        #endregion

        #region Methods

        /// <summary>
        /// Get the string that contains the serialized data for this activity 
        /// as xml.
        /// </summary>
        /// <returns></returns>
        internal override string GetXmlString()
        {
            return this.SerializeObject<SystemActivity>(this);
        }

        /// <summary>
        /// Create a string that contains some specific information regarding 
        /// the activity.
        /// </summary>
        /// <returns>The activity string</returns>
        protected override string CreateActivityString()
        {
            // No need to create a string, so let's return the empty string.
            return String.Empty;
        }
        
        #endregion
    }
}
