﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using ZedGraph;
using DcnSwSmd.TraceListeners.Data;
using System.Globalization;

namespace DcnSwSmd
{
    /// <summary>
    /// Form to display the Total Voting Results.
    /// </summary>
    public partial class TotalVotingResultsForm : Form
    {
        /// <summary>
        /// Constructor.
        /// </summary>
        /// <param name="votingIndvResults"></param>
        public TotalVotingResultsForm(SMDVotingData votingIndvResults)
        {
            InitializeComponent();
            miFullScreen.Checked = true;
            miFullScreen.Text = "Full Screen";
            miFullScreen.Click += new EventHandler(OnFullScreenClick);
            cmIndvResults.MenuItems.Add(miFullScreen);
            ResetUserInterface();
            this.ContextMenu = cmIndvResults;
        }

        private void OnFullScreenClick(object sender, EventArgs e)
        {
            if (this.miFullScreen.Checked)
            {
                this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.FixedSingle;
                this.miFullScreen.Checked = false;
            }
            else if (!this.miFullScreen.Checked)
            {
                this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.None;
                this.WindowState = FormWindowState.Maximized;
                this.miFullScreen.Checked = true;
            }
        }
       
        private delegate void UpdateVotingData(SMDVotingData votingTotalResults, bool votingStopped, int intermediateResults);

        /// <summary>
        /// Function to load the required data to display.
        /// </summary>
        /// <param name="votingTotalResults"></param>
        public void LoadVotingData(SMDVotingData votingTotalResults, bool votingStopped, int intermediateResults)
        {
            try
            {
                if (this.InvokeRequired)
                {
                    UpdateVotingData obj = new UpdateVotingData(LoadVotingData);
                    this.Invoke(obj, votingTotalResults, votingStopped, intermediateResults);
                }
                else
                {
                    if (graphTotalResults != null && graphTotalResults.GraphPane != null)
                    {
                        TimeSpan votingTimeLimit;
                        HideLabels();
                        this.headerControl.lblSubject.Text = Properties.Resources.TotalResultsTxtSubject;
                        this.headerControl.lblDate.Text = DateTime.Now.ToShortDateString().ToString(CultureInfo.InvariantCulture);
                        this.headerControl.lblTime.Text = DateTime.Now.ToShortTimeString().ToString(CultureInfo.InvariantCulture);
                        if (votingTotalResults.VotingData != null)
                        {
                            this.subjectLabel.Text = votingTotalResults.VotingData.Subject;
                            if (votingTotalResults.VotingData.RemainingVotingTime <= -1)
                            {
                                this.headerControl.lblTimeLimit.Text = Properties.Resources.VotingResultsTxtTimeLimit + ": NA";
                            }
                            else
                            {
                                votingTimeLimit = new TimeSpan(votingTotalResults.VotingData.RemainingVotingTime * TimeSpan.TicksPerSecond);
                                this.headerControl.lblTimeLimit.Text = Properties.Resources.VotingResultsTxtTimeLimit + ": " + string.Format(System.Globalization.CultureInfo.InvariantCulture, RemainingvotingTimeFormatString, Math.Abs(votingTimeLimit.Minutes), Math.Abs(votingTimeLimit.Seconds));
                            }

                        }

                        CreateTotalsGraphPane();

                        // Set fontspec for graph
                        FontSpec fontSpec = new FontSpec();
                        fontSpec.Size = 20;
                        fontSpec.FontColor = System.Drawing.SystemColors.HighlightText;
                        Border fontBorder = new Border();
                        fontBorder.IsVisible = false;
                        fontSpec.Border = fontBorder;
                        Fill fill = new Fill();
                        fill.Type = FillType.None;
                        fontSpec.Fill = fill;

                        // Set legend style / position
                        Border legendBorder = new Border();

                        graphTotalResults.GraphPane.Legend.Border = legendBorder;
                        graphTotalResults.GraphPane.Legend.Position = LegendPos.Left;
                        graphTotalResults.GraphPane.Legend.FontSpec = fontSpec;
                        graphTotalResults.GraphPane.Legend.Fill = fill;
                        graphTotalResults.GraphPane.Legend.IsVisible = false;
                        graphTotalResults.GraphPane.YAxis.Scale.FormatAuto = true;
                        graphTotalResults.GraphPane.YAxis.Scale.MajorStepAuto = false;

                        graphTotalResults.Dock = DockStyle.Fill;
                        int ansId = 0;
                        int totalCasts = 0;
                        int notVoted = 0;
                        string answerText = String.Empty;
                        Color pieSliceColor = new Color();

                        if (votingTotalResults.IndividualResults != null && intermediateResults <= -1)
                        {
                            ResetUserInterface();
                            UnhideLabels();
                        }
                        else
                        {
                            pieSliceColor = Color.White;
                            if (votingTotalResults.TotalResults != null)
                            {
                                foreach (VotingAnswerResultContainer answer in votingTotalResults.TotalResults.VotingAnswerResults)
                                {
                                    totalCasts += answer.NumberOfCasts;

                                    ansId = answer.AnswerId;
                                    int answerIndex = votingTotalResults.Answers.FindIndex(delegate(AnswerContainer ansCont) { return ansCont.Id == ansId; });
                                    if (answerIndex > -1)
                                    {
                                        answerText = votingTotalResults.Answers[answerIndex].AnswerText;
                                    }
                                    else
                                    {
                                        notVoted++;
                                        answerText = "Not Voted";
                                    }

                                    #region Parlimentary

                                    if (answerText == Properties.Resources.TotalResultAnswerYes.ToString())
                                    {
                                        if (answer.NumberOfCasts > 0)
                                        {
                                            pieSliceColor = Color.Green;
                                        }
                                        

                                        lblAns1.Text = Properties.Resources.TotalResultAnswerYes + ":  ";
                                        lblAns1.Visible = true;
                                        lblAns1Value.Visible = true;
                                        lblAns1Value.ForeColor = Color.Green;
                                        lblAns1Value.Text = answer.NumberOfCasts.ToString();
                                    }
                                    else if (answerText == Properties.Resources.TotalResultAnswerNo.ToString())
                                    {
                                        if (answer.NumberOfCasts > 0)
                                        {
                                            pieSliceColor = Color.Red;
                                        }

                                        lblAns2.Text = Properties.Resources.TotalResultAnswerNo + ":  ";
                                        lblAns2.ForeColor = Color.Red;
                                        lblAns2.Visible = true;
                                        lblAns2Value.ForeColor = Color.Red;
                                        lblAns2Value.Visible = true;
                                        lblAns2Value.Text = answer.NumberOfCasts.ToString();
                                    }
                                    else if (answerText == Properties.Resources.TotalResultAnswerAbstain.ToString())
                                    {
                                        if (answer.NumberOfCasts > 0)
                                        {
                                            pieSliceColor = Color.Yellow;
                                        }
                                        lblAns3.Text = Properties.Resources.TotalResultAnswerAbstain + ":  ";
                                        lblAns3.ForeColor = Color.Yellow;
                                        lblAns3.Visible = true;
                                        lblAns3Value.Visible = true;
                                        lblAns3Value.ForeColor = Color.Yellow;
                                        lblAns3Value.Text = answer.NumberOfCasts.ToString();
                                    }
                                    else if (answerText == Properties.Resources.TotalResultAnswerDNPV.ToString())
                                    {
                                        if (answer.NumberOfCasts > 0)
                                        {
                                            pieSliceColor = Color.Orange;
                                        }
                                        lblAns4.Text = Properties.Resources.TotalResultAnswerDNPV + ":  ";
                                        lblAns4.ForeColor = Color.Orange;
                                        lblAns4.Visible = true;
                                        lblAns4Value.ForeColor = Color.Orange;
                                        lblAns4Value.Visible = true;
                                        lblAns4Value.Text = answer.NumberOfCasts.ToString();
                                    }
                                    #endregion

                                    #region AudienceResponse

                                    else if (answerText == "--")
                                    {
                                        if (answer.NumberOfCasts > 0)
                                        {
                                            pieSliceColor = Color.Red;
                                        }
                                        lblAns1.Text = "--: ";
                                        lblAns1.ForeColor = Color.Red;
                                        lblAns1.Visible = true;
                                        
                                        lblAns1Value.ForeColor = Color.Red;
                                        lblAns1Value.Text = answer.NumberOfCasts.ToString();
                                        lblAns1Value.Visible = true;
                                    }
                                    else if (answerText == "-")
                                    {
                                        if (answer.NumberOfCasts > 0)
                                        {
                                            pieSliceColor = Color.Orange;
                                        }
                                        lblAns2.Text = "-: ";
                                        lblAns2.ForeColor = Color.Orange;
                                        lblAns2.Visible = true;

                                        lblAns2Value.ForeColor = Color.Orange;
                                        lblAns2Value.Text = answer.NumberOfCasts.ToString();
                                        lblAns2Value.Visible = true;
                                        
                                    }
                                    else if (answerText == "0")
                                    {
                                        if (answer.NumberOfCasts > 0)
                                        {
                                            pieSliceColor = Color.Yellow;
                                        }
                                        lblAns3.Text = "0: ";
                                        lblAns3.ForeColor = Color.Yellow;
                                        lblAns3.Visible = true;
                                        lblAns3Value.ForeColor = Color.Yellow;
                                        lblAns3Value.Visible = true;
                                        lblAns3Value.Text = answer.NumberOfCasts.ToString();
                                    }
                                    else if (answerText == "+")
                                    {
                                        if (answer.NumberOfCasts > 0)
                                        {
                                            pieSliceColor = Color.Green;
                                        }
                                        lblAns4.Text = "+: ";
                                        lblAns4.ForeColor = Color.Green;
                                        lblAns4.Visible = true;
                                        lblAns4Value.ForeColor = Color.Green;
                                        lblAns4Value.Visible = true;
                                        lblAns4Value.Text = answer.NumberOfCasts.ToString();
                                    }
                                    else if (answerText == "++")
                                    {
                                        if (answer.NumberOfCasts > 0)
                                        {
                                            pieSliceColor = Color.DarkGreen;
                                        }
                                        lblAns5.Text = "++: ";
                                        lblAns5.Visible = true;
                                        lblAns5.ForeColor = Color.DarkGreen;

                                        lblAns5Value.ForeColor = Color.DarkGreen;
                                        lblAns5Value.Visible = true;
                                        lblAns5Value.Text = answer.NumberOfCasts.ToString(); ;
                                    }

                                    #endregion

                                    #region OpinionPoll

                                    else if (answerText == "1")
                                    {
                                        if (answer.NumberOfCasts > 0)
                                        {
                                            pieSliceColor = Color.Red;
                                        }
                                        lblAns1.Text = "1: ";
                                        lblAns1.ForeColor = Color.Red;
                                        lblAns1.Visible = true;
                                       
                                        lblAns1Value.Visible = true;
                                        lblAns1Value.ForeColor = Color.Red;
                                        lblAns1Value.Text = answer.NumberOfCasts.ToString();
                                    }
                                    else if (answerText == "2")
                                    {
                                        if (answer.NumberOfCasts > 0)
                                        {
                                            pieSliceColor = Color.Orange;
                                        }
                                        lblAns2.Text = "2: ";
                                        lblAns2.ForeColor = Color.Orange;
                                        lblAns2.Visible = true;

                                        lblAns2Value.ForeColor = Color.Orange;
                                        lblAns2Value.Text = answer.NumberOfCasts.ToString();
                                        lblAns2Value.Visible = true;
                                    }
                                    else if (answerText == "3")
                                    {
                                        if (answer.NumberOfCasts > 0)
                                        {
                                            pieSliceColor = Color.Yellow;
                                        }
                                        lblAns3.Text = "3: ";
                                        lblAns3.ForeColor = Color.Yellow;
                                        lblAns3.Visible = true;

                                        lblAns3Value.ForeColor = Color.Yellow;
                                        lblAns3Value.Text = answer.NumberOfCasts.ToString();
                                        lblAns3Value.Visible = true;
                                    }
                                    else if (answerText == "4")
                                    {
                                        if (answer.NumberOfCasts > 0)
                                        {
                                            pieSliceColor = Color.Green;
                                        } 
                                        lblAns4.Text = "4: ";
                                        lblAns4.ForeColor =Color.Green;
                                        lblAns4.Visible = true;

                                        lblAns4Value.Text = answer.NumberOfCasts.ToString();
                                        lblAns4Value.ForeColor = Color.Green;
                                        lblAns4Value.Visible = true;
                                    }
                                    else if (answerText == "5")
                                    {
                                        if (answer.NumberOfCasts > 0)
                                        {
                                            pieSliceColor = Color.DarkGreen;
                                        }
                                        lblAns5.Text = "5: ";
                                        lblAns5.ForeColor = Color.DarkGreen;
                                        lblAns5.Visible = true;
                                        
                                        lblAns5Value.ForeColor = Color.DarkGreen;
                                        lblAns5Value.Text = answer.NumberOfCasts.ToString();
                                        lblAns5Value.Visible = true;
                                        
                                    }

                                    #endregion

                                    PieItem pie = graphTotalResults.GraphPane.AddPieSlice(answer.NumberOfCasts, pieSliceColor, 0, answerText);
                                    pie.LabelType = PieLabelType.None;
                                }

                                lblTotal.Visible = true;
                                lblTotal.Text = Properties.Resources.TotalResultsTxtTotal + ":  " + totalCasts.ToString();
                                lblNotVoted.Visible = true;
                                lblNotVoted.Text = Properties.Resources.TotalResultAnswerNotVoted + ": " + (votingTotalResults.TotalResults.MaximumMajority - totalCasts).ToString();
                                double divided = totalCasts / NumberOfYTicks;
                                int divideCount = 1;
                                while (divided > NumberOfYTicks)
                                {
                                    divided /= NumberOfYTicks;
                                    divideCount++;
                                }
                                graphTotalResults.GraphPane.YAxis.Scale.MajorStep = (Math.Ceiling(divided) * Math.Pow(NumberOfYTicks, divideCount - 1));
                                if (graphTotalResults.GraphPane.YAxis.Scale.MajorStep < 1)
                                {
                                    graphTotalResults.GraphPane.YAxis.Scale.MajorStep = 1;
                                }

                                graphTotalResults.AxisChange();
                                graphTotalResults.Invalidate();
                                ResumeLayout();
                            }
                        }
                    }
                }
            }
            catch (Exception )
            {
                
            }
        }

        public void LoadVotingDataOnVotingStart(SMDVotingData votingTotalResults, bool votingStopped, int intermediateResults)
        {
            try
            {
                if (this.InvokeRequired)
                {
                    UpdateVotingData obj = new UpdateVotingData(LoadVotingDataOnVotingStart);
                    this.Invoke(obj, votingTotalResults, votingStopped, intermediateResults);
                }
                else
                {
                    if (graphTotalResults != null && graphTotalResults.GraphPane != null)
                    {
                        TimeSpan votingTimeLimit;
                        HideLabels();
                        this.headerControl.lblSubject.Text = Properties.Resources.TotalResultsTxtSubject;
                        this.headerControl.lblDate.Text = DateTime.Now.ToShortDateString().ToString(CultureInfo.InvariantCulture);
                        this.headerControl.lblTime.Text = DateTime.Now.ToShortTimeString().ToString(CultureInfo.InvariantCulture);
                        if (votingTotalResults.VotingData != null)
                        {
                            this.subjectLabel.Text = votingTotalResults.VotingData.Subject;
                            if (votingTotalResults.VotingData.RemainingVotingTime <= -1)
                            {
                                this.headerControl.lblTimeLimit.Text = Properties.Resources.VotingResultsTxtTimeLimit + ": NA";
                            }
                            else
                            {
                                votingTimeLimit = new TimeSpan(votingTotalResults.VotingData.RemainingVotingTime * TimeSpan.TicksPerSecond);
                                this.headerControl.lblTimeLimit.Text = Properties.Resources.VotingResultsTxtTimeLimit + ": " + string.Format(System.Globalization.CultureInfo.InvariantCulture, RemainingvotingTimeFormatString, Math.Abs(votingTimeLimit.Minutes), Math.Abs(votingTimeLimit.Seconds));
                            }

                        }

                        CreateTotalsGraphPane();

                        // Set fontspec for graph
                        FontSpec fontSpec = new FontSpec();
                        fontSpec.Size = 20;
                        fontSpec.FontColor = System.Drawing.SystemColors.HighlightText;
                        Border fontBorder = new Border();
                        fontBorder.IsVisible = false;
                        fontSpec.Border = fontBorder;
                        Fill fill = new Fill();
                        fill.Type = FillType.None;
                        fontSpec.Fill = fill;

                        // Set legend style / position
                        Border legendBorder = new Border();

                        graphTotalResults.GraphPane.Legend.Border = legendBorder;
                        graphTotalResults.GraphPane.Legend.Position = LegendPos.Left;
                        graphTotalResults.GraphPane.Legend.FontSpec = fontSpec;
                        graphTotalResults.GraphPane.Legend.Fill = fill;
                        graphTotalResults.GraphPane.Legend.IsVisible = false;
                        graphTotalResults.GraphPane.YAxis.Scale.FormatAuto = true;
                        graphTotalResults.GraphPane.YAxis.Scale.MajorStepAuto = false;

                        graphTotalResults.Dock = DockStyle.Fill;
                       
                        int totalCasts = 0;
                        int ansId = 0;
                        string answerText = String.Empty;
                        Color pieSliceColor = new Color();

                        if (intermediateResults <= -1)
                        {
                            ResetUserInterface();
                            UnhideLabels();
                        }
                        else
                        {
                            pieSliceColor = Color.White;
                            if (votingTotalResults != null && votingTotalResults.Answers != null)
                            {
                                foreach (AnswerContainer answer in votingTotalResults.Answers)
                                {
                                    ansId = answer.Id;
                                    int answerIndex = votingTotalResults.Answers.FindIndex(delegate(AnswerContainer ansCont) { return ansCont.Id == ansId; });
                                    if (answerIndex > -1)
                                    {
                                        answerText = votingTotalResults.Answers[answerIndex].AnswerText;
                                    }
                                    else
                                    {
                                        answerText = "Not Voted";
                                    }
                                    #region Parlimentary

                                    if (answerText == Properties.Resources.TotalResultAnswerYes.ToString())
                                    {
                                        lblAns1.Text = Properties.Resources.TotalResultAnswerYes + ":  ";
                                        lblAns1.ForeColor = Color.Green;
                                        lblAns1.Visible = true;
                                        lblAns1Value.ForeColor = Color.Green;
                                        lblAns1Value.Text = "0";
                                        lblAns1Value.Visible = true;
                                    }
                                    else if (answerText == Properties.Resources.TotalResultAnswerNo.ToString())
                                    {
                                        lblAns2.Text = Properties.Resources.TotalResultAnswerNo + ":  ";
                                        lblAns2.ForeColor = Color.Red;
                                        lblAns2.Visible = true;
                                        lblAns2Value.ForeColor = Color.Red;
                                        lblAns2Value.Text = "0";
                                        lblAns2Value.Visible = true;
                                    }
                                    else if (answerText == Properties.Resources.TotalResultAnswerAbstain.ToString())
                                    {
                                        lblAns3.Text = Properties.Resources.TotalResultAnswerAbstain + ":  ";
                                        lblAns3.ForeColor = Color.Yellow;
                                        lblAns3.Visible = true;
                                        lblAns3Value.ForeColor = Color.Yellow;
                                        lblAns3Value.Text = "0";
                                        lblAns3Value.Visible = true;
                                    }
                                    else if (answerText == Properties.Resources.TotalResultAnswerDNPV.ToString())
                                    {
                                        
                                        lblAns4.Text = Properties.Resources.TotalResultAnswerDNPV + ":  ";
                                        lblAns4.ForeColor = Color.Orange;
                                        lblAns4.Visible = true;
                                        lblAns4Value.ForeColor = Color.Orange;
                                        lblAns4Value.Text = "0";
                                        lblAns4Value.Visible = true;
                                       
                                    }
                                    #endregion

                                    #region AudienceResponse

                                    else if (answerText == "--")
                                    {
                                        lblAns1.Text = "--: ";
                                        lblAns1.ForeColor = Color.Red;
                                        lblAns1.Visible = true;
                                      
                                        lblAns1Value.Text = "0";
                                        lblAns1Value.ForeColor = Color.Red;
                                        lblAns1Value.Visible = true;
                                    }
                                    else if (answerText == "-")
                                    {
                                        lblAns2.Text = "-: ";
                                        lblAns2.ForeColor = Color.Orange;
                                        lblAns2.Visible = true;

                                        lblAns2Value.ForeColor = Color.Orange;
                                        lblAns2Value.Text ="0";
                                        lblAns2Value.Visible = true;
                                    }
                                    else if (answerText == "0")
                                    {
                                        lblAns3.Text = "0: ";
                                        lblAns3.ForeColor = Color.Yellow;
                                        lblAns3.Visible = true;

                                        lblAns3Value.ForeColor = Color.Yellow;
                                        lblAns3Value.Text = "0";
                                        lblAns3Value.Visible = true;
                                    }
                                    else if (answerText == "+")
                                    {
                                        
                                        lblAns4.Text = "+: ";
                                        lblAns4.ForeColor = Color.Green;
                                        lblAns4.Visible = true;
                                       
                                        lblAns4Value.Text = "0";
                                        lblAns4Value.ForeColor = Color.Green;
                                        lblAns4Value.Visible = true;
                                    }
                                    else if (answerText == "++")
                                    {
                                        lblAns5.Text = "++: ";
                                        lblAns5.ForeColor = Color.DarkGreen;
                                        lblAns5.Visible = true;
                                        
                                        lblAns5Value.Text = "0";
                                        lblAns5Value.ForeColor = Color.DarkGreen;
                                        lblAns5Value.Visible = true;
                                    }

                                    #endregion

                                    #region OpinionPoll

                                    else if (answerText == "1")
                                    {
                                        
                                        lblAns1.Text = "1: ";
                                        lblAns1.ForeColor = Color.Red;
                                        lblAns1.Visible = true;

                                        lblAns1Value.Text = "0";
                                        lblAns1Value.ForeColor = Color.Red;
                                        lblAns1Value.Visible = true;
                                    }
                                    else if (answerText == "2")
                                    {
                                        lblAns2.Text = "2: ";
                                        lblAns2.ForeColor = Color.Orange;
                                        lblAns2.Visible = true;

                                        lblAns2Value.ForeColor = Color.Orange;
                                        lblAns2Value.Text = "0";
                                        lblAns2Value.Visible = true;
                                    }
                                    else if (answerText == "3")
                                    {
                                        lblAns3.Text = "3: ";
                                        lblAns3.ForeColor = Color.Yellow;
                                        lblAns3.Visible = true;

                                        lblAns3Value.ForeColor = Color.Yellow;
                                        lblAns3Value.Text = "0";
                                        lblAns3Value.Visible = true;
                                    }
                                    else if (answerText == "4")
                                    {
                                        lblAns4.Text = "4: ";
                                        lblAns4.ForeColor = Color.Green;
                                        lblAns4.Visible = true;

                                        lblAns4Value.Text = "0";
                                        lblAns4Value.ForeColor = Color.Green;
                                        lblAns4Value.Visible = true;
                                    }
                                    else if (answerText == "5")
                                    {
                                        lblAns5.Text = "5: ";
                                        lblAns5.ForeColor = Color.DarkGreen;
                                        lblAns5.Visible = true;
                                        

                                        lblAns5Value.Text = "0";
                                        lblAns5Value.ForeColor = Color.DarkGreen;
                                        lblAns5Value.Visible = true;
                                    }
                                    #endregion

                                }
                                PieItem pie = graphTotalResults.GraphPane.AddPieSlice(0, pieSliceColor, 0, answerText);
                                pie.LabelType = PieLabelType.None;
                                lblTotal.Visible = true;
                                this.lblTotal.Text = Properties.Resources.TotalResultsTxtTotal + ": 0";
                                lblNotVoted.Visible = true;
                                this.lblNotVoted.Text = Properties.Resources.TotalResultAnswerNotVoted + ": 0";
                                double divided = totalCasts / NumberOfYTicks;
                                int divideCount = 1;
                                while (divided > NumberOfYTicks)
                                {
                                    divided /= NumberOfYTicks;
                                    divideCount++;
                                }
                                graphTotalResults.GraphPane.YAxis.Scale.MajorStep = (Math.Ceiling(divided) * Math.Pow(NumberOfYTicks, divideCount - 1));
                                if (graphTotalResults.GraphPane.YAxis.Scale.MajorStep < 1)
                                {
                                    graphTotalResults.GraphPane.YAxis.Scale.MajorStep = 1;
                                }

                                graphTotalResults.AxisChange();
                                graphTotalResults.Invalidate();
                                ResumeLayout();
                            }
                        }
                    }
                }
            }
            catch (Exception)
            {
                
            }
        }
        private delegate void UpdateHeaderDataDeleagte(SMDVotingData votingTotalResults);
        public void UpdateHeaderData(SMDVotingData votingTotalResults)
        {
            if (this.InvokeRequired)
            {
                UpdateHeaderDataDeleagte obj = new UpdateHeaderDataDeleagte(UpdateHeaderData);
                this.Invoke(obj, votingTotalResults);
            }
            else
            {
                this.headerControl.lblSubject.Text = Properties.Resources.TotalResultsTxtSubject;
                this.headerControl.lblDate.Text = DateTime.Now.ToShortDateString().ToString(CultureInfo.InvariantCulture);
                this.headerControl.lblTime.Text = DateTime.Now.ToShortTimeString().ToString(CultureInfo.InvariantCulture);
                if (votingTotalResults.VotingData != null)
                {
                    this.subjectLabel.Text = votingTotalResults.VotingData.Subject;
                    if (votingTotalResults.VotingData.RemainingVotingTime <= -1)
                    {
                        this.headerControl.lblTimeLimit.Text = Properties.Resources.VotingResultsTxtTimeLimit + ": NA";
                    }
                    else
                    {
                        TimeSpan votingTimeLimit;
                        votingTimeLimit = new TimeSpan(votingTotalResults.VotingData.RemainingVotingTime * TimeSpan.TicksPerSecond);
                        this.headerControl.lblTimeLimit.Text = Properties.Resources.VotingResultsTxtTimeLimit + ": " + string.Format(System.Globalization.CultureInfo.InvariantCulture, RemainingvotingTimeFormatString, Math.Abs(votingTimeLimit.Minutes), Math.Abs(votingTimeLimit.Seconds));
                    }
                }
            }
        }

        private void HideLabels()
        {
            lblAns1Value.Visible = false;
            lblTotal.Visible = false;
            lblNotVoted.Visible = false;
            lblAns4Value.Visible = false;
            lblAns3Value.Visible = false;
            lblAns2Value.Visible = false;
            lblAns5Value.Visible = false;
            lblAns1.Visible = false;
            lblAns2.Visible = false;
            lblAns3.Visible = false;
            lblAns4.Visible = false;
            lblAns5.Visible = false;

        }
        private void UnhideLabels()
        {
            lblAns1Value.Visible = true;
            lblTotal.Visible = true;
            lblNotVoted.Visible = true;
            lblAns4Value.Visible = true;
            lblAns3Value.Visible = true;
            lblAns2Value.Visible = true;
            lblAns1.Visible = true;
            lblAns2.Visible = true;
            lblAns3.Visible = true;
            lblAns4.Visible = true;
            lblAns5.Visible = true;
        }
        private void ResetUserInterface()
        {
            this.lblNotVoted.Text = Properties.Resources.TotalResultAnswerNotVoted + ": 0";
            this.lblTotal.Text = Properties.Resources.TotalResultsTxtTotal + ": 0";
            this.lblAns1Value.Text = "0";
            this.lblAns2Value.Text = "0";
            this.lblAns3Value.Text = "0";
            this.lblAns4Value.Text = "0";
            this.lblAns5Value.Text = "0";
        }
        private void CreateTotalsGraphPane()
        {
            if (graphTotalResults != null)
            {
                GraphPane myPane = new GraphPane(graphTotalResults.Bounds, "", "", "");
                graphTotalResults.Dock = DockStyle.Fill;
                myPane.YAxis.IsVisible = false;

                // Set chart properties
                myPane.YAxis.Scale.FormatAuto = false;
                myPane.YAxis.Scale.MajorStepAuto = false;
                myPane.YAxis.MinorGrid.IsVisible = false;
                myPane.YAxis.MinorTic.Size = 0F;
                myPane.Y2Axis.IsVisible = false;

                myPane.Chart.Fill.Color = graphTotalResults.BackColor;
                myPane.Chart.Fill.Type = FillType.Solid;
                myPane.Chart.Border.IsVisible = false;

                // Set pane properties
                myPane.Fill.Color = graphTotalResults.BackColor;
                myPane.Fill.Type = FillType.Solid;
                myPane.XAxis.IsVisible = false;
                myPane.Border.IsVisible = false;
                
                // Set the new pane
                graphTotalResults.GraphPane = myPane;
            }
        }
        
        public void AddOrUpdateInterimResults(SMDVotingData votingIndvResults)
        {
            try
            {
                if (votingIndvResults != null)
                {
                    if (votingIndvResults.IndividualResults != null && votingIndvResults.IndividualResults.Count > 0)
                    {
                       
                        string participantId = String.Empty;
                        string answerId = String.Empty;

                        for (int iCount = 0; iCount < votingIndvResults.IndividualResults.Count && iCount < 10; iCount++)
                        {
                            answerId = votingIndvResults.IndividualResults[iCount].AnswerId.ToString();
                            participantId = votingIndvResults.IndividualResults[iCount].Participant.Id.ToString(CultureInfo.InvariantCulture);
                            dtInterimIndvResults.Rows.Add(participantId, answerId);
                        }
                    }
                }
                
            }
            catch (Exception)
            {

            }
        }
        public int IntermediateResults
        {
            get
            {
                return interMediateResults;
            }
            set
            {
                interMediateResults = value;
            }
        }

        public DataTable DtIntermediateIndvResults
        {
            get
            {
                return dtInterimIndvResults;
            }
            set
            {
                dtInterimIndvResults = value;
            }
        }

        public delegate void UpdateVotingRemainingTimeDelegate(SMDVotingData votingResults);

        public void UpdateRemainingVotingTime(SMDVotingData votingResults)
        {
           
            if (this.InvokeRequired)
            {
                UpdateVotingRemainingTimeDelegate obj = new UpdateVotingRemainingTimeDelegate(UpdateRemainingVotingTime);
                this.Invoke(obj, votingResults);
            }
            else
            {
                
                if (votingResults != null)
                {
                    if (votingResults.VotingData != null)
                    {

                        TimeSpan votingTimeLimit;


                        if (votingResults.VotingData.RemainingVotingTime <= -1)
                        {
                            this.headerControl.lblTimeLimit.Text = Properties.Resources.VotingResultsTxtTimeLimit + ": NA";
                        }
                        else
                        {
                            votingTimeLimit = new TimeSpan(votingResults.VotingData.RemainingVotingTime * TimeSpan.TicksPerSecond);
                            this.headerControl.lblTimeLimit.Text = Properties.Resources.VotingResultsTxtTimeLimit + ": " + string.Format(System.Globalization.CultureInfo.InvariantCulture, RemainingvotingTimeFormatString, Math.Abs(votingTimeLimit.Minutes), Math.Abs(votingTimeLimit.Seconds));
                        }
                    }
                }
            }
        }

        public delegate void UpdateHeaderDateAndTimeDelegate();
        public void UpdateHeaderDateAndTime()
        {
            if (this.InvokeRequired)
            {
                UpdateHeaderDateAndTimeDelegate obj = new UpdateHeaderDateAndTimeDelegate(UpdateHeaderDateAndTime);
                this.Invoke(obj);
            }
            else
            {
                this.headerControl.lblDate.Text = DateTime.Now.ToShortDateString().ToString(CultureInfo.InvariantCulture);
                this.headerControl.lblTime.Text = DateTime.Now.ToShortTimeString().ToString(CultureInfo.InvariantCulture);
            }
        }
        #region Fields

        public const string RemainingvotingTimeFormatString = "{0:00}:{1:00}";
        /// <summary>
        /// Number of Y tick marks to use.
        /// </summary>
        private const int NumberOfYTicks = 10;
        /// <summary>
        /// A datatable to hold the interim individual Voting results.
        /// </summary>
        DataTable dtInterimIndvResults;
        int interMediateResults = -1;
        ContextMenu cmIndvResults = new System.Windows.Forms.ContextMenu();
        MenuItem miFullScreen = new MenuItem();
        #endregion

        private void TotalVotingResultsForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            this.Dispose();
            graphTotalResults = null;
        }

        private void TotalVotingResultsForm_MaximumSizeChanged(object sender, EventArgs e)
        {

        }
    }
}
