﻿#region Copyright Robert Bosch GmbH, 2009
// Bosch Security Systems B.V.
// BU Communication Systems - Breda, The Netherlands
//
// Project             : SandStorm
// Module              : Services
// File name  $Workfile:   AnswerContainer.cs  $
//      Last Save $Date:   Oct 14 2009 17:19:30  $
//            $Revision:   1.5  $
//         Last $Author:   sr82bda  $
// Creation Date       : 22 Jul 2008
// First Author        : JP van Mackelenbergh
//
// Description         :    This contains the data regarding an answer of a 
//                          vote.
//
#endregion

using System;
using System.Collections.Generic;
using System.Text;
using System.Xml.Serialization;
using System.Reflection;

namespace DcnSwSmd.TraceListeners.Data
{
   
    [Obfuscation]
    [XmlRootAttribute(ElementName = "AnswerContainer", IsNullable = false)]
    public class AnswerContainer
    {
        #region Constructor

        public AnswerContainer()
        {
        }

        public AnswerContainer(Int32 identifier, string answerText,
            string legendText, bool correct, Int32 score)
        {
            _identifier = identifier;

            _answerText = answerText;
            _legendText = legendText;

            _correct = correct;
            _score = score;
        }

        #endregion

        #region Properties

        /// <summary>
        /// The identifier for this answer.
        /// </summary>
        [XmlAttribute]
        public Int32 Id
        {
            get
            {
                return _identifier;
            }
            set
            {
                _identifier = value;
            }
        }

        /// <summary>
        /// The text that represents this answer.
        /// </summary>
        [XmlAttribute]
        public string AnswerText
        {
            get
            {
                return _answerText;
            }
            set
            {
                _answerText = value;
            }
        }

        /// <summary>
        /// The legend text for this answer.
        /// </summary>
        [XmlAttribute]
        public string LegendText
        {
            get
            {
                return _legendText;
            }
            set
            {
                _legendText = value;
            }
        }

        /// <summary>
        /// Is this answer the correct answer.
        /// </summary>
        [XmlAttribute]
        public bool Correct
        {
            get
            {
                return _correct;
            }
            set
            {
                _correct = value;
            }
        }

        /// <summary>
        /// When multiple answers are correct then this score will indicate 
        /// which answer has the preference.
        /// </summary>
        [XmlAttribute]
        public Int32 Score
        {
            get
            {
                return _score;
            }
            set
            {
                _score = value;
            }
        }

        #endregion

        #region Methods

        #endregion

        #region Fields

        /// <summary>
        /// The identifier for this answer.
        /// </summary>
        private Int32 _identifier;

        /// <summary>
        /// The text for this answer.
        /// </summary>
        private string _answerText;

        /// <summary>
        /// The legend text for this answer.
        /// </summary>
        private string _legendText;

        /// <summary>
        /// Is this answer the correct answer.
        /// </summary>
        private bool _correct;

        /// <summary>
        /// When multiple answers are correct then this score will indicate 
        /// which answer has the preference.
        /// </summary>
        private Int32 _score;

        #endregion
    }
    
}
