#region Copyright Robert Bosch GmbH, 2012
// Bosch Security Systems B.V.
// BU Communication Systems - Breda, The Netherlands
//
// Project             : SandStorm
// Module              : Services
// File name  $Workfile:   DiscussionActivity.cs  $
//      Last Save $Date:   Feb 16 2012 17:19:30  $
//            $Revision:   1.5  $
//         Last $Author:   sr82bda  $
// Creation Date       : 22 Jul 2008
// First Author        : JP van Mackelenbergh
//
// Description         :    This contains the data regarding the discussion 
//                          activity.
//
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Reflection;
using DcnSwSmd.TraceListeners.Data;
using System.Xml;
using System.Xml.Serialization;
using System.IO;
using System.Windows.Forms;
using System.Data;
using System.Collections;
using System.Globalization;

namespace DcnSwSmd
{
    [Obfuscation]
    [XmlRootAttribute(ElementName = "DiscussionActivity", IsNullable = false)]
    public class DiscussionActivity : Activity
    {
        #region constructor

        /// <summary>
        /// Constructor.
        /// </summary>
        public DiscussionActivity()
        {
            Topic = EnumTopic.Session;
        }

        /// <summary>
        /// Constructor.
        /// </summary>
        public DiscussionActivity(EnumActivityType activityType)
        {
            Type = activityType;
        }
        #endregion

        #region Methods

        /// <summary>
        /// Get the string that contains the serialized data for this activity 
        /// as xml.
        /// </summary>
        internal override string GetXmlString()
        {
            return this.SerializeObject<DiscussionActivity>(this);
        }

        private static Byte[] StringToUTF8ByteArray(string pXmlString)
        {
            UTF8Encoding encoding = new UTF8Encoding();
            byte[] byteArray = encoding.GetBytes(pXmlString);
            return byteArray;
        }
        /// <summary>
        /// Create a string that contains some specific information regarding 
        /// the activity.
        /// </summary>
        /// <returns>The activity string</returns>
        protected override string CreateActivityString()
        {
            StringBuilder stringToReturn = new StringBuilder();
            if (this.Type == EnumActivityType.SessionStarted)
            {
                if (Discussion != null)
                {
                    stringToReturn.Append("Discussion Info" + Discussion.Id);
                }
            }


            return stringToReturn.ToString();

        }

        #endregion

        #region Properties
        /// <summary>
        /// Discussion Container object.
        /// </summary>
        public DiscussionContainer Discussion
        {
            get
            {
                return discussion;
            }
            set
            {
                discussion = value;
            }
        }

        #endregion

        #region Fileds
        DiscussionContainer discussion;
        #endregion
    }
}
