#region Copyright Robert Bosch GmbH, 2009
// Bosch Security Systems B.V.
// BU Communication Systems - Breda, The Netherlands
//
// Project             : SandStorm
// Module              : Services
// File name  $Workfile:   DiscussionContainer.cs  $
//      Last Save $Date:   Oct 14 2009 17:19:32  $
//            $Revision:   1.5  $
//         Last $Author:   sr82bda  $
// Creation Date       : 22 Jul 2008
// First Author        : JP van Mackelenbergh
//
// Description         :    This contains the data regarding a discussion.
//
#endregion

using System;
using System.Collections.Generic;
using System.Text;
using System.Xml.Serialization;
using System.Diagnostics;
using System.Reflection;

namespace DcnSwSmd.TraceListeners.Data
{
    [Obfuscation]
    [XmlRootAttribute(ElementName = "DiscussionContainer", IsNullable = false)]
    public class DiscussionContainer
    {
        #region Constructor

        public DiscussionContainer()
        {
        }

        public DiscussionContainer(Int32 identifier)
        {
            _identifier = identifier;
        }

        #endregion

        #region Properties

        /// <summary>
        /// The identifier for this discussion.
        /// </summary>
        [XmlAttribute]
        public Int32 Id
        {
            get
            {
                return _identifier;
            }
            set
            {
                _identifier = value;
            }
        }

        /// <summary>
        /// The data for this discussion.
        /// </summary>
        public DiscussionDataContainer DiscussionData
        {
            get
            {
                return _discussionData;
            }
            set
            {
                _discussionData = value;
            }
        }

        /// <summary>
        /// The list with the pending requests.
        /// </summary>
        public RequestListContainer RequestList
        {
            get
            {
                return _requestList;
            }
            set
            {
                _requestList = value;
            }
        }

        /// <summary>
        /// The list with the pending responses.
        /// </summary>
        public ResponseListContainer ResponseList
        {
            get
            {
                return _responseList;
            }
            set
            {
                _responseList = value;
            }
        }

        /// <summary>
        /// The list with the active speakers.
        /// </summary>
        public ActiveListContainer ActiveList
        {
            get
            {
                return _activeList;
            }
            set
            {
                _activeList = value;
            }
        }

        /// <summary>
        /// The list with the active response speakers.
        /// </summary>
        public ActiveResponseListContainer ActiveResponseList
        {
            get
            {
                return _activeResponseList;
            }
            set
            {
                _activeResponseList = value;
            }
        }

        /// <summary>
        /// The list with special equipment.
        /// </summary>
        public SpecialEquipmentListContainer SpecialEquipmentList
        {
            get
            {
                return _specialEquipmentList;
            }
            set
            {
                _specialEquipmentList = value;
            }
        }

        #endregion

        #region Methods

        /// <summary>
        /// Create the data for this discussion based upon the passed 
        /// variables.
        /// </summary>
        public void CreateDiscussionData(Int32 numberOfActiveMicrophones)
        {
            _discussionData = 
                new DiscussionDataContainer(numberOfActiveMicrophones);
        }

        #endregion

        #region Fields

        private Int32 _identifier;

        private DiscussionDataContainer _discussionData;

        private RequestListContainer _requestList;
        private ResponseListContainer _responseList;
        private ActiveListContainer _activeList;
        private ActiveResponseListContainer _activeResponseList;
        private SpecialEquipmentListContainer _specialEquipmentList;

        #endregion
    }
}
