﻿#region Copyright Robert Bosch GmbH, 2010
// Bosch Security Systems B.V.
// BU Communication Systems - Breda, The Netherlands
//
// Project             : SandStorm
// Module              : Services
// File name  $Workfile:   InterpretationActivity.cs  $
//      Last Save $Date:   Nov 16 2010  $
//            $Revision:   1.0  $
//         Last $Author:   msn4kor  $
// Creation Date       : 16 Nov 2010
// First Author        : Sandeepkumar M
//
// Description         : This contains the data regarding a interpretation  activity.
//
#endregion

using System;
using System.Collections.Generic;
using System.Text;
using System.Xml.Serialization;
using Bosch.Dcn.Ecpc.Operational.Diagnostics;
using System.Reflection;
using System.Globalization;

namespace DcnSwSmd.TraceListeners.Data
{
    [Obfuscation]
    [XmlRootAttribute(ElementName = "InterpretationActivity", IsNullable = false)]
    public class InterpretationActivity : Activity
    {
        #region Constructor

        public InterpretationActivity()
        {
            Topic = EnumTopic.Interpretation;
        }

        public InterpretationActivity(EnumActivityType activityType)
            : this()
        {
            this.Type = activityType;
        }

        #endregion

        #region Properties

        public DeskContainer Desk
        {
            get
            {
                return _desk;
            }
            set
            {
                _desk = value;
            }
        }

        #endregion

        #region Methods

        /// <summary>
        /// Get the string that contains the serialized data for this activity 
        /// as xml.
        /// </summary>
        internal override string GetXmlString()
        {
            return this.SerializeObject<InterpretationActivity>(this);
        }

        /// <summary>
        /// Create a string that contains some specific information regarding 
        /// the activity.
        /// </summary>
        /// <returns>The activity string</returns>
        protected override string CreateActivityString()
        {
            StringBuilder stringToReturn = new StringBuilder();

            if (_desk.Seat != null)
            {
                stringToReturn.AppendFormat(CultureInfo.InstalledUICulture,
                    "Desk Number: '{0}' Booth Number: '{1}'  SeatId: '{2}' SeatName: '{3}' MicrophoneActive: '{4}' \n SourceChannel: '{5}' SourceChannelAbbr: '{6}' SourceChannelName: '{7}' \n DestinationChannel: '{8}' DestinationChannelAbbr: '{9}' DestinationChannelName: '{10}'",
                    _desk.Number,
                    _desk.Booth.Number,
                    _desk.Seat.Id,
                    _desk.Seat.SeatData.Name,
                    _desk.Seat.SeatData.MicrophoneActive,
                    _desk.SourceChannel.Channel.Number,
                    _desk.SourceChannel.Channel.Language.Abbreviation,
                    _desk.SourceChannel.Channel.Language.Name,
                    _desk.DestinationChannel.Channel.Number,
                    _desk.DestinationChannel.Channel.Language.Abbreviation,
                    _desk.DestinationChannel.Channel.Language.Name
                    );
            }
            return stringToReturn.ToString();
        }

        #endregion

        #region Fields

        private DeskContainer _desk = new DeskContainer();

        #endregion
    }
}
