﻿#region Copyright Robert Bosch GmbH, 2012
// Bosch Security Systems B.V.
// BU Communication Systems - Breda, The Netherlands
//
// Project             : SandStorm
// Module              : Services
// File name  $Workfile:   ListContainer.cs  $
//      Last Save $Date:   Dec 08 2011 17:19:36  $
//            $Revision:   1.5  $
//         Last $Author:   sr82bda  $
// Creation Date       : 22 Jul 2008
// First Author        : JP van Mackelenbergh
//
// Description         :    This ListContainer is a generic class of which the 
//                          ActiveListContainer, ActiveResponseListContainer, 
//                          etc. are derived.
//
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Diagnostics.CodeAnalysis;
using System.Xml.Serialization;

namespace DcnSwSmd.TraceListeners.Data
{
    public class ListContainer
    {
        #region Constructor

        public ListContainer()
        {
        }

        #endregion

        #region Properties

        /// <summary>
        /// Get the number of participants present in this list.
        /// </summary>
        [XmlIgnoreAttribute]
        public Int32 Count
        {
            get
            {
                if (_participants != null)
                {
                    return _participants.Count;
                }

                return 0;
            }
        }

        /// <summary>
        /// The collection of participants which are contained in this list.
        /// </summary>

        [SuppressMessage("Microsoft.Design", "CA1002:DoNotExposeGenericLists")]
        [SuppressMessage("Microsoft.Usage", "CA2227:CollectionPropertiesShouldBeReadOnly")]
        public List<ParticipantContainer> Participants
        {
            get
            {
                return _participants;
            }
            set
            {
                _participants = value;
            }
        }

        #endregion

        #region Methods

        /// <summary>
        /// Add the passed participant to the internal collection.
        /// </summary>
        public void AddParticipant(ParticipantContainer participantContainer)
        {
            if (_participants == null)
            {
                _participants = new List<ParticipantContainer>();
            }

            _participants.Add(participantContainer);
        }

        #endregion

        #region Fields

        private List<ParticipantContainer> _participants;

        #endregion
    }
}
