#region Copyright Robert Bosch GmbH, 2009
// Bosch Security Systems B.V.
// BU Communication Systems - Breda, The Netherlands
//
// Project             : SandStorm
// Module              : Services
// File name  $Workfile:   MeetingDataContainer.cs  $
//      Last Save $Date:   Oct 14 2009 17:19:40  $
//            $Revision:   1.5  $
//         Last $Author:   sr82bda  $
// Creation Date       : 22 Jul 2008
// First Author        : JP van Mackelenbergh
//
// Description         :    This contains the data regarding a meeting.
//
#endregion

using System;
using System.Collections.Generic;
using System.Text;
using System.Xml.Serialization;
using System.Reflection;

namespace DcnSwSmd.TraceListeners.Data
{
    [Obfuscation]
    [XmlRootAttribute(ElementName = "MeetingDataContainer", IsNullable = false)]
    public class MeetingDataContainer
    {
        #region Constructor

        public MeetingDataContainer()
        {
        }

        public MeetingDataContainer(string subject, string description, DateTime dateTime)
        {
            _subject = subject;
            _description = description;
            _dateTime = dateTime;
        }

        #endregion

        #region Properties

        /// <summary>
        /// The subject for this meeting.
        /// </summary>
        [XmlAttribute]
        public string Subject
        {
            get
            {
                return _subject;
            }
            set
            {
                _subject = value;
            }
        }

        /// <summary>
        /// The date/time at which this meeting takes place.
        /// </summary>
        [XmlAttribute]
        public DateTime DateTime
        {
            get
            {
                return _dateTime;
            }
            set
            {
                _dateTime = value;
            }
        }

        /// <summary>
        /// The description for this meeting.
        /// </summary>
        [XmlAttribute]
        public string Description
        {
            get
            {
                return _description;
            }
            set
            {
                _description = value;
            }
        }

        #endregion

        #region Fields

        private string _subject;

        private DateTime _dateTime;

        private string _description;

        #endregion
    }
}
