#region Copyright Robert Bosch GmbH, 2009
// Bosch Security Systems B.V.
// BU Communication Systems - Breda, The Netherlands
//
// Project             : SandStorm
// Module              : Services
// File name  $Workfile:   ParticipantDataContainer.cs  $
//      Last Save $Date:   Oct 14 2009 17:19:44  $
//            $Revision:   1.5  $
//         Last $Author:   sr82bda  $
// Creation Date       : 22 Jul 2008
// First Author        : JP van Mackelenbergh
//
// Description         :    This contains the data regarding a participant.
//
#endregion

using System;
using System.Collections.Generic;
using System.Text;
using System.Xml.Serialization;
using System.Reflection;

namespace DcnSwSmd.TraceListeners.Data
{
    [Obfuscation]
    [XmlRootAttribute(ElementName = "ParticipantDataContainer", IsNullable = false)]
    public class ParticipantDataContainer
    {
        #region Constructor

        public ParticipantDataContainer()
        {
        }

        public ParticipantDataContainer(
            bool present,
            string firstName, string middleName, string lastName, 
            string title, string country, 
            bool microphoneAuthorisation, bool votingAuthorisation, 
            Int32 votingWeight, 
            Int32 remainingSpeechTime,
            bool speechTimerOnHold)
        {
            _present = present;

            _firstName = firstName;
            _middleName = middleName;
            _lastName = lastName;

            _title = title;
            _country = country;

            _votingWeight = votingWeight;
            _microphoneAuthorisation = microphoneAuthorisation;
            _votingAuthorisation = votingAuthorisation;

            _remainingSpeechTime = remainingSpeechTime;
            _speechTimerOnHold = speechTimerOnHold;
        }

        #endregion

        #region Properties

        /// <summary>
        /// Indicates that the current participant is present or not.
        /// </summary>
        [XmlAttribute]
        public bool Present
        {
            get
            {
                return _present;
            }
            set
            {
                _present = value;
            }
        }

        /// <summary>
        /// The voting weight for this participant.
        /// </summary>
        [XmlAttribute]
        public Int32 VotingWeight
        {
            get
            {
                return _votingWeight;
            }
            set
            {
                _votingWeight = value;
            }
        }

        /// <summary>
        /// Is this participant authorized to vote.
        /// </summary>
        [XmlAttribute]
        public bool VotingAuthorisation
        {
            get
            {
                return _votingAuthorisation;
            }
            set
            {
                _votingAuthorisation = value;
            }
        }

        /// <summary>
        /// Is this participant authorized to use the microphone.
        /// </summary>
        [XmlAttribute]
        public bool MicrophoneAuthorisation
        {
            get
            {
                return _microphoneAuthorisation;
            }
            set
            {
                _microphoneAuthorisation = value;
            }
        }

        /// <summary>
        /// The first name of this participant.
        /// </summary>
        [XmlAttribute]
        public string FirstName
        {
            get
            {
                return _firstName;
            }
            set
            {
                _firstName = value;
            }
        }

        /// <summary>
        /// The middle name of this participant.
        /// </summary>
        [XmlAttribute]
        public string MiddleName
        {
            get
            {
                return _middleName;
            }
            set
            {
                _middleName = value;
            }
        }

        /// <summary>
        /// The last name of this participant.
        /// </summary>
        [XmlAttribute]
        public string LastName
        {
            get
            {
                return _lastName;
            }
            set
            {
                _lastName = value;
            }
        }

        /// <summary>
        /// The title if this participant.
        /// </summary>
        [XmlAttribute]
        public string Title
        {
            get
            {
                return _title;
            }
            set
            {
                _title = value;
            }
        }

        /// <summary>
        /// The country set for this participant.
        /// </summary>
        [XmlAttribute]
        public string Country
        {
            get
            {
                return _country;
            }
            set
            {
                _country = value;
            }
        }

        /// <summary>
        /// The remaining speech time in seconds for this participant.
        /// </summary>
        [XmlAttribute]
        public Int32 RemainingSpeechTime
        {
            get
            {
                return _remainingSpeechTime;
            }
            set
            {
                _remainingSpeechTime = value;
            }
        }


        /// <summary>
        /// Is speech timer is on hold or not
        /// </summary>
        [XmlAttribute]
        public bool SpeechTimerOnHold
        {
            get
            {
                return _speechTimerOnHold;
            }
            set
            {
                _speechTimerOnHold = value;
            }
        }


        #endregion

        #region Methods

        #endregion

        #region Fields

        private bool _present;

        private string _firstName;

        private string _middleName;

        private string _lastName;

        private string _title;

        private string _country;

        private bool _microphoneAuthorisation;
        
        private bool _votingAuthorisation;

        private Int32 _votingWeight;

        private Int32 _remainingSpeechTime;

        private bool _speechTimerOnHold;

        #endregion
    }
}
