#region Copyright Robert Bosch GmbH, 2012
// Bosch Security Systems B.V.
// BU Communication Systems - Breda, The Netherlands
//
// Project             : SandStorm
// Module              : Services
// File name  $Workfile:   StationActivity.cs  $
//      Last Save $Date:   Dec 08 2012 17:19:52  $
//            $Revision:   1.6  $
//         Last $Author:   sr82bda  $
// Creation Date       : 22 Jul 2008
// First Author        : JP van Mackelenbergh
//
// Description         :    This contains the data regarding a station 
//                          activity.
//
#endregion

using System;
using System.Collections.Generic;
using System.Text;
using System.Xml.Serialization;
using System.Reflection;
using System.Diagnostics.CodeAnalysis;

[module: SuppressMessage("Microsoft.Globalization", "CA1305:SpecifyIFormatProvider", Scope = "member", Target = "DcnSwSmd.TraceListeners.Data.SeatActivity.#CreateActivityString()", MessageId = "System.Text.StringBuilder.AppendFormat(System.String,System.Object[])")]
namespace DcnSwSmd.TraceListeners.Data
{
    [Obfuscation]
    [XmlRootAttribute(ElementName = "SeatActivity", IsNullable = false)]
    public class SeatActivity : Activity
    {
        #region Constructor

        public SeatActivity()
        {
            Topic = EnumTopic.Seat;
        }

        public SeatActivity(EnumActivityType activityType)
            : this()
        {
            this.Type = activityType;
        }

        #endregion

        #region Properties

        public SeatContainer Seat
        {
            get
            {
                return _seat;
            }
            set
            {
                _seat = value;
            }
        }

        #endregion

        #region Methods

        /// <summary>
        /// Get the string that contains the serialized data for this activity 
        /// as xml.
        /// </summary>
        internal override string GetXmlString()
        {
            return this.SerializeObject<SeatActivity>(this);
        }

        /// <summary>
        /// Create a string that contains some specific information regarding 
        /// the activity.
        /// </summary>
        /// <returns>The activity string</returns>
        protected override string CreateActivityString()
        {
            StringBuilder stringToReturn = new StringBuilder();

            Int32 remainingSpeechTime = -1;

            if ((_seat.Participant != null) && (_seat.Participant.ParticipantData != null))
            {
                remainingSpeechTime =
                    _seat.Participant.ParticipantData.RemainingSpeechTime;
            }

            if (_seat.SeatData != null)
            {
                stringToReturn.AppendFormat(
                    " Name: '{0}' MicrophoneActive: '{1}' RemainingSpeechTime: {2} Seat type : {3}",
                    _seat.SeatData.Name,
                    _seat.SeatData.MicrophoneActive,
                    remainingSpeechTime,
                    _seat.SeatData.SeatType);
            }

            return stringToReturn.ToString();
        }

        #endregion

        #region Fields

        private SeatContainer _seat = new SeatContainer();

        #endregion
    }
}
