#region Copyright Robert Bosch GmbH, 2009
// Bosch Security Systems B.V.
// BU Communication Systems - Breda, The Netherlands
//
// Project             : SandStorm
// Module              : Services
// File name  $Workfile:   StationContainer.cs  $
//      Last Save $Date:   Oct 14 2009 17:19:54  $
//            $Revision:   1.5  $
//         Last $Author:   sr82bda  $
// Creation Date       : 22 Jul 2008
// First Author        : JP van Mackelenbergh
//
// Description         :    This contains the data regarding a station.
//
#endregion

using System;
using System.Collections.Generic;
using System.Text;
using System.Xml.Serialization;
using System.Diagnostics;
using System.Reflection;

namespace DcnSwSmd.TraceListeners.Data
{
    [Obfuscation]
    [XmlRootAttribute(ElementName = "SeatContainer", IsNullable = false)]
    public class SeatContainer
    {
        #region Constructor

        public SeatContainer()
        {
        }

        public SeatContainer(Int32 identifier)
        {
            _identifier = identifier;
        }

        #endregion

        #region Properties

        /// <summary>
        /// Uniquely identifies the seat.
        /// </summary>
        [XmlAttribute]
        public Int32 Id
        {
            get
            {
                return _identifier;
            }
            set
            {
                _identifier = value;
            }
        }

        /// <summary>
        /// The data for this station.
        /// </summary>
        public SeatDataContainer SeatData
        {
            get
            {
                return _seatData;
            }
            set
            {
                _seatData = value;
            }
        }

        /// <summary>
        /// The participant seated at this seat.
        /// </summary>
        public ParticipantContainer Participant
        {
            get
            {
                return _participant;
            }
            set
            {
                if (value == null)
                {
                    throw new ArgumentNullException("value");
                }

                // Make sure that the relation from the participant to the 
                // seat is not yet set. This will create a circular reference 
                // that will interfere when serializing the object to xml.
                Debug.Assert(value.Seat == null, 
                    "This will create a circular reference !");

                _participant = value;
            }
        }

        public bool IsReposnding
        {
            get
            {
                return _isResponding;
            }
            set
            {
                _isResponding = value;
            }
        }
        #endregion

        #region Methods

        /// <summary>
        /// Create the seat data based on the passed variables and store it 
        /// internally.
        /// </summary>
        /// <returns>The seat data container</returns>
        public SeatDataContainer CreateSeatData(string name, bool microphoneActive,string seatType)
        {
            _seatData = new SeatDataContainer(name, microphoneActive,seatType);

            return _seatData;
        }

        #endregion

        #region Fields

        private Int32 _identifier;

        private SeatDataContainer _seatData;

        private ParticipantContainer _participant;

        private bool _isResponding = false;

        #endregion
    }
}
