#region Copyright Robert Bosch GmbH, 2009
// Bosch Security Systems B.V.
// BU Communication Systems - Breda, The Netherlands
//
// Project             : SandStorm
// Module              : Services
// File name  $Workfile:   StationDataContainer.cs  $
//      Last Save $Date:   Oct 14 2009 17:19:56  $
//            $Revision:   1.5  $
//         Last $Author:   sr82bda  $
// Creation Date       : 22 Jul 2008
// First Author        : JP van Mackelenbergh
//
// Description         :    This contains the data regarding a station.
//
#endregion

using System;
using System.Collections.Generic;
using System.Text;
using System.Xml.Serialization;
using System.Diagnostics;
using System.Reflection;

namespace DcnSwSmd.TraceListeners.Data
{
    [Obfuscation]
    [XmlRootAttribute(ElementName = "SeatDataContainer", IsNullable = false)]
    public class SeatDataContainer
    {
        #region Constructor

        public SeatDataContainer()
        {
        }

        public SeatDataContainer(string name, bool microphoneActive,string seatType)
        {
            _name = name;
            _microphoneActive = microphoneActive;
            _seatType = seatType;
        }

        #endregion

        #region Properties

        /// <summary>
        /// The name for this seat.
        /// </summary>
        [XmlAttribute]
        public string Name
        {
            get
            {
                return _name;
            }
            set
            {
                _name = value;
            }
        }

        /// <summary>
        /// Is the microphone active on this seat.
        /// </summary>
        [XmlAttribute]
        public bool MicrophoneActive
        {
            get
            {
                return _microphoneActive;
            }
            set
            {
                _microphoneActive = value;
            }
        }

        /// <summary>
        /// Seat type of the station
        /// </summary>
        [XmlAttribute]
        public string SeatType
        {
            get
            {
                return _seatType;
            }
            set
            {
                _seatType = value;
            }
        }

        /// <summary>
        /// Indicates whether a station is a special station.
        /// True - Special Station.
        /// false not a special station.
        /// </summary>
        [XmlAttribute]
        public bool IsSpecialStation
        {
            get { return _isSpecialStation; }
            set { _isSpecialStation = value; }
        }

        #endregion

        #region Fields

        private string _name;
        private bool _microphoneActive;
        private string _seatType;
        private bool _isSpecialStation;

        

        #endregion
    }
}
