#region Copyright Robert Bosch GmbH, 2012
// Bosch Security Systems B.V.
// BU Communication Systems - Breda, The Netherlands
//
// Project             : SandStorm
// Module              : Services
// File name  $Workfile:   SessionActivity.cs  $
//      Last Save $Date:   Oct 14 2009 17:19:48  $
//            $Revision:   1.6  $
//         Last $Author:   sr82bda  $
// Creation Date       : 22 Jul 2008
// First Author        : JP van Mackelenbergh
//
// Description         :    This contains the data regarding a session  
//                          activity.
//
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Reflection;
using DcnSwSmd.TraceListeners.Data;
using System.Xml;
using System.Xml.Serialization;
using System.IO;
using System.Windows.Forms;
using System.Data;
using System.Collections;
using System.Globalization;

namespace DcnSwSmd.TraceListeners.Data
{
    [Obfuscation]
    [XmlRootAttribute(ElementName = "SessionActivity", IsNullable = false)]
    public class SessionActivity : Activity
    {
        #region constructor

        /// <summary>
        /// Constructor.
        /// </summary>
        public SessionActivity()
        {
            Topic = EnumTopic.Session;
        }

        /// <summary>
        /// Constructor.
        /// </summary>
        /// <param name="activityType"></param>
        public SessionActivity(EnumActivityType activityType)
        {
            Type = activityType;
        }
        #endregion

        #region Methods

        /// <summary>
        /// Get the string that contains the serialized data for this activity 
        /// as xml.
        /// </summary>
        internal override string GetXmlString()
        {
            return this.SerializeObject<SessionActivity>(this);
        }

        private static Byte[] StringToUTF8ByteArray(string pXmlString)
        {
            UTF8Encoding encoding = new UTF8Encoding();
            byte[] byteArray = encoding.GetBytes(pXmlString);
            return byteArray;
        }
        /// <summary>
        /// Create a string that contains some specific information regarding 
        /// the activity.
        /// </summary>
        /// <returns>The activity string</returns>
        protected override string CreateActivityString()
        {
            StringBuilder stringToReturn = new StringBuilder();
            if (this.Type == EnumActivityType.SessionStarted)
            {
                if (Session != null)
                {
                    stringToReturn.Append("Session Info" + Session.SessionData.Description);
                }
            }
            return stringToReturn.ToString();

        }
        #endregion

        #region Properties
        /// <summary>
        /// Object to hold the session data.
        /// </summary>
        public SessionContainer Session
        {
            get
            {
                return session;
            }
            set
            {
                session = value;
            }
        }
        #endregion


        #region Fileds
        SessionContainer session;
        #endregion
    }
}
