#region Copyright Robert Bosch GmbH, 2012
// Bosch Security Systems B.V.
// BU Communication Systems - Breda, The Netherlands
//
// Project             : SandStorm
// Module              : Services
// File name  $Workfile:   SessionContainer.cs  $
//      Last Save $Date:   Dec 08 2011 17:19:48  $
//            $Revision:   1.7  $
//         Last $Author:   sr82bda  $
// Creation Date       : 22 Jul 2008
// First Author        : JP van Mackelenbergh
//
// Description         :    This contains the data regarding a session.
//
#endregion

using System;
using System.Collections.Generic;
using System.Text;
using System.Xml.Serialization;
using System.Reflection;
using System.Diagnostics.CodeAnalysis;
using DcnSwSmd.Collections;

namespace DcnSwSmd.TraceListeners.Data
{
    [Obfuscation]
    [XmlRootAttribute(ElementName = "SessionContainer", IsNullable = false)]
    public class SessionContainer
    {
        #region Constructor

        public SessionContainer()
        {
        }

        #endregion

        #region Properties

        /// <summary>
        /// Unique identifier for this session.
        /// </summary>
        [XmlAttribute]
        public Int32 Id
        {
            get
            {
                return _identifier;
            }
            set
            {
                _identifier = value;
            }
        }

        /// <summary>
        /// The data that belongs to this session.
        /// </summary>
        public SessionDataContainer SessionData
        {
            get
            {
                return _sessionData;
            }
            set
            {
                _sessionData = value;
            }
        }

        /// <summary>
        /// The discussion that takes place within this session.
        /// </summary>
        public DiscussionContainer Discussion
        {
            get
            {
                return _discussion;
            }
            set
            {
                _discussion = value;
            }
        }

        /// <summary>
        /// The collection of votings that takes/took place within this 
        /// session.
        /// </summary>
        [SuppressMessage("Microsoft.Design", "CA1002:DoNotExposeGenericLists")]
        [SuppressMessage("Microsoft.Usage", "CA2227:CollectionPropertiesShouldBeReadOnly")]

        public List<VotingContainer> Votings
        {
            get
            {
                return _votings;
            }
            set
            {
                _votings = value;
            }
        }

        public List<GroupContainer> Groups
        {
            get
            {
                return _group;
            }
            set
            {
                _group = value;
            }
        }

        public List<ParticipantContainer> Particpants
        {
            get
            {
                return _participants;
            }
            set
            {
                _participants = value;
            }
        }

        #endregion

        #region Methods

        /// <summary>
        /// Create the data for this session based on the passed variables.
        /// </summary>
        /// <param name="subject">The subject for this session</param>
        /// <param name="description">The description for this session</param>
        /// <param name="isDone">Is this session done</param>
        public void CreateSessionData(string subject, string description, 
            bool isDone)
        {
            _sessionData = new SessionDataContainer(subject, description, isDone);
        }

        /// <summary>
        /// Create the discussion within this session based upon the passed 
        /// identifier.
        /// </summary>
        /// <param name="identifier">Identifies the discussion</param>
        /// <returns>The created discussion</returns>
        public DiscussionContainer CreateDiscussion(int identifier)
        {
            _discussion = new DiscussionContainer(Id);

            return _discussion;
        }


        public void CreateGroups(string name, long groupRemainingSpeechTime, STOPWATCH_STATE stopWatchState)
        {
            if (_group == null)
            {
                _group = new List<GroupContainer>();
            }
            GroupContainer group = new GroupContainer(name, groupRemainingSpeechTime, stopWatchState);
            _group.Add(group);
        }

        /// <summary>
        /// Create a voting based on the passed attributes and have it added 
        /// to the internal collection of votings.
        /// </summary>
        /// <returns>The added voting container</returns>
        public VotingContainer CreateAndAddVoting(int identifier, 
            string name, string subject, string votingType, 
            Int32 remainingVotingTime)
        {
            if (_votings == null)
            {
                _votings = new List<VotingContainer>();
            }

            VotingContainer voting = new VotingContainer(identifier);
            voting.CreateVotingData(name, subject, votingType, 
                remainingVotingTime);

            _votings.Add(voting);

            return voting;
        }

        #endregion

        #region Fields

        private Int32 _identifier;

        private SessionDataContainer _sessionData;

        private DiscussionContainer _discussion;

        private List<VotingContainer> _votings;

        private List<GroupContainer> _group;

        private List<ParticipantContainer> _participants;

        #endregion
    }
}
