#region Copyright Robert Bosch GmbH, 2009
// Bosch Security Systems B.V.
// BU Communication Systems - Breda, The Netherlands
//
// Project             : SandStorm
// Module              : Services
// File name  $Workfile:   VotingActivity.cs  $
//      Last Save $Date:   Oct 14 2009 17:19:58  $
//            $Revision:   1.5  $
//         Last $Author:   sr82bda  $
// Creation Date       : 22 Jul 2008
// First Author        : JP van Mackelenbergh
//
// Description         :    This contains the data regarding a voting activity.
//
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Reflection;
using DcnSwSmd.TraceListeners.Data;
using System.Xml;
using System.Xml.Serialization;
using System.IO;
using System.Windows.Forms;
using System.Data;
using System.Collections;
using System.Globalization;

namespace DcnSwSmd.TraceListeners.Data
{
    /// <summary>
    /// Class to parse the Voting Activity.
    /// </summary>
    [XmlRootAttribute(ElementName = "VotingActivity", IsNullable = false)]
    public class VotingActivity : Activity
    {
        #region Constructor

        public VotingActivity()
        {
            Topic = EnumTopic.Voting;
        }

        /// <summary>
        /// Constructor.
        /// </summary>
        /// <param name="activityType"></param>
        public VotingActivity(EnumActivityType activityType) : this()
        {
            this.Type = activityType;
        }

        #endregion

        #region Properties

        /// <summary>
        /// The voting container for which this activity is send.
        /// </summary>
        public VotingContainer Voting
        {
            get
            {
                return voting;
            }
            set
            {
                voting = value;
            }
        }

        #endregion

        #region Methods

        /// <summary>
        /// Get the string that contains the serialized data for this activity 
        /// as xml.
        /// </summary>
        /// <returns></returns>
        internal override string GetXmlString()
        {
            return this.SerializeObject<VotingActivity>(this);
        }

        /// <summary>
        /// Create a string that contains some specific information regarding 
        /// the activity.
        /// </summary>
        /// <returns>The activity string</returns>
        protected override string CreateActivityString()
        {                       
            StringBuilder stringToReturn = new StringBuilder();
           
            if (this.Type == EnumActivityType.VotingInterimResult)
            {
                if (voting.VotingResults != null)
                {
                    if (voting.VotingResults.IndividualResults.Count > 0)
                    {
                        stringToReturn.AppendFormat("Voting:- Individual Result \tParticipant's Name: {0} {1} \tParticipants Answer: {2} \t Group: {3}", voting.VotingResults.IndividualResults.Last().Participant.ParticipantData.FirstName, voting.VotingResults.IndividualResults.Last().Participant.ParticipantData.LastName, voting.VotingResults.IndividualResults.Last().AnswerId, voting.VotingResults.IndividualResults.Last().Participant.Group.Name);
                    }
                    else
                    {
                        //stringToReturn.AppendFormat("VotingResults Results is {0}", 0);
                    }
                }
            }

            if (this.Type == EnumActivityType.VotingStarted)
            {

                stringToReturn.AppendFormat("Voting Started");
                stringToReturn.AppendFormat("Voting Subject {0}", voting.VotingData.Subject);
                if (Voting.VotingData.RemainingVotingTime <= -1)
                {
                    stringToReturn.AppendFormat("Voting Time Limit: NA");
                }
                else
                {
                    stringToReturn.AppendFormat("Voting Time Limit: {0}", Voting.VotingData.RemainingVotingTime);
                }
            }

            if (this.Type == EnumActivityType.VotingStopped)
            {
                if (Voting.VotingResults != null)
                {
                    if (Voting.VotingResults.IndividualResults != null )
                    {
                        if (Voting.VotingResults.IndividualResults.Count < 1)
                        {
                            stringToReturn.AppendFormat("Voting Individual Results: Voting Individual Results not available. ");
                        }
                    }

                    if (Voting.VotingResults.VotingTotalResults != null)
                    {
                        if (Voting.VotingResults.VotingTotalResults.VotingAnswerResults.Count < 1)
                        {
                            stringToReturn.AppendFormat("Voting Total Results: Total Results not available. ");
                        }
                    }
                }
            }
            
            return stringToReturn.ToString();
        }

        #endregion

        #region Fields

        private VotingContainer voting;
        
        #endregion
    }
}
