#region Copyright Robert Bosch GmbH, 2009
// Bosch Security Systems B.V.
// BU Communication Systems - Breda, The Netherlands
//
// Project             : SandStorm
// Module              : Services
// File name  $Workfile:   VotingDataContainer.cs  $
//      Last Save $Date:   Oct 14 2009 17:20:02  $
//            $Revision:   1.6  $
//         Last $Author:   sr82bda  $
// Creation Date       : 22 Jul 2008
// First Author        : JP van Mackelenbergh
//
// Description         :    This contains the data regarding the voting within 
//                          a meeting.
//
#endregion

using System;
using System.Collections.Generic;
using System.Text;
using System.Xml.Serialization;
using System.Reflection;

namespace DcnSwSmd.TraceListeners.Data
{
    [Obfuscation]
    [XmlRootAttribute(ElementName = "VotingDataContainer", IsNullable = false)]
    public class VotingDataContainer
    {
        #region Constructor

        public VotingDataContainer()
        {
        }

        /// <summary>
        /// Create the container for VotingData.
        /// </summary>
        /// <param name="name">The name of the voting</param>
        /// <param name="subject">The subject of the voting</param>
        /// <param name="votingType">The type of the voting</param>
        public VotingDataContainer(string name, string subject,
            string votingType)
        {
            _name = name;
            _subject = subject;
            _votingType = votingType;
            _remainingVotingTime = INFINITE_VOTING_TIME;
        }

        /// <summary>
        /// Create the container for VotingData.
        /// </summary>
        /// <param name="name">The name of the voting</param>
        /// <param name="subject">The subject of the voting</param>
        /// <param name="votingType">The type of the voting</param>
        /// <param name="remainingVoteTime">The remaing voting time in 
        /// seconds. When set to -1 then it is infinite</param>
        public VotingDataContainer(string name, string subject, 
            string votingType, Int32 remainingVoteTime)
        {
            _name = name;
            _subject = subject;
            _votingType = votingType;
            _remainingVotingTime = remainingVoteTime;
        }

        #endregion

        #region Properties

        /// <summary>
        /// The name of this voting.
        /// </summary>
        [XmlAttribute]
        public string Name
        {
            get
            {
                return _name;
            }
            set
            {
                _name = value;
            }
        }

        /// <summary>
        /// The subject for this voting.
        /// </summary>
        [XmlAttribute]
        public string Subject
        {
            get
            {
                return _subject;
            }
            set
            {
                _subject = value;
            }
        }

        /// <summary>
        /// The type of this voting.
        /// </summary>
        [XmlAttribute]
        public string VotingType
        {
            get
            {
                return _votingType;
            }
            set
            {
                _votingType = value;
            }
        }

        /// <summary>
        /// The remaining time to vote in seconds.
        /// </summary>
        [XmlAttribute]
        public Int32 RemainingVotingTime
        {
            get
            {
                return _remainingVotingTime;
            }
            set
            {
                _remainingVotingTime = value;
            }
        }

        #endregion

        #region Fields

        private string _name;
        private string _subject;
        
        private string _votingType;

        private Int32 _remainingVotingTime;

        /// <summary>
        /// The voting time will be considered infinite when it is set to -1.
        /// </summary>
        private const Int32 INFINITE_VOTING_TIME = -1;

        #endregion
    }
}
