#region Copyright Robert Bosch GmbH, 2012
// Bosch Security Systems B.V.
// BU Communication Systems - Breda, The Netherlands
//
// Project             : SandStorm
// Module              : Services
// File name  $Workfile:   VotingGroupResultContainer.cs  $
//      Last Save $Date:   Dec 08 2011 17:20:02  $
//            $Revision:   1.5  $
//         Last $Author:   sr82bda  $
// Creation Date       : 22 Jul 2008
// First Author        : JP van Mackelenbergh
//
// Description         :    This contains the data regarding the results of a 
//                          group.
//
#endregion

using System;
using System.Collections.Generic;
using System.Text;
using System.Xml.Serialization;
using System.Reflection;
using System.Diagnostics.CodeAnalysis;

namespace DcnSwSmd.TraceListeners.Data
{
    [Obfuscation]
    [XmlRootAttribute(ElementName = "VotingGroupResultContainer", IsNullable = false)]
    public class VotingGroupResultContainer
    {
        #region Constructor

        public VotingGroupResultContainer()
        {
        }

        #endregion

        #region Properties

        /// <summary>
        /// The answers given withing this group.
        /// </summary>
        [SuppressMessage("Microsoft.Design", "CA1002:DoNotExposeGenericLists")]
        [SuppressMessage("Microsoft.Usage", "CA2227:CollectionPropertiesShouldBeReadOnly")]
        public List<VotingAnswerResultContainer> VotingAnswerResults
        {
            get
            {
                return _votingAnswerResults;
            }
            set
            {
                _votingAnswerResults = value;
            }
        }

        /// <summary>
        /// The group responsible for these results.
        /// </summary>
        public GroupContainer Group
        {
            get
            {
                return _group;
            }
            set
            {
                _group = value;
            }
        }

        /// <summary>
        /// The number of participants that were present within this group.
        /// </summary>
        [XmlAttribute]
        public Int32 NumberOfAuthorizedPresentParticipants
        {
            get
            {
                return _numberOfAuthorizedPresentParticipants;
            }
            set
            {
                _numberOfAuthorizedPresentParticipants = value;
            }
        }

        /// <summary>
        /// The number of participants that were present within this group and 
        /// that have not casted a vote.
        /// </summary>
        [XmlAttribute]
        public Int32 NumberOfAuthorizedPresentParticipantsWithoutVote
        {
            get
            {
                return _numberOfAuthorizedPresentParticipantsWithoutVote;
            }
            set
            {
                _numberOfAuthorizedPresentParticipantsWithoutVote = value;
            }
        }

        #endregion

        #region Methods

        #endregion

        #region Fields

        private List<VotingAnswerResultContainer> _votingAnswerResults;

        private GroupContainer _group;

        private Int32 _numberOfAuthorizedPresentParticipants;

        private Int32 _numberOfAuthorizedPresentParticipantsWithoutVote;

        #endregion
    }
}
