#region Copyright Robert Bosch GmbH, 2012
// Bosch Security Systems B.V.
// BU Communication Systems - Breda, The Netherlands
//
// Project             : SandStorm
// Module              : Services
// File name  $Workfile:   VotingResultsContainer.cs  $
//      Last Save $Date:   Dec 08 2011 17:20:06  $
//            $Revision:   1.5  $
//         Last $Author:   sr82bda  $
// Creation Date       : 22 Jul 2008
// First Author        : JP van Mackelenbergh
//
// Description         :    This contains the data regarding the results of a 
//                          voting within a meeting.
//
#endregion

using System;
using System.Collections.Generic;
using System.Text;
using System.Xml.Serialization;
using System.Reflection;
using System.Diagnostics.CodeAnalysis;

namespace DcnSwSmd.TraceListeners.Data
{
    [Obfuscation]
    [XmlRootAttribute(ElementName = "VotingResultsContainer", IsNullable = false)]
    public class VotingResultsContainer
    {
        #region Constructor

        public VotingResultsContainer()
        {
        }

        #endregion

        #region Properties

        /// <summary>
        /// The collection of individual results within this voting.
        /// </summary>
        [SuppressMessage("Microsoft.Design", "CA1002:DoNotExposeGenericLists")]
        [SuppressMessage("Microsoft.Usage", "CA2227:CollectionPropertiesShouldBeReadOnly")]
        public List<VotingIndividualResultContainer> IndividualResults
        {
            get
            {
                return _votingIndividualResults;
            }
            set
            {
                _votingIndividualResults = value;
            }
        }

        /// <summary>
        /// The total results for this voting.
        /// </summary>
        public VotingTotalResultsContainer VotingTotalResults
        {
            get
            {
                return _votingTotalResults;
            }
            set
            {
                _votingTotalResults = value;
            }
        }

        /// <summary>
        /// The results per group for this voting.
        /// </summary>
        /// 
        [SuppressMessage("Microsoft.Design", "CA1002:DoNotExposeGenericLists")]
        [SuppressMessage("Microsoft.Usage", "CA2227:CollectionPropertiesShouldBeReadOnly")]

        public List<VotingGroupResultContainer> VotingGroupResults
        {
            get
            {
                return _votingGroupResults;
            }
            set
            {
                _votingGroupResults = value;
            }
        }

        #endregion

        #region Methods

        /// <summary>
        /// Create a new container for the individual results and set the 
        /// participant from which the results originate. The given answer 
        /// is also stored.
        /// </summary>
        /// <returns>The newly created container which is already added to the 
        /// list of individual results</returns>
        public VotingIndividualResultContainer CreateAndAddIndividualResults(
            ParticipantContainer participant, Int32 answerId)
        {
            VotingIndividualResultContainer individualResultsContainer = 
                new VotingIndividualResultContainer();

            individualResultsContainer.Participant = participant;
            individualResultsContainer.AnswerId = answerId;

            return individualResultsContainer;
        }

        #endregion

        #region Fields

        private List<VotingIndividualResultContainer> _votingIndividualResults;

        private VotingTotalResultsContainer _votingTotalResults;

        private List<VotingGroupResultContainer> _votingGroupResults;

        #endregion
    }
}
