#region Copyright Robert Bosch GmbH, 2012
// Bosch Security Systems B.V.
// BU Communication Systems - Breda, The Netherlands
//
// Project             : SandStorm
// Module              : Services
// File name  $Workfile:   VotingTotalResultsContainer.cs  $
//      Last Save $Date:   Dec 08 2011 17:20:06  $
//            $Revision:   1.6  $
//         Last $Author:   sr82bda  $
// Creation Date       : 22 Jul 2008
// First Author        : JP van Mackelenbergh
//
// Description         :    This contains the data regarding the total results of 
//                          a single voting within a meeting.
//
#endregion

using System;
using System.Collections.Generic;
using System.Text;
using System.Xml.Serialization;
using System.Reflection;
using System.Diagnostics.CodeAnalysis;

namespace DcnSwSmd.TraceListeners.Data
{
    [Obfuscation]
    [XmlRootAttribute(ElementName = "VotingTotalResultsContainer", IsNullable = false)]
    public class VotingTotalResultsContainer
    {
        #region Constructor

        public VotingTotalResultsContainer()
        {
        }

        #endregion

        #region Properties

        /// <summary>
        /// Is this voting approved based on the Quorum and/or Majority.
        /// </summary>
        [XmlAttribute]
        public bool Approved
        {
            get
            {
                return _approved;
            }
            set
            {
                _approved = value;
            }
        }

        /// <summary>
        /// Required quorum for voting.
        /// </summary>
        [XmlAttribute]
        public Int32 RequiredQuorum
        {
            get
            {
                return _requiredQuorum;
            }
            set
            {
                _requiredQuorum = value;
            }
        }

        /// <summary>
        /// Actual quorum for voting.
        /// </summary>
        [XmlAttribute]
        public Int32 ActualQuorum
        {
            get
            {
                return _actualQuorum;
            }
            set
            {
                _actualQuorum = value;
            }
        }

        /// <summary>
        /// Maximum quorum for voting.
        /// </summary>
        [XmlAttribute]
        public Int32 MaximumQuorum
        {
            get
            {
                return _maximumQuorum;
            }
            set
            {
                _maximumQuorum = value;
            }
        }

        /// <summary>
        /// Required majority for voting.
        /// </summary>
        [XmlAttribute]
        public Int32 RequiredMajority
        {
            get
            {
                return _requiredMajority;
            }
            set
            {
                _requiredMajority = value;
            }
        }

        /// <summary>
        /// Actual majority for voting.
        /// </summary>
        [XmlAttribute]
        public Int32 ActualMajority
        {
            get
            {
                return _actualMajority;
            }
            set
            {
                _actualMajority = value;
            }
        }

        /// <summary>
        /// Maximum majority for voting.
        /// </summary>
        [XmlAttribute]
        public Int32 MaximumMajority
        {
            get
            {
                return _maximumMajority;
            }
            set
            {
                _maximumMajority = value;
            }
        }

        /// <summary>
        /// The number of participants that have been authorized and present.
        /// </summary>
        [XmlAttribute]
        public Int32 NumberOfAuthorizedPresentParticipants
        {
            get
            {
                return _numberOfAuthorizedPresentParticipants;
            }
            set
            {
                _numberOfAuthorizedPresentParticipants = value;
            }
        }

        /// <summary>
        /// The number of participants that have been authorized and present 
        /// but without a casted vote.
        /// </summary>
        [XmlAttribute]
        public Int32 NumberOfAuthorizedPresentParticipantsWithoutVote
        {
            get
            {
                return _numberOfAuthorizedPresentParticipantsWithoutVote;
            }
            set
            {
                _numberOfAuthorizedPresentParticipantsWithoutVote = value;
            }
        }

        /// <summary>
        /// The results for the answers that have been given for this voting.
        /// </summary>
        [SuppressMessage("Microsoft.Design", "CA1002:DoNotExposeGenericLists")]
        [SuppressMessage("Microsoft.Usage", "CA2227:CollectionPropertiesShouldBeReadOnly")]

        public List<VotingAnswerResultContainer> VotingAnswerResults
        {
            get
            {
                return _votingAnswerResults;
            }
            set
            {
                _votingAnswerResults = value;
            }
        }

        #endregion

        #region Methods

        /// <summary>
        /// Create an answer result and add it to the internal collection.
        /// </summary>
        /// <returns>The newly created answer result</returns>
        public VotingAnswerResultContainer CreateAndAddAnswerResult()
        {
            if ( _votingAnswerResults == null )
            {
                _votingAnswerResults = new List<VotingAnswerResultContainer>();
            }

            VotingAnswerResultContainer answerResultContainer = 
                new VotingAnswerResultContainer();

            _votingAnswerResults.Add(answerResultContainer);

            return answerResultContainer;
        }

        #endregion

        #region Fields

        /// <summary>
        /// Is this voting approved based on the Quorum and/or Majority.
        /// </summary>
        private bool _approved;

        /// <summary>
        /// Required quorum for voting.
        /// </summary>
        private Int32 _requiredQuorum;

        /// <summary>
        /// Actual quorum for voting.
        /// </summary>
        private Int32 _actualQuorum;

        /// <summary>
        /// Maximum quorum for voting.
        /// </summary>
        private Int32 _maximumQuorum;

        /// <summary>
        /// Required majority for voting.
        /// </summary>
        private Int32 _requiredMajority;

        /// <summary>
        /// Actual majority for voting.
        /// </summary>
        private Int32 _actualMajority;

        /// <summary>
        /// Maximum majority for voting.
        /// </summary>
        private Int32 _maximumMajority;

        /// <summary>
        /// The number of participants that have been authorized and present.
        /// </summary>
        private Int32 _numberOfAuthorizedPresentParticipants;

        /// <summary>
        /// The number of participants that have been authorized and present 
        /// but without a casted vote.
        /// </summary>
        private Int32 _numberOfAuthorizedPresentParticipantsWithoutVote;

        /// <summary>
        /// The results for the answers that have been given for this voting.
        /// </summary>
        private List<VotingAnswerResultContainer> _votingAnswerResults;

        #endregion
    }
}
