﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Xml;
using System.Xml.Serialization;
using System.IO;

namespace DcnSwSmd
{
    /// <summary>
    /// This class is used as the base class for all classes that need to 
    /// possibility to be serialized/deserialized to/from an xml string.
    /// </summary>
    /// <remarks>
    /// This source code is provided for educational purposes only, without 
    /// warranties of any kind.
    /// </remarks>
    public abstract class XmlObject
    {
        #region Methods

        /// <summary>
        /// To convert a Byte Array of Unicode values (UTF-8 encoded) to a 
        /// complete String.
        /// </summary>
        /// <param name="characters">Unicode Byte Array to be converted to 
        /// String</param>
        /// <returns>String converted from Unicode Byte Array</returns>
        private String UTF8ByteArrayToString(Byte[] characters)
        {
            UTF8Encoding encoding = new UTF8Encoding();
            String constructedString = encoding.GetString(characters);
            return (constructedString);
        }

        /// <summary>
        /// Method to convert a custom Object to XML string
        /// </summary>
        /// <param name="pObject">Object that is to be serialized to XML
        /// </param>
        /// <returns>XML string</returns>
        protected String SerializeObject<T>(T pObject)
        {
            try
            {
                String XmlizedString = null;

                MemoryStream memoryStream = new MemoryStream();
                XmlSerializer xs = new XmlSerializer(typeof(T));
                XmlTextWriter xmlTextWriter =
                    new XmlTextWriter(memoryStream, Encoding.UTF8);

                xs.Serialize(xmlTextWriter, pObject);
                memoryStream.Dispose();

                memoryStream = (MemoryStream)xmlTextWriter.BaseStream;
                XmlizedString = UTF8ByteArrayToString(memoryStream.ToArray());

                return XmlizedString;
            }
            catch (Exception)
            {
                return null;
            }
        }

        /// <summary>
        /// This needs to be implemented by the derived classes. It is used 
        /// to get a string in which all public properties are stored in XML 
        /// format.
        /// </summary>
        internal abstract string GetXmlString();

        #endregion
    }
}
