﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;

namespace DcnSwSmd
{
    /// <summary>  
    /// A class used by Tasks to report progress or completion updates back to the UI.  
    /// </summary>  
    public sealed class ActivityLogUpdater
    {
        /// <summary>  
        /// The underlying scheduler for the UI's synchronization context.  
        /// </summary>  
        private readonly TaskScheduler executer;

        /// <summary>  
        /// Initializes a new instance of the <see cref="ActivityLogUpdater"/> class. 
        /// This should be run on a UI thread.  
        /// </summary>  
        public ActivityLogUpdater()
        {
            this.executer = TaskScheduler.FromCurrentSynchronizationContext();
        }

        /// <summary>  
        /// task scheduler which executes tasks on the UI thread. 
        /// </summary>  
        public TaskScheduler Executer
        {
            get { return this.executer; }
        }


        /// <summary>
        ///  Reports the progress to the UI thread.
        /// </summary>
        /// <param name="action">The action to perform in the context of the UI thread.  </param>
        /// <returns>The task queued to the UI thread.</returns>
        public Task AsyncProgressActivity(Action action)
        {
            return Task.Factory.StartNew(action, CancellationToken.None, TaskCreationOptions.None, this.executer);
        }

        /// <summary>
        /// Reports the progress to the UI thread
        /// </summary>
        /// <param name="action"> action to perform on UI Thread</param>
        public void ReportActivity(Action action)
        {
            this.AsyncProgressActivity(action).Wait();
        }
    }
}
